@@ -5,7 +5,6 @@
  *
  * Original: Michael Sandrof
  * V2 by Carl V. Loesch (lynx@dm.unirm1.it)
- * V2.EPIC by jfn (jnelson@acronet.net)
  */
 
 #ifndef _CONFIG_H_
@@ -44,16 +43,6 @@
  */
 #define DEFAULT_SERVER "localhost irc.efnet.net irc.undernet.org irc.dal.net"
 
-/*
- * Maildir support is experimental in EPIC4.  More complete support is 
- * available in EPIC5, including support for switching back and forth 
- * between mbox and maildir at runtime.  For now, you must pick one or
- * the other at compile time.
- *	0 - Mail checking for mbox
- *	1 - Mail checking for maildir
- */
-#define MAIL_CHECKING_TYPE 0
-
 /*
  * This is an experimental feature to thwart infinite recursion.  It is not
  * very sophisticated so it's not turned on by default.  In the future I 
@@ -66,68 +55,6 @@
  */
 #define MAX_STACK_FRAMES 20000
 
-/*
- * The left and right brace characters ('{', '}') are special characters in
- * the IRC character set, becuase they represent scandanavian characters and
- * are not expected to be treated any differently than any other alphanumeric
- * character.  When this is #define'd, the client makes a best guess attempt
- * to determine if a { is being used as a brace, or as an alphanumeric char.
- *
- * The way it determines is simple:
- *	An lbrace ('{') is a normal character UNLESS ONE OF THE FOLLOWING:
- *		* It is the first character on a line
- *		* It is preceeded by whitespace
- *		* It is preceeded by a right paren (')')
- *		* It is preceeded by a right brace ('}')
- * Similarly,
- *	An rbrace ('}') is a normal character UNLESS ONE OF THE FOLLOWING:
- *		* It is the last character on a line
- *		* It is followed by whitespace
- *		* It is followed by a left paren ('(')
- *		* It is followed by a left brace ('{')
- *
- * If the following is #define'd, the above rules are in effect.  If the
- * following is #undef'd, then the normal, traditional rules apply and every
- * { and } is counted for bracing purposes.
- *
- * Some day, if this works out, this will become the default behavior -- 
- * but not today.
- */
-#undef BRACE_LOAD_HACK
-
-/*
- * A recent change was to make the %S status line expando always display
- * the server information, even if you were connected to only one server.
- * However, a few people expressly dislike this change and want the old 
- * behavior.  If this is #define'd, you get the old behavior.  If this is
- * #undef'd, you get the new behavior.
- */
-#define OLD_STATUS_S_EXPANDO_BEHAVIOR
-
-/*
- * Oooops!  This was supposed to be here, but it never did make it.  Which
- * explains a lot.  Anyhow, #define this if you want to have the option of
- * doing floating point math in EPIC.  #undef this if you always want integer
- * operations to always be done.  The client can still do integer operations 
- * if this is defined, just /set floating_point_math OFF.
- *
- * Note that if this is defined, the client actually will do all of your
- * operations in floating point and then the *result* will be truncated.
- * This is different from when this is #undef when *each term* will be 
- * truncated.  For example:
- *
- * 	#undef'd  4.2 - 2.6 	-> (4 - 2) -> 2
- *	#define'd 4.2 - 2.6     -> 1.6 -> 1
- */
-#define FLOATING_POINT_SUPPORT
-
-/*
- * Undef this if your FIONREAD is not useful.  At this time, it makes very
- * little difference because we dont use FIONREAD for anything useful.  But
- * nevertheless, if youre allergic to mild bsd-isms, then you can #undef this.
- */
-#define HAVE_USEFUL_FIONREAD
-
 /*
  * This sets how you want to see the 368 numeric to be hooked.  The default
  * (#undef) is the traditional EPIC behavior.
@@ -152,8 +79,7 @@
  * I moved this here because it seemed to be the most appropriate
  * place for it.  Define this if you want support for ``/window create''
  * and its related features.  If you dont want it, youll save some code,
- * and you wont need 'wserv', and if you do want it, you can have it in
- * all of its broken glory.
+ * and you wont need 'wserv'.
  */
 #define WINDOW_CREATE
 
@@ -230,7 +156,7 @@
  *  not be used.
  */
 #undef SOFT_SECURE
-#define VALID_UID_FILE "/home/jnelson/..."
+#define VALID_UID_FILE "/home/user/..."
 
 /*
  *  This allows you to use the third security option.  If you define this,
@@ -238,7 +164,7 @@
  *  the uids (integers, not usernames) that will not be allowed to execute
  *  the resulting program.
  */
-/*#define INVALID_UID_FILE "/home/jnelson/...."*/
+/*#define INVALID_UID_FILE "/home/user/...."*/
 
 /*
  * This part lets you deny certain hosts from running your irc client.
@@ -252,7 +178,7 @@
 #undef HOST_SECURE
 
 #ifdef HOST_SECURE
-#define INVALID_HOST_FILE "/home/jnelson/...host.deny"
+#define INVALID_HOST_FILE "/home/user/...host.deny"
 #endif
 
 
@@ -298,13 +224,6 @@
  */
 #define SERVERS_FILE "ircII.servers"
 
-
-/*
- * define this if you want your irc client to exit after an 
- * operator kill.  I have no idea why you would, though.
- */
-#undef QUIT_ON_OPERATOR_KILL
-
 /*
  * The compile sequence records the user/host/time of the compile,
  * which can be useful for tampering and newbie reasons.  If you want
@@ -325,6 +244,7 @@
  */
 #define DEFAULT_IRCPATH "~/.epic:~/.irc:%s/script:."
 
+#define IMPLIED_ON_HOOKS
 
 /*
  * Below are the IRCII variable defaults.  For boolean variables, use 1 for
@@ -336,20 +256,10 @@
 #define DEFAULT_ALLOW_C1_CHARS 0
 #define DEFAULT_ALT_CHARSET 1
 #define DEFAULT_ALWAYS_SPLIT_BIGGEST 1
-#define DEFAULT_AUTO_NEW_NICK 1
-#define DEFAULT_AUTO_RECONNECT 1
-#define DEFAULT_AUTO_RECONNECT_DELAY 0
-#define DEFAULT_AUTO_REJOIN 0
-#define DEFAULT_AUTO_REJOIN_CONNECT 1
-#define DEFAULT_AUTO_REJOIN_DELAY 0
-#define DEFAULT_AUTO_UNMARK_AWAY 0
-#define DEFAULT_AUTO_WHOWAS 1
-#define DEFAULT_BAD_STYLE 0
 #define DEFAULT_BANNER "***"
+#define DEFAULT_BANNER_EXPAND 0
 #define DEFAULT_BEEP 1
 #define DEFAULT_BEEP_MAX 3
-#define DEFAULT_BEEP_ON_MSG "NONE"
-#define DEFAULT_BEEP_WHEN_AWAY 1
 #define DEFAULT_BLINK_VIDEO 1
 #define	DEFAULT_BOLD_VIDEO 1
 #define DEFAULT_CHANNEL_NAME_WIDTH 0
@@ -359,119 +269,110 @@
 #define DEFAULT_CLOCK_INTERVAL 60
 #define DEFAULT_CMDCHARS "/"
 #define DEFAULT_COLOR 1
-#define DEFAULT_COMMAND_MODE 0
 #define DEFAULT_COMMENT_HACK 1
-#define DEFAULT_CONNECT_TIMEOUT 30
 #define DEFAULT_CONTINUED_LINE "+"
 #define DEFAULT_CPU_SAVER_AFTER 0
 #define DEFAULT_CPU_SAVER_EVERY 60
 #define DEFAULT_CURRENT_WINDOW_LEVEL NULL
 #define DEFAULT_DCC_AUTO_SEND_REJECTS 1
+#define DEFAULT_DCC_CONNECT_TIMEOUT 30
 #define DEFAULT_DCC_DEQUOTE_FILENAMES 1
 #define DEFAULT_DCC_LONG_PATHNAMES 1
 #define DEFAULT_DCC_SLIDING_WINDOW 1
-#define DEFAULT_DCC_TIMEOUT 3600
+#define DEFAULT_DCC_STORE_PATH NULL
+#define DEFAULT_DCC_USE_GATEWAY_ADDR 0
+#define DEFAULT_DEFAULT_SCRIPT_ENCODING "ISO-8859-1"
 #define DEFAULT_DISPATCH_UNKNOWN_COMMANDS 0
 #define DEFAULT_DISPLAY 1
 #define DEFAULT_DISPLAY_ANSI 1
 #define DEFAULT_DISPLAY_PC_CHARACTERS 4
 #define DEFAULT_DO_NOTIFY_IMMEDIATELY 1
 #define DEFAULT_EIGHT_BIT_CHARACTERS 1
-#define DEFAULT_EXEC_PROTECTION 0
 #define DEFAULT_FLOATING_POINT_MATH 0
 #define DEFAULT_FLOATING_POINT_PRECISION 16
 #define DEFAULT_FLOOD_AFTER 3
 #define DEFAULT_FLOOD_IGNORE 0
 #define DEFAULT_FLOOD_MASKUSER 0
-#define DEFAULT_FLOOD_RATE 3
-#define DEFAULT_FLOOD_RATE_PER 1
+#define DEFAULT_FLOOD_RATE 1
+#define DEFAULT_FLOOD_RATE_PER 10
 #define DEFAULT_FLOOD_USERS 3
 #define DEFAULT_FLOOD_WARNING 0
-#define DEFAULT_FULL_STATUS_LINE 1
-#define DEFAULT_HELP_PAGER 1
-#define DEFAULT_HELP_PROMPT 1
-#define DEFAULT_HELP_WINDOW 0
 #define DEFAULT_HIDE_PRIVATE_CHANNELS 0
-#define DEFAULT_HIGH_BIT_ESCAPE 2
 #define DEFAULT_HIGHLIGHT_CHAR "BOLD"
-#define DEFAULT_HISTORY 150
-#define DEFAULT_HISTORY_CIRCLEQ 1
-#define DEFAULT_HOLD_INTERVAL 10
-#define DEFAULT_HOLD_MODE 0
+#define DEFAULT_HIGH_BIT_ESCAPE 2
 #define DEFAULT_HOLD_SLIDER 100
 #define DEFAULT_INDENT 0
-#define DEFAULT_INPUT_ALIASES 0
+#define DEFAULT_INPUT_INDICATOR_LEFT "+ "
+#define DEFAULT_INPUT_INDICATOR_RIGHT " +"
 #define DEFAULT_INPUT_PROMPT "> "
-#define DEFAULT_INPUT_PROTECTION 1
 #define DEFAULT_INSERT_MODE 1
 #define DEFAULT_INVERSE_VIDEO 1
 #define DEFAULT_KEY_INTERVAL 1000
 #define DEFAULT_LASTLOG 256
 #define DEFAULT_LASTLOG_LEVEL "ALL"
+#define DEFAULT_LASTLOG_REWRITE NULL
 #define DEFAULT_LOG 0
 #define DEFAULT_LOGFILE "irc.log"
 #define DEFAULT_MAIL 2
 #define DEFAULT_MAIL_INTERVAL 60
-/* #define DEFAULT_MAIL_TYPE "mbox" */
-#define DEFAULT_MAX_RECONNECTS 4
+#define DEFAULT_MAIL_TYPE "mbox"
 #define DEFAULT_METRIC_TIME 0
+#define DEFAULT_MIRC_BROKEN_DCC_RESUME 0
 #define DEFAULT_MODE_STRIPPER 0
 #define DEFAULT_ND_SPACE_MAX 160
 #define DEFAULT_NEW_SERVER_LASTLOG_LEVEL "ALL,-DCC"
-#define DEFAULT_NO_CTCP_FLOOD 1
-#define DEFAULT_NO_FAIL_DISCONNECT 0
 #define DEFAULT_NOTIFY 1
 #define DEFAULT_NOTIFY_INTERVAL 60
 #define DEFAULT_NOTIFY_LEVEL "ALL"
 #define DEFAULT_NOTIFY_ON_TERMINATION 1
 #define DEFAULT_NOTIFY_USERHOST_AUTOMATIC 1
 #define DEFAULT_NO_CONTROL_LOG 0
-#define DEFAULT_NUM_OF_WHOWAS 1
+#define DEFAULT_NO_CTCP_FLOOD 1
+#define DEFAULT_NO_FAIL_DISCONNECT 0
+#define DEFAULT_OLD_MATH_PARSER 0
+#define DEFAULT_OLD_SERVER_LASTLOG_LEVEL "NONE"
 #define DEFAULT_PAD_CHAR ' '
 #define DEFAULT_QUIT_MESSAGE "ircII %s -- Are we there yet?"
-#define DEFAULT_RANDOM_LOCAL_PORTS 0
 #define DEFAULT_RANDOM_SOURCE 0
-#define DEFAULT_REVERSE_STATUS_LINE 1
+#define DEFAULT_SCREEN_OPTIONS NULL
 #define DEFAULT_SCROLLBACK 256
 #define DEFAULT_SCROLLBACK_RATIO 50
 #define DEFAULT_SCROLL_LINES 1
-#define DEFAULT_SECURITY 0
 #define DEFAULT_SHELL "/bin/sh"
 #define DEFAULT_SHELL_FLAGS "-c"
 #define DEFAULT_SHELL_LIMIT 0
 #define DEFAULT_SHOW_CHANNEL_NAMES 1
-#define DEFAULT_SHOW_END_OF_MSGS 1
 #define DEFAULT_SHOW_NUMERICS 0
 #define	DEFAULT_SHOW_STATUS_ALL 0
-#define DEFAULT_SHOW_WHO_HOPCOUNT 1
 #define DEFAULT_STATUS_AWAY " (Away)"
 #define DEFAULT_STATUS_CHANNEL " %C"
 #define DEFAULT_STATUS_CHANOP "@"
-#define DEFAULT_STATUS_HALFOP "%"
-#define DEFAULT_STATUS_SSL_ON "*SSL*"
-#define DEFAULT_STATUS_SSL_OFF "*RAW*"
 #define DEFAULT_STATUS_CLOCK " %T"
 #define DEFAULT_STATUS_CPU_SAVER " (%L)"
-#define DEFAULT_STATUS_FORMAT "%T [%R] %*%=%@%N%#%S%H%B%Q%A%C%+%I%O%M%F%L %D %U %W"
-#define DEFAULT_STATUS_FORMAT1 "%T [%R] %*%=%@%N%#%S%H%B%Q%A%C%+%I%O%M%F%L %U "
-#define DEFAULT_STATUS_FORMAT2 "%W %X %Y %Z "
+#define DEFAULT_STATUS_FORMAT "%T [%R] %*%=%@%N%#%S%{1}H%H%B%Q%A%C%+%I%O%M%F%L %D %U %W"
+#define DEFAULT_STATUS_FORMAT1 "%T [%R] %*%=%@%N%#%S%{1}H%H%B%Q%A%C%+%I%O%M%F%L %U "
+#define DEFAULT_STATUS_FORMAT2 "%W %X %Y %Z "
+#define DEFAULT_STATUS_HALFOP "%"
 #define DEFAULT_STATUS_HOLD " Held: "
 #define DEFAULT_STATUS_HOLD_LINES "%B"
+#define DEFAULT_STATUS_HOLDMODE " (Hold) "
 #define DEFAULT_STATUS_INSERT ""
-#define DEFAULT_STATUS_MODE " (+%+)"
 #define DEFAULT_STATUS_MAIL " (Mail: %M)"
-#define DEFAULT_STATUS_NICK "%N"
-#define DEFAULT_STATUS_NO_REPEAT 0
+#define DEFAULT_STATUS_MODE " (+%+)"
+#define DEFAULT_STATUS_NICKNAME "%N"
 #define	DEFAULT_STATUS_NOSWAP "(NOSWAP)"
 #define	DEFAULT_STATUS_NOTIFY " (W: %F)"
+#define DEFAULT_STATUS_NO_REPEAT 0
 #define DEFAULT_STATUS_OPER "*"
 #define DEFAULT_STATUS_OVERWRITE " (Overwrite)"
 #define DEFAULT_STATUS_QUERY " (Query: %Q)"
 #define DEFAULT_STATUS_SCROLLBACK " (Scroll)"
 #define DEFAULT_STATUS_SERVER " (%S)"
+#define DEFAULT_STATUS_SSL_OFF "*RAW*"
+#define DEFAULT_STATUS_SSL_ON "*SSL*"
 #define DEFAULT_STATUS_TRUNCATE_RHS 1
 #define DEFAULT_STATUS_UMODE " (+%#)"
-#define DEFAULT_STATUS_USER "ircII-EPIC4 -- Type /help for help"
+#define DEFAULT_STATUS_USER "EPIC5 -- Visit http://help.epicsol.org/ for help"
 #define DEFAULT_STATUS_USER1 ""
 #define DEFAULT_STATUS_USER2 ""
 #define DEFAULT_STATUS_USER3 ""
@@ -514,15 +415,12 @@
 #define DEFAULT_STATUS_VOICE "+"
 #define DEFAULT_STATUS_WINDOW "^^^^^^^^"
 #define DEFAULT_SUPPRESS_FROM_REMOTE_SERVER 0
-#define DEFAULT_SUPPRESS_SERVER_MOTD 0
-#define DEFAULT_SWITCH_CHANNEL_ON_PART 1
 #define DEFAULT_SWITCH_CHANNELS_BETWEEN_WINDOWS 1
 #define DEFAULT_TAB 1
 #define	DEFAULT_TAB_MAX 0
 #define DEFAULT_TERM_DOES_BRIGHT_BLINK 0
 #define DEFAULT_UNDERLINE_VIDEO 1
-#define DEFAULT_USERINFO "EPIC4 -- Get your laundry brighter, not just whiter!"
-#define DEFAULT_VERBOSE_CTCP 1
+#define DEFAULT_USER_INFORMATION "EPIC5 -- Into the abyss"
 #define DEFAULT_WORD_BREAK ".,; \t"
 #define DEFAULT_WSERV_TYPE "screen"
 #define DEFAULT_XTERM "xterm"
@@ -547,13 +445,10 @@
  * you do =)  Dont change any of these unless you know what it will do.
  */
 #undef EMACS_KEYBINDINGS	/* meta-key keybindings. */
-#undef EPIC_DEBUG		/* force coredump on panic */
-#define EXEC_COMMAND		/* allow /exec comamnd */
 #undef HACKED_DCC_WARNING	/* warn if handshake != sender */
 #undef HARD_UNFLASH		/* do a hard reset instead of soft on refresh */
 #undef NO_BOTS			/* no bots allowed */
 #undef NO_CHEATING		/* always do it the "right" way, no shortcuts */
-#undef STRIP_EXTRANEOUS_SPACES	/* strip leading and trailing spaces */
 
 #undef	I_DONT_TRUST_MY_USERS	/* There are certain things that the stock
 				   ircII client doesnt allow users to do
