@@ -9,8 +9,11 @@
 #ifndef __hook_h__
 #define __hook_h__
 
+#define FIRST_NAMED_HOOK 1000
+
 enum HOOK_TYPES {
-	ACTION_LIST,
+	ACTION_LIST = FIRST_NAMED_HOOK,
+	CHANNEL_LOST_LIST,
 	CHANNEL_NICK_LIST,
 	CHANNEL_SIGNOFF_LIST,
 	CHANNEL_SYNC_LIST,
@@ -44,9 +47,9 @@ enum HOOK_TYPES {
 	INPUT_LIST,
 	INVITE_LIST,
 	JOIN_LIST,
+	KEYBINDING_LIST,
 	KICK_LIST,
 	KILL_LIST,
-	LEAVE_LIST,
 	LIST_LIST,
 	MAIL_LIST,
 	MODE_LIST,
@@ -54,13 +57,17 @@ enum HOOK_TYPES {
 	MSG_LIST,
 	MSG_GROUP_LIST,
 	NAMES_LIST,
+	NEW_NICKNAME_LIST,
 	NICKNAME_LIST,
 	NOTE_LIST,
 	NOTICE_LIST,
 	NOTIFY_SIGNOFF_LIST,
 	NOTIFY_SIGNON_LIST,
+	NUMERIC_LIST,
 	ODD_SERVER_STUFF_LIST,
+	OPERWALL_LIST,
 	OPER_NOTICE_LIST,
+	PART_LIST,
 	PONG_LIST,
 	PUBLIC_LIST,
 	PUBLIC_MSG_LIST,
@@ -78,25 +85,31 @@ enum HOOK_TYPES {
 	SERVER_ESTABLISHED_LIST,
 	SERVER_LOST_LIST,
 	SERVER_NOTICE_LIST,
+	SERVER_STATUS_LIST,
 	SET_LIST,
+	SIGNAL_LIST,
 	SIGNOFF_LIST,
 	SILENCE_LIST,
 	SSL_SERVER_CERT_LIST,
 	STATUS_UPDATE_LIST,
 	SWITCH_CHANNELS_LIST,
+	SWITCH_QUERY_LIST,
 	SWITCH_WINDOWS_LIST,
 	TIMER_LIST,
 	TOPIC_LIST,
+	UNKNOWN_COMMAND_LIST,
+	UNKNOWN_SET_LIST,
 	UNLOAD_LIST,
 	WALL_LIST,
 	WALLOP_LIST,
 	WHO_LIST,
-	WIDELIST_LIST,
 	WINDOW_LIST,
 	WINDOW_COMMAND_LIST,
 	WINDOW_CREATE_LIST,
 	WINDOW_BEFOREKILL_LIST,
 	WINDOW_KILL_LIST,
+	WINDOW_NOTIFIED_LIST,
+	WINDOW_SERVER_LIST,
 	YELL_LIST,
 	ZZZZ_THIS_ALWAYS_COMES_LAST_ZZZZ
 };
@@ -106,11 +119,16 @@ enum HOOK_TYPES {
 
 	BUILT_IN_COMMAND(oncmd);
 	BUILT_IN_COMMAND(shookcmd);
+
 	int	do_hook 		(int, const char *, ...) __A(2);
+	int	do_hook_with_result	(int, char **, const char *, ...) __A(3);
+	char *	hookctl			(char *);
 	void	flush_on_hooks 		(void);
 	void	unload_on_hooks		(char *);
 	void	save_hooks 		(FILE *, int);
 	void	do_stack_on		(int, char *);
 	int	hook_find_free_serial	(int, int, int);
 
+	extern int deny_all_hooks;
+
 #endif /* __hook_h_ */
