@@ -53,8 +53,6 @@
 #define BLINK_TOG_STR	"\006"
 #define ALL_OFF		'\017'		/* ^O */
 #define ALL_OFF_STR	"\017"
-#define ROM_CHAR	'\022'		/* ^R */
-#define ROM_CHAR_STR	"\022"
 #define ND_SPACE	'\023'		/* ^S */
 #define ND_SPACE_STR	"\023"
 #define ALT_TOG		'\005'		/* ^E (think Extended) */
@@ -81,7 +79,6 @@
 #define PATH_LEN 1024
 
 /* irc.c's global variables */
-extern		int	away_set;
 extern		int	background;
 extern		int	current_numeric;
 extern		int	dead;
@@ -91,31 +88,23 @@ extern		int	foreground;
 extern		int	global_beep_ok;
 extern		int	inhibit_logging;
 extern		int	irc_port;
-extern		int	ircrc_loaded;
-extern		int	loading_global;
 extern		int	oper_command;
 extern		int	privileged_output;
 extern		int	quick_startup;
-extern		int	trusted_output;
 extern		int	use_flow_control;
 extern		int	use_iexten;
-extern		int	who_mask;
 extern		int	use_input;
 extern		int	waiting_out;
 extern		int	waiting_in;
-extern		char *	args_str;
-extern		char *	cannot_open;
 extern const	char *	compile_info;
-extern		char *	cut_buffer;
+extern		unsigned char *	cut_buffer;
 extern		char *	default_channel;
-extern		char	empty_string[];
-extern		char	space[];
-extern		char	star[];
-extern		char	dot[];
+extern	const	char 	empty_string[];
+extern	const	char	space[];
+extern	const	char	star[];
+extern	const	char	dot[];
 extern		char	hostname[NAME_LEN + 1];
 extern const 	char 	internal_version[];
-extern		char *	ircrc_file;
-extern		char *	epicrc_file;
 extern		char *	startup_file;
 extern		char *	irc_lib;
 extern const 	char 	irc_version[];
@@ -123,9 +112,9 @@ extern		char *	last_notify_nick;
 extern		char *	LocalHostName;
 extern		char *	my_path;
 extern		char	nickname[NICKNAME_LEN + 1];
-extern		char	off[];
-extern		char	on[];
-extern		char	one[];
+/* extern	const	char	my_off[]; */
+extern	const	char	on[];
+extern	const	char	one[];
 extern		char	realname[REALNAME_LEN + 1];
 extern	const	char	ridiculous_version_name[];
 extern	const unsigned long commit_id;
@@ -134,34 +123,28 @@ extern	const	char *	unknown_userhost;
 extern	const	char 	useful_info[];
 extern		char	username[NAME_LEN + 1];
 extern		char	userhost[NAME_LEN + 1];
-extern		char	zero[];
-extern		char	comma[];
+extern	const	char	zero[];
+extern	const	char	comma[];
 extern		char *	highlight_char;
+extern		int	do_window_notifies;
 
-extern 		ISA *	LocalIPv4Addr;
+extern 		char *	LocalIPv4HostName;
 #ifdef INET6
-extern		ISA6 *	LocalIPv6Addr;
+extern 		char *	LocalIPv6HostName;
 #endif
 extern		fd_set  readables, held_readables;
 extern		fd_set  writables, held_writables;
-extern		int	global_max_fd;
 extern struct timeval 	start_time;
 extern struct timeval	idle_time;
 extern struct timeval	now;
 extern struct timeval	input_timeout;
+extern	unsigned	window_display;		/* This is in window.h */
 
 /* irc.c's extern functions */
-	char	get_a_char 		(void);
-	void	get_line_return 	(char, char *);
-	void	get_line 		(char *, int, void (*) (char, char *));
 	void	io 			(const char *);
 	void	irc_exit 		(int, const char *, ...) /*__A(2)*/ __N;
-	void	irc_quit 		(char, char *);
+	BUILT_IN_KEYBINDING(irc_quit);
+
+        void    load_ircrc              (void);
 
-#include "reg.h"
-	void	dump_load_stack		(int);	 /* XXX command.c */
-const	char *	current_filename	(void);  /* XXX command.c */
-const	char *	current_loader		(void);	 /* XXX command.c */
-	int	current_line		(void);	 /* XXX command.c */
-	char *	current_package		(void);  /* XXX command.c */
 #endif /* __irc_h */
