@@ -2,7 +2,7 @@
  * server.h: header for server.c 
  *
  * Copyright 1990 Michael Sandrof
- * Copyright 1997 EPIC Software Labs
+ * Copyright 1997, 2007 EPIC Software Labs
  * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
  */
 
@@ -18,9 +18,6 @@
 #include "notify.h"
 #include "alist.h"
 
-/* XXXX Ick.  Gross.  Bad. XXX */
-struct notify_stru;
-
 typedef array A005;
 typedef struct
 {
@@ -35,23 +32,42 @@ typedef struct WaitCmdstru
         struct  WaitCmdstru *next;
 } WaitCmd;
 
+typedef struct ServerInfo 
+{
+	int	clean;
+        char *  freestr;
+	char *	fulldesc;
+        int     refnum;
+        const char *  host;
+        int     port;
+        const char *  password;
+        const char *  nick;
+        const char *  group;
+        const char *  server_type;
+        const char *  proto_type;
+	const char *  vhost;
+} ServerInfo;
+
 /* Server: a structure for the server_list */
 typedef	struct
 {
-	char	*name;			/* the name of the server */
+	ServerInfo *info;		/* Canonical information */
+
+	AI 	*addrs;			/* Returned by getaddrinfo */
+const	AI	*next_addr;		/* The next one to try upon failure */
+	int	addr_counter;		/* How far we're into "addrs" */
+	ssize_t	addr_len;
+	ssize_t	addr_offset;
+
 	char	*itsname;		/* the server's idea of its name */
-	char	*password;		/* password for that server */
-	int	port;			/* port number on that server */
-	char	*group;			/* Server group it belongs to */
+	Bucket	*altnames;		/* Alternate handles for the server */
 	char	*nickname;		/* Authoritative nickname for us */
 	char	*s_nickname;		/* last NICK command sent */
 	char	*d_nickname;		/* Default nickname to use */
-	size_t	fudge_factor;		/* How much s_nickname's fudged */
-	int	nickname_pending;	/* Is a NICK command pending? */
-	int	resetting_nickname;	/* Is a nickname reset in progress? */
-	int	registration_pending;	/* Is a registration going on ? */
-	int	registered;		/* true if registration is assured */
-	int	rejoined_channels;	/* Has we tried to auto-rejoin? */
+	char	*unique_id;		/* Unique ID (for ircnet) */
+
+	int	status;			/* See above */
+
 	char	*userhost;		/* my userhost on this server */
 	char	*away;			/* away message for this server */
 	int	operator;		/* true if operator */
@@ -59,48 +75,45 @@ typedef	struct
 					 * defined above */
 	int	server2_8;		/* defined if we get an 001 numeric */
 	char	*version_string;	/* what is says */
-	long	flags;			/* Various flags */
-	long	flags2;			/* More Various flags */
-	char	*umodes;		/* Possible user modes */
 	char	umode[54];		/* Currently set user modes */
-	int	s_takes_arg;		/* Set to 1 if s user mode has arg */
 	int	des;			/* file descriptor to server */
-	int	eof;			/* eof flag for server */
 	int	sent;			/* set if something has been sent,
 					 * used for redirect */
 	char	*redirect;		/* Who we're redirecting to here */
+	int	who_max;		/* Max pending whos */
 	WhoEntry *	who_queue;	/* Who queue */
-	IsonEntry *	ison_wait;	/* Ison wait queue */
+	int	ison_len;		/* Max ison characters */
+	int	ison_max;		/* Max pending isons */
 	IsonEntry *	ison_queue;	/* Ison queue */
+	IsonEntry *	ison_wait;	/* Ison wait queue */
+	int	userhost_max;		/* Max pending userhosts */
 	UserhostEntry *	userhost_queue;	/* Userhost queue */
+	UserhostEntry *	userhost_wait;	/* Userhost wait queue */
 
 	SS	local_sockname; 	/* sockname of this connection */
 	SS	remote_sockname; 	/* sockname of this connection */
 	SS	uh_addr;		/* ip address the server sees */
 	NotifyList	notify_list;	/* Notify list for this server */
-	int	reconnects;		/* Number of reconnects done */
 	char 	*cookie;		/* Erf/TS4 "cookie" value */
-	int	save_channels;		/* True if abnormal connection */
 	int	line_length;		/* How long a protocol command may be */
 	int	max_cached_chan_size;	/* Bigger channels won't cache U@H */
 	int	closing;		/* True if close_server called */
-	int	reconnect_to;		/* Server to connect to on EOF */
 	char	*quit_message;		/* Where we stash a quit message */
 	A005	a005;			/* 005 settings kept kere. */
+	int	stricmp_table;		/* Which case insensitive map to use */
+	int	autoclose;		/* Whether the server is closed when
+					   there are no windows on it */
 
 	int	funny_min;		/* Funny stuff */
 	int	funny_max;
 	int	funny_flags;
 	char *	funny_match;
 
-#ifdef HAVE_SSL
-	SSL_CTX*	ctx;
-	SSL_METHOD*	meth;
-#endif
-	void *	ssl_fd;
-	int	try_ssl;		/* SSL requested on next connection. */
 	int	ssl_enabled;		/* Current SSL status. */
 
+	char *	realname;		/* The actual realname. */
+	char *	default_realname;	/* The default realname. */
+
         int             doing_privmsg;
         int             doing_notice;
         int             doing_ctcp;
@@ -117,7 +130,6 @@ typedef	struct
         char *          sent_nick;
         char *          sent_body;
 
-	int		(*dgets) (char *, int, int, void *);
 }	Server;
 extern	Server	**server_list;
 #endif	/* NEED_SERVER_LIST */
@@ -164,18 +176,6 @@ static __inline__ Server *	get_server (int server)
 #define NOSERV		-2
 #define FROMSERV	-1
 
-/*
- * type definition to distinguish different
- * server versions
- */
-#define Server2_8	1
-#define Server2_9	2
-#define Server2_10	3
-#define Server_u2_8	4
-#define Server_u2_9	5
-#define Server_u2_10	6
-#define Server_u3_0	7
-
 /* Funny stuff */
 #define FUNNY_PUBLIC            1 << 0
 #define FUNNY_PRIVATE           1 << 1
@@ -183,36 +183,49 @@ static __inline__ Server *	get_server (int server)
 #define FUNNY_USERS             1 << 4
 #define FUNNY_NAME              1 << 5
 
+#define SERVER_CREATED		0
+#define SERVER_RECONNECT	1
+#define SERVER_DNS		2
+#define SERVER_CONNECTING	3
+#define SERVER_SSL_CONNECTING	4
+#define SERVER_REGISTERING	5
+#define SERVER_SYNCING		6
+#define SERVER_ACTIVE		7
+#define SERVER_EOF		8
+#define SERVER_ERROR		9
+#define SERVER_CLOSING		10
+#define SERVER_CLOSED		11
+#define SERVER_DELETED		12
+extern	const char *server_states[13];
+
 
 
 	BUILT_IN_COMMAND(servercmd);
 	BUILT_IN_COMMAND(disconnectcmd);
+	BUILT_IN_COMMAND(reconnectcmd);
 
-	void	add_to_server_list 		(const char *, int, 
-						 const char *, const char *, 
-						 const char *, const char *,
-						 int);
-	int	find_in_server_list		(const char *, int);
-	void	destroy_server_list		(void);
-	int	find_server_refnum		(char *, char **rest);
-	int	parse_server_index		(const char *, int);
-	void	parse_server_info		(char **, char **, char **,
-						 char **, char **, char **);
-	void	build_server_list		(char *, char *);
-	int	read_server_file		(void);
-	void	display_server_list		(void);
-	char *	create_server_list		(void);	/* MALLOC */
-	int	server_list_size		(void);
+	int	str_to_servref		(Char *);
+	int	str_to_servref_with_update	(const char *desc);
+	int	str_to_newserv		(Char *);
+	void	destroy_server_list	(void);
+	void	add_servers		(char *, Char *);
+	int	read_default_server_file (void);
+	void	display_server_list	(void);
+	char *	create_server_list	(void);	/* MALLOC */
+	int	server_list_size	(void);
+	int	is_server_valid		(int refnum);
 
-	void	do_server 			(fd_set *, fd_set *);
 	void	flush_server			(int);
 	void	send_to_server			(const char *, ...) __A(1);
 	void	send_to_aserver			(int, const char *, ...) __A(2);
 	void	send_to_aserver_raw		(int, size_t len, const char *buffer);
-	int	connect_to_new_server		(int, int, int);
+	int	grab_server_address		(int);
+	int	connect_to_server		(int);
 	int	close_all_servers		(const char *);
 	void	close_server			(int, const char *);
 
+	void	do_server			(int);
+
 	void	set_server_away			(int, const char *);
 const	char *	get_server_away			(int);
 
@@ -221,7 +234,7 @@ const	char *	get_possible_umodes		(int);
 const	char *	get_umode			(int);
 	void	clear_user_modes		(int);
 	void    reinstate_user_modes    	(void);
-	void    update_user_mode        	(const char *);
+	void    update_user_mode        	(int, const char *);
 	void	set_server_flag			(int, int, int);
 	int	get_server_flag			(int, int);
 
@@ -234,6 +247,11 @@ const	char *	get_server_name			(int);
 const	char *	get_server_itsname		(int);
 	void	set_server_group		(int, const char *);
 const	char *	get_server_group		(int);
+const	char *    get_server_server_type	(int);
+	void    set_server_server_type		(int, const char *);
+	void	set_server_vhost		(int, const char *);
+const	char *	get_server_vhost		(int);
+
 const	char *	get_server_type			(int);
 	void	set_server_version_string	(int, const char *);
 const 	char *	get_server_version_string	(int);
@@ -241,12 +259,11 @@ const 	char *	get_server_version_string	(int);
 const	char *	get_server_cipher		(int);
  
 	void	register_server			(int, const char *);
-	void	server_registration_is_not_pending (int);
 	void	password_sendline		(char *, char *);
-	char *	set_server_password		(int, const char *);
 	int	is_server_open			(int);
 	int	is_server_registered		(int);
-	void	server_is_registered		(int, int);
+	void	server_is_registered		(int, const char *, const char *);
+	int	is_server_active		(int);
 	int	auto_reconnect_callback		(void *);
 	int	server_reconnects_to		(int, int);
 	int	reconnect			(int, int);
@@ -270,9 +287,7 @@ const	char *	get_server_nickname		(int);
 	void	change_server_nickname		(int, const char *);
 const	char *	get_pending_nickname		(int);
 	void	accept_server_nickname		(int, const char *);
-	void	fudge_nickname			(int);
-	void	nickname_sendline		(char *, char *);
-	void	reset_nickname			(int);
+	void   nickname_change_rejected		(int, const char *);
 
 	void	set_server_redirect		(int, const char *);
 const	char *	get_server_redirect		(int);
@@ -290,7 +305,7 @@ const	char *	get_server_redirect		(int);
 	void	make_005			(int);
 	void	destroy_005			(int);
 const	char*	get_server_005			(int, const char *);
-	void	set_server_005			(int, char*, char*);
+	void	set_server_005			(int, char*, const char*);
 
         void    server_hard_wait 		(int);
         void    server_passive_wait 		(int, const char *);
@@ -302,8 +317,6 @@ const	char*	get_server_005			(int, const char *);
         int     get_server_doing_notice 	(int);
         void    set_server_doing_ctcp 		(int, int);
         int     get_server_doing_ctcp 		(int);
-	void	set_server_nickname_pending	(int, int);
-	int	get_server_nickname_pending	(int);
 	void	set_server_sent			(int, int);
 	int	get_server_sent			(int);
 	void	set_server_try_ssl		(int, int);
@@ -318,6 +331,14 @@ const	char*	get_server_005			(int, const char *);
 	int	get_server_line_length		(int);
 	void	set_server_max_cached_chan_size	(int, int);
 	int	get_server_max_cached_chan_size	(int);
+	void	set_server_ison_max		(int, int);
+	int	get_server_ison_max		(int);
+	void	set_server_userhost_max		(int, int);
+	int	get_server_userhost_max		(int);
+	void	set_server_status		(int, int);
+	int	get_server_status		(int);
+	void	set_server_autoclose		(int, int);
+	int	get_server_autoclose		(int);
 
         void    set_server_invite_channel       (int, const char *);
 const char *    get_server_invite_channel       (int);
@@ -339,6 +360,12 @@ const char *    get_server_quit_message		(int);
 const char *	get_server_cookie         	(int);
 	void	set_server_last_notify_nick	(int, const char *);
 const char *	get_server_last_notify_nick    	(int);
+	void	set_server_unique_id		(int, const char *);
+const char *	get_server_unique_id    	(int);
+	void	set_server_realname		(int, const char *);
+const char *	get_server_realname		(int);
+	void	set_server_default_realname	(int, const char *);
+const char *	get_server_default_realname	(int);
 
 	void	set_server_funny_min         	(int, int);
 	int	get_server_funny_min         	(int);
@@ -352,6 +379,20 @@ const char *	get_server_funny_match         	(int);
 
         void    set_server_window_count         (int, int);
         int     get_server_window_count         (int);
+        void    set_server_stricmp_table        (int, int);
+        int     get_server_stricmp_table        (int);
+        void    set_server_ison_len             (int, int);
+        int     get_server_ison_len             (int);
 
 	char *	serverctl			(char *);
+
+	int	server_more_addrs		(int);
+	int	server_addrs_left		(int);
+	int	get_server_by_desc		(const char *, int);
+
+const char *	get_server_altname		(int refnum, int which);
+	int     which_server_altname		(int refnum, const char *);
+
+
 #endif /* _SERVER_H_ */
+
