@@ -25,7 +25,7 @@ typedef enum {
 	Timeval	TimerTimeout 	(void);
 	char *	timerctl	(char *);
 	void	dump_timers	(void);
-	void    timers_swap_winrefs (int oldref, int newref);
-	void	timer_merge_winrefs (int oldref, int newref);
+	void    timers_swap_winrefs (unsigned oldref, unsigned newref);
+	void	timers_merge_winrefs (unsigned oldref, unsigned newref);
 
 #endif /* _TIMER_H_ */
