@@ -1,4 +1,4 @@
-/* $EPIC: exec.c,v 1.45 2009/11/14 05:39:10 jnelson Exp $ */
+/* $EPIC: exec.c,v 1.46 2012/11/23 16:04:49 jnelson Exp $ */
 /*
  * exec.c: handles exec'd process for IRCII 
  *
@@ -552,8 +552,10 @@ say("Output from process %d (%s) now going to you", i, proc->name);
 			 * Sever all ties we had with the parent ircII process
 			 */
 			setsid();
-			setuid(getuid());
-			setgid(getgid());
+			if (setuid(getuid()))
+				_exit(0);
+			if (setgid(getgid()))
+				_exit(0);
 			my_signal(SIGINT, SIG_IGN);
 			my_signal(SIGQUIT, SIG_DFL);
 			my_signal(SIGSEGV, SIG_DFL);
