@@ -1,4 +1,4 @@
-/* $EPIC: expr.c,v 1.23 2004/01/31 22:15:01 jnelson Exp $ */
+/* $EPIC: expr.c,v 1.41 2012/11/24 18:53:36 jnelson Exp $ */
 /*
  * expr.c -- The expression mode parser and the textual mode parser
  * #included by alias.c -- DO NOT DELETE
@@ -39,7 +39,7 @@
 
 /* Function decls */
 static	void	TruncateAndQuote (char **, const char *, ssize_t, const char *);
-static	char	*alias_special_char(char **, char *, const char *, char *, int *);
+static	char	*alias_special_char(char **, char *, const char *, char *);
 static	void	do_alias_string (char *, char *);
 
 char *alias_string = NULL;
@@ -103,7 +103,7 @@ static char	*lastop (char *ptr)
 #define NU_UNIT 8
 
 /*
- * Cleaned up/documented by Jeremy Nelson, Feb 1996.
+ * Cleaned up/documented in Feb 1996.
  *
  * What types of operators does ircII (EPIC) support?
  *
@@ -127,12 +127,11 @@ static char	*lastop (char *ptr)
  * Backslash quotes any character not within [] or (),
  * which have their own quoting rules (*sigh*)
  */ 
-static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
+static	char	*next_unit (char *str, const char *args, int stage)
 #if 0
 	char	*str,			/* start of token */
 		*args;			/* the values of $0, $1, etc */
-	int	*arg_flag,		/* whether we use $0, $1, etc */
-		stage;			/* What parsing level were at */
+	int	stage;			/* What parsing level were at */
 #endif
 {
 	char	*ptr,			/* pointer to the current op */
@@ -141,8 +140,7 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 		*lastc,			/* used to denote end of token-set */
 		*tmp = NULL,		/* place to malloc_sprintf() into */
 		op;			/* the op were working on */
-	int	got_sloshed = 0,	/* If the last char was a slash */
-		display;
+	int	got_sloshed = 0;	/* If the last char was a slash */
 
 	char	*result1 = (char *) 0,	/* raw lefthand-side of operator */
 		*result2 = (char *) 0,	/* raw righthand-side of operator */
@@ -173,10 +171,10 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 	ptr++;								\
 									\
 	/* Figure out the variable name were working on */		\
-	varname = expand_alias(str, args, arg_flag, NULL);		\
+	varname = expand_alias(str, args);			\
 	lastc = varname + strlen(varname) - 1;				\
-	while (lastc > varname && *lastc == ' ')			\
-		*lastc-- = '\0';					\
+	while (lastc > varname && isspace(*lastc))			\
+		*lastc-- = 0;						\
 	while (my_isspace(*varname))					\
 		 varname++;						\
 									\
@@ -186,7 +184,7 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 	new_free(&result1);						\
 									\
 	/* Get the value of the explicit argument */			\
-	result2 = next_unit(ptr, args, arg_flag, stage);		\
+	result2 = next_unit(ptr, args, stage);				\
 	var2 = func (result2);						\
 	new_free(&result2);						\
 }
@@ -222,8 +220,8 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 	*ptr++ = '\0';							\
 									\
 	/* Get the two explicit operands */				\
-	result1 = next_unit(str, args, arg_flag, stage);		\
-	result2 = next_unit(ptr, args, arg_flag, stage);		\
+	result1 = next_unit(str, args, stage);		\
+	result2 = next_unit(ptr, args, stage);		\
 									\
 	/* Convert them with the specified function */			\
 	var1 = func (result1);						\
@@ -279,8 +277,8 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 		lastc--;
 
 	/* and remove trailing spaces */
-	while (my_isspace(*lastc))
-		*lastc-- = '\0';
+	while (lastc > str && my_isspace(*lastc))
+		*lastc-- = 0;
 
 	/* Must have arguments.  Fill in if not supplied */
 	if (!args)
@@ -293,7 +291,7 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 	if (stage == NU_UNIT && *lastc == ')' && *str == '(')
 	{
 		str++, *lastc-- = '\0';
-		return next_unit(str, args, arg_flag, NU_EXPR);
+		return next_unit(str, args, NU_EXPR);
 	}
 
 
@@ -374,7 +372,7 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 			else
 				ptr2 = NULL;
 
-			result1 = call_function(str, args, arg_flag);
+			result1 = call_function(str, args);
 
 			if (ptr2 && savec)
 				*ptr2 = savec;
@@ -398,7 +396,7 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 			if (ptr && *ptr)
 			{
 				malloc_strcat(&result1, ptr);
-				result2 = next_unit(result1, args, arg_flag, stage);
+				result2 = next_unit(result1, args, stage);
 				new_free(&result1);
 				result1 = result2;
 			}
@@ -418,12 +416,10 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 		{
 			ssize_t span;
 
-			display = window_display;
-
 			if ((span = MatchingBracket(ptr + 1, '{', '}')) >= 0)
 				ptr2 = ptr + 1 + span;
 			else
-				ptr2 = ptr + strlen(ptr);
+				ptr2 = ptr + strlen(ptr) - 1;
 
 			if (stage != NU_UNIT)
 			{
@@ -504,7 +500,6 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 			    if (right[1] == '*' && right[2] == 0)
 			    {
 				result1 = malloc_strdup(args);
-				*arg_flag = 1;
 			    }
 			    else
 			    {
@@ -515,16 +510,10 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 				if (end && !*end)
 				{
 					if (j < 0)
-						result1 = extractw2(args, SOS, -j);
+						result1 = extractew2(args, SOS, -j);
 					else
-						result1 = extractw2(args, j, j);
+						result1 = extractew2(args, j, j);
 
-					/* 
-					 * Oops. this would have been set by
-					 * expand alias, so we have to do it
-					 * explicitly here!
-					 */
-					*arg_flag = 1;
 				}
 
 				/*
@@ -532,18 +521,17 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 				 */
 				else if (*end == '-' && !end[1])
 				{
-					result1 = extractw2(args, j, EOS);
-					*arg_flag = 1;
+					result1 = extractew2(args, j, EOS);
 				}
 				else
-					result1 = expand_alias(right, args, arg_flag, NULL);
+					result1 = expand_alias(right, args);
 			    }
 			}
 			else if (!strchr(right, '$') && !strchr(right, '\\'))
 				result1 = malloc_strdup(right);
 			else
 #endif
-				result1 = expand_alias(right, args, arg_flag, NULL);
+				result1 = expand_alias(right, args);
 
 			/*
 			 * You need to look closely at this, as this test 
@@ -578,7 +566,7 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 				if (ptr && *ptr)
 				{
 					strlcat(result2, ptr, size);
-					result1 = next_unit(result2, args, arg_flag, stage);
+					result1 = next_unit(result2, args, stage);
 				}
 				else
 				{
@@ -597,7 +585,7 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 			else if (ptr && *ptr)
 			{
 				malloc_strcat(&result1, ptr);
-				result2 = next_unit(result1, args, arg_flag, stage);
+				result2 = next_unit(result1, args, stage);
 				new_free(&result1);
 				result1 = result2;
 			}
@@ -650,7 +638,7 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 				else			/* postfix */
 					prefix = 0, ptr2 = str, *ptr++ = 0;
 
-				varname = expand_alias(ptr2, args, arg_flag, NULL);
+				varname = expand_alias(ptr2, args);
 				upper(varname);
 
 				if (!(result1 = get_variable(varname)))
@@ -677,30 +665,28 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 				break;
 
 
-#ifdef FLOATING_POINT_SUPPORT
-			SETUP_FLOAT_OPERATION(NU_ADD)
+			if (get_int_var(FLOATING_POINT_MATH_VAR))
+			{
+			    SETUP_FLOAT_OPERATION(NU_ADD)
 
-			if (op == '-')
+			    if (op == '-')
 				dvalue3 = dvalue1 - dvalue2;
-			else
+			    else
 				dvalue3 = dvalue1 + dvalue2;
 
-#if 0
-			tmp = malloc_sprintf(NULL, "%.50g", dvalue3);
-			canon_number(tmp);
-#else
-			tmp = malloc_strdup(ftoa(dvalue3));
-#endif
-#else
-			SETUP_INTEGER_OPERATION(NU_ADD)
-
-			if (op == '-')
-				value3 = value1 - value2;
+			    tmp = malloc_strdup(ftoa(dvalue3));
+			}
 			else
+			{
+			    SETUP_INTEGER_OPERATION(NU_ADD)
+
+			    if (op == '-')
+				value3 = value1 - value2;
+			    else
 				value3 = value1 + value2;
 
-			tmp = malloc_strdup(ltoa(value3));
-#endif
+			    tmp = malloc_strdup(ltoa(value3));
+			}
 			CLEANUP_IMPLIED()
 			return tmp;
 		}
@@ -730,11 +716,7 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 			{
 				*ptr++ = '\0';
 				SETUP_BINARY(dvalue1, dvalue2, atof)
-#if 0
-				return malloc_sprintf(NULL, "%.50g", pow(dvalue1, dvalue2));
-#else
 				return malloc_strdup(ftoa(pow(dvalue1, dvalue2)));
-#endif
 			}
 
 			SETUP_FLOAT_OPERATION(NU_MULT)
@@ -752,12 +734,7 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 					dvalue3 = (int)dvalue1 % (int)dvalue2;
 			}
 
-#if 0
-			tmp = malloc_sprintf(NULL, "%.50g", dvalue3);
-			canon_number(tmp);
-#else
 			tmp = malloc_strdup(ftoa(dvalue3));
-#endif
 			CLEANUP_IMPLIED()
 			return tmp;
 		}
@@ -775,8 +752,8 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 			{
 				*ptr++ = '\0';
 				ptr++;
-				result1 = next_unit(str, args, arg_flag, stage);
-				result2 = next_unit(ptr, args, arg_flag, stage);
+				result1 = next_unit(str, args, stage);
+				result2 = next_unit(ptr, args, stage);
 				malloc_strcat(&result1, result2);
 				new_free(&result2);
 				return result1;
@@ -816,8 +793,8 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 
 
 	/* 
-	 * Reworked - Jeremy Nelson, Feb 1994
-	 * Reworked again, Feb 1996 (jfn)
+	 * Reworked in Feb 1994
+	 * Reworked again in Feb 1996
 	 * 
 	 * X, XX, and X= are all supported, where X is one of "&" (and), 
 	 * "|" (or) and "^" (xor).  The XX forms short-circuit, as they
@@ -831,10 +808,10 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 				*ptr++ = '\0';
 				ptr++;
 
-				result1 = next_unit(str, args, arg_flag, stage);
+				result1 = next_unit(str, args, stage);
 				if (check_val(result1))
 				{
-					result2 = next_unit(ptr, args, arg_flag, stage);
+					result2 = next_unit(ptr, args, stage);
 					value3 = check_val(result2);
 				}
 				else
@@ -878,10 +855,10 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 				*ptr++ = '\0';
 				ptr++;
 
-				result1 = next_unit(str, args, arg_flag, stage);
+				result1 = next_unit(str, args, stage);
 				if (!check_val(result1))
 				{
-					result2 = next_unit(ptr, args, arg_flag, stage);
+					result2 = next_unit(ptr, args, stage);
 					value3 = check_val(result2);
 				}
 				else
@@ -924,8 +901,8 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 				*ptr++ = '\0';
 				ptr++;
 
-				value1 = check_val((result1 = next_unit(str, args, arg_flag, stage)));
-				value2 = check_val((result2 = next_unit(ptr, args, arg_flag, stage)));
+				value1 = check_val((result1 = next_unit(str, args, stage)));
+				value2 = check_val((result2 = next_unit(ptr, args, stage)));
 				new_free(&result1);
 				new_free(&result2);
 
@@ -967,7 +944,7 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 			ssize_t span;
 
 			*ptr++ = 0;
-			result1 = next_unit(str, args, arg_flag, stage);
+			result1 = next_unit(str, args, stage);
 			span = MatchingBracket(ptr, '?', ':');
 
 			/* Unbalanced :, or possibly missing */
@@ -981,9 +958,9 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 			*ptr2++ = 0;
 
 			if ( check_val(result1) )
-				result2 = next_unit(ptr, args, arg_flag, stage);
+				result2 = next_unit(ptr, args, stage);
 			else
-				result2 = next_unit(ptr2, args, arg_flag, stage);
+				result2 = next_unit(ptr2, args, stage);
 
 			/* XXXX - needed? */
 			ptr2[-1] = ':';
@@ -1009,8 +986,8 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 			{
 				*ptr++ = 0;
 				ptr++;
-				result1 = next_unit(str, args, arg_flag, stage);
-				result2 = next_unit(ptr, args, arg_flag, stage);
+				result1 = next_unit(str, args, stage);
+				result2 = next_unit(ptr, args, stage);
 				if (wild_match(result2, result1))
 					malloc_strcpy(&result1, one);
 				else
@@ -1023,11 +1000,11 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 			{
 				*ptr++ = '\0';
 				upper(str);
-				result1 = expand_alias(str, args, arg_flag, NULL);
-				result2 = next_unit(ptr, args, arg_flag, stage);
+				result1 = expand_alias(str, args);
+				result2 = next_unit(ptr, args, stage);
 
 				lastc = result1 + strlen(result1) - 1;
-				while (lastc > result1 && *lastc == ' ')
+				while (lastc > result1 && isspace(*lastc))
 					*lastc-- = '\0';
 				for (varname = result1; my_isspace(*varname);)
 					varname++;
@@ -1047,8 +1024,8 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 			{
 				*ptr++ = '\0';
 				ptr++;
-				result1 = next_unit(str, args, arg_flag, stage);
-				result2 = next_unit(ptr, args, arg_flag, stage);
+				result1 = next_unit(str, args, stage);
+				result2 = next_unit(ptr, args, stage);
 				if (!my_stricmp(result1, result2))
 					malloc_strcpy(&result1, one);
 				else
@@ -1077,8 +1054,8 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 				*ptr++ = 0;
 				ptr++;
 
-				result1 = next_unit(str, args, arg_flag, stage);
-				result2 = next_unit(ptr, args, arg_flag, stage);
+				result1 = next_unit(str, args, stage);
+				result2 = next_unit(ptr, args, stage);
 				value1 = my_atol(result1);
 				value2 = my_atol(result2);
 				if (op == '<')
@@ -1100,8 +1077,8 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 					value3 = 0;
 
 				*ptr++ = '\0';
-				result1 = next_unit(str, args, arg_flag, stage);
-				result2 = next_unit(ptr, args, arg_flag, stage);
+				result1 = next_unit(str, args, stage);
+				result2 = next_unit(ptr, args, stage);
 
 				if ((my_isdigit(result1)) && (my_isdigit(result2)))
 				{
@@ -1140,7 +1117,7 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 		{
 			if (ptr == str && stage == NU_UNIT)
 			{
-				result1 = next_unit(str+1, args, arg_flag, stage);
+				result1 = next_unit(str+1, args, stage);
 				if (isdigit(*result1))
 					value1 = ~my_atol(result1);
 				else
@@ -1164,7 +1141,7 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 		{
 			if (ptr == str && stage == NU_UNIT)
 			{
-				result1 = next_unit(str+1, args, arg_flag, stage);
+				result1 = next_unit(str+1, args, stage);
 
 				if (my_isdigit(result1))
 				{
@@ -1183,8 +1160,8 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 				*ptr++ = 0;
 				ptr++;
 
-				result1 = next_unit(str, args, arg_flag, stage);
-				result2 = next_unit(ptr, args, arg_flag, stage);
+				result1 = next_unit(str, args, stage);
+				result2 = next_unit(ptr, args, stage);
 
 				if (!wild_match(result2, result1))
 					malloc_strcpy(&result1, one);
@@ -1200,8 +1177,8 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 				*ptr++ = '\0';
 				ptr++;
 
-				result1 = next_unit(str, args, arg_flag, stage);
-				result2 = next_unit(ptr, args, arg_flag, stage);
+				result1 = next_unit(str, args, stage);
+				result2 = next_unit(ptr, args, stage);
 
 				if (!my_stricmp(result1, result2))
 					malloc_strcpy(&result1, zero);
@@ -1228,8 +1205,8 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 			if (stage == NU_EXPR)
 			{
 				*ptr++ = '\0';
-				result1 = next_unit(str, args, arg_flag, stage);
-				result2 = next_unit(ptr, args, arg_flag, stage);
+				result1 = next_unit(str, args, stage);
+				result2 = next_unit(ptr, args, stage);
 				new_free(&result1);
 				return result2;
 			}
@@ -1248,7 +1225,7 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 	 * If were not done parsing, parse it again.
 	 */
 	if (stage != NU_UNIT)
-		return next_unit(str, args, arg_flag, stage + 1);
+		return next_unit(str, args, stage + 1);
 
 	/*
 	 * If the result is a number, return it.
@@ -1279,7 +1256,7 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
 	if (op)
 	{
 		if (op == '#')
-			value1 = count_words(result1, DWORD_YES, "\"");
+			value1 = count_words(result1, DWORD_EXTRACTW, "\"");
 		else if (op == '@')
 			value1 = strlen(result1);
 		new_free(&result1);
@@ -1297,17 +1274,76 @@ static	char	*next_unit (char *str, const char *args, int *arg_flag, int stage)
  * about this at some future date. The ^ function and some fixes by
  * troy@cbme.unsw.EDU.AU (Troy Rollo) 
  */
-char	*parse_inline (char *str, const char *args, int *args_flag)
+char	*parse_inline (char *str, const char *args)
 {
-	if (x_debug & DEBUG_NEW_MATH)
-		return matheval(str, args, args_flag);
+	if (get_int_var(OLD_MATH_PARSER_VAR))
+		return next_unit(str, args, NU_EXPR);
 	else
-		return next_unit(str, args, args_flag, NU_EXPR);
+		return matheval(str, args);
 }
 
 
 
 /**************************** TEXT MODE PARSER *****************************/
+/*
+ * next_statement: Determines the length of the first statement in 'string'.
+ *
+ * A statement ends at the first semicolon EXCEPT:
+ *   -- Anything inside (...) or {...} doesn't count
+ */
+ssize_t	next_statement (const char *string)
+{
+	const char *ptr;
+	int	paren_count = 0, brace_count = 0;
+
+	if (!string || !*string)
+		return -1;
+
+	for (ptr = string; *ptr; ptr++)
+	{
+	    switch (*ptr)
+	    {
+		case ';':
+		    if (paren_count + brace_count)
+		    	break;
+		    goto all_done;
+		case LEFT_PAREN:
+		    paren_count++;
+		    break;
+		case LEFT_BRACE:
+		    brace_count++;
+		    break;
+		case RIGHT_PAREN:
+		    if (paren_count)
+			paren_count--;
+		    break;
+		case RIGHT_BRACE:
+		    if (brace_count)
+			brace_count--;
+		    break;
+		case '\\':
+		    ptr++;
+		    if (!*ptr)
+			goto all_done;
+		    break;
+	    }
+	}
+
+all_done:
+	if (paren_count != 0)
+	{
+		privileged_yell("[%d] More ('s than )'s found in this "
+				"statement: \"%s\"", paren_count, string);
+	}
+	else if (brace_count != 0)
+	{
+		privileged_yell("[%d] More {'s than }'s found in this "
+				"statement: \"%s\"", brace_count, string);
+	}
+
+	return (ssize_t)(ptr - string);
+}
+
 /*
  * expand_alias: Expands inline variables in the given string and returns the
  * expanded string in a new string which is malloced by expand_alias(). 
@@ -1316,12 +1352,9 @@ char	*parse_inline (char *str, const char *args, int *args_flag)
  *
  * Behaviour is modified by the following:
  *	Anything between brackets (...) {...} is left unmodified.
- *	If more_text is supplied, the text is broken up at
- *		semi-colons and returned one at a time. The unprocessed
- *		portion is written back into more_text.
  *	Backslash escapes are unescaped.
  */
-char	*expand_alias	(const char *string, const char *args, int *args_flag, ssize_t *more_text)
+char	*expand_alias	(const char *string, const char *args)
 {
 	char	*buffer = NULL,
 		*ptr,
@@ -1331,25 +1364,16 @@ char	*expand_alias	(const char *string, const char *args, int *args_flag, ssize_
 	char	quote_temp[2];
 	char	ch;
 	int	is_quote = 0;
-	char *	unescape = empty_string;
+	const char *	unescape = empty_string;
 	size_t	buffclue = 0;
 
 	if (!string || !*string)
 		return malloc_strdup(empty_string);
 
-	if (*string == '@' && more_text)
-	{
-		unescape = NULL;
-		*args_flag = 1; /* Stop the @ command from auto appending */
-	}
 	quote_temp[1] = 0;
 
 	stuff = LOCAL_COPY(string);
-	free_stuff = stuff;
-
-	ptr = stuff;
-	if (more_text)
-		*more_text = -1;
+	ptr = free_stuff = stuff;
 
 	while (ptr && *ptr)
 	{
@@ -1365,22 +1389,7 @@ char	*expand_alias	(const char *string, const char *args, int *args_flag, ssize_
 		    case '$':
 		    {
 			char *buffer1 = NULL;
-			/*
-			 * The test here ensures that if we are in the 
-			 * expression evaluation command, we don't expand $. 
-			 * In this case we are only coming here to do command 
-			 * separation at ';'s.  If more_text is not defined, 
-			 * and the first character is '@', we have come here 
-			 * from [] in an expression.
-			 */
-			if (more_text && *string == '@')
-			{
-				ptr++;
-				break;
-			}
 			*ptr++ = 0;
-
-			/* Just a lonely old $ at the end of string. */
 			if (!*ptr)
 			{
 				malloc_strcat_c(&buffer, empty_string, &buffclue);
@@ -1397,7 +1406,7 @@ char	*expand_alias	(const char *string, const char *args, int *args_flag, ssize_
 				quote_temp[0] = *ptr;
 				malloc_strcat(&quote_str, quote_temp);
 			}
-			stuff = alias_special_char(&buffer1, ptr, args, quote_str, args_flag);
+			stuff = alias_special_char(&buffer1, ptr, args, quote_str);
 			malloc_strcat_c(&buffer, buffer1, &buffclue);
 			new_free(&buffer1);
 			if (quote_str)		/* Why ``stuff''? */
@@ -1406,18 +1415,6 @@ char	*expand_alias	(const char *string, const char *args, int *args_flag, ssize_
 			break;
 		    }
 
-		    case ';':
-		    {
-			if (!more_text)
-			{
-				ptr++;
-				break;
-			}
-			*more_text = (ptr - free_stuff) + 1;
-			*ptr = 0; /* To terminate the loop */
-			break;
-		    }
-
 		    case LEFT_PAREN:
 		    case LEFT_BRACE:
 		    {
@@ -1428,12 +1425,11 @@ char	*expand_alias	(const char *string, const char *args, int *args_flag, ssize_
 			malloc_strcat_ues_c(&buffer, stuff, unescape, &buffclue);
 			stuff = ptr;
 
-			*args_flag = 1;
 			if ((span = MatchingBracket(stuff + 1, ch, 
 					(ch == LEFT_PAREN) ?
 					RIGHT_PAREN : RIGHT_BRACE)) < 0)
 			{
-				put_it("Unmatched %c", ch);
+				privileged_yell("Unmatched %c starting at [%-.20s]", ch, stuff + 1);
 				/* 
 				 * DO NOT ``OPTIMIZE'' THIS BECAUSE
 				 * *STUFF IS NUL SO STRLEN(STUFF) IS 0!
@@ -1471,9 +1467,9 @@ char	*expand_alias	(const char *string, const char *args, int *args_flag, ssize_
 	if (get_int_var(DEBUG_VAR) & DEBUG_EXPANSIONS)
 		privileged_yell("Expanded [%s] to [%s]", string, buffer);
 
-#if 0					/* Maybe a good idea later? */
-	if (!buffer && !more_text)
-		panic("expand_alias [%s] returning NULL!", string);
+#if 0						/* Maybe a good idea later? */
+	if (!buffer)
+		panic(1, "expanded_alias [%s] returning NULL!", string);
 #endif
 
 	return buffer;
@@ -1488,11 +1484,8 @@ char	*expand_alias	(const char *string, const char *args, int *args_flag, ssize_
  * character), the args to the alias, and a character indication what
  * characters in the string should be quoted with a backslash.  It returns a
  * pointer to the character right after the converted alias.
- * 
- * The args_flag is set to 1 if any of the $n, $n-, $n-m, $-m, $*, or $() 
- * is used in the alias.  Otherwise it is left unchanged.
  */
-static	char	*alias_special_char (char **buffer, char *ptr, const char *args, char *quote_em, int *args_flag)
+static	char	*alias_special_char (char **buffer, char *ptr, const char *args, char *quote_em)
 {
 	char	*tmp,
 		c;
@@ -1512,7 +1505,7 @@ static	char	*alias_special_char (char **buffer, char *ptr, const char *args, cha
 			if (*ptr == '$')
 			{
 				char *	str;
-				str = expand_alias(ptr, args, args_flag, NULL);
+				str = expand_alias(ptr, args);
 				length = my_atol(str);
 				new_free(&str);
 			}
@@ -1581,14 +1574,14 @@ static	char	*alias_special_char (char **buffer, char *ptr, const char *args, cha
 			if (ptr)
 				*ptr++ = 0;
 			else
-				yell("Unmatched ( (continuing anyways)");
+				yell("Unmatched ( after $ starting at [%-.20s] (continuing anyways)", ph);
 
 			/*
 			 * Keep expanding as long as neccesary.
 			 */
 			do
 			{
-				tmp2 = expand_alias(tmp, args, args_flag, NULL);
+				tmp2 = expand_alias(tmp, args);
 				if (tmpsav)
 					new_free(&tmpsav);
 				tmpsav = tmp = tmp2;
@@ -1596,7 +1589,7 @@ static	char	*alias_special_char (char **buffer, char *ptr, const char *args, cha
 			while (tmp && *tmp == '$');
 
 			alias_special_char(&sub_buffer, tmp, args, 
-						quote_em, args_flag);
+						quote_em);
 
 			/* Some kind of bogus expando */
 			if (sub_buffer == NULL)
@@ -1608,25 +1601,6 @@ static	char	*alias_special_char (char **buffer, char *ptr, const char *args, cha
 
 			new_free(&sub_buffer);
 			new_free(&tmpsav);
-			*args_flag = 1;
-			return (ptr);
-		}
-
-		/*
-		 * $!...! is the history mechanism.  Given a string, it will
-		 * return the most previously executed command that contains
-		 * that string.  Or something like that.
-		 */
-		case '!':
-		{
-			if ((ptr = strchr(tmp, '!')))
-				*ptr++ = 0;
-
-			if ((tmp = do_history(tmp, empty_string)) != NULL)
-			{
-				TruncateAndQuote(buffer, tmp, length, quote_em);
-				new_free(&tmp);
-			}
 			return (ptr);
 		}
 
@@ -1651,9 +1625,9 @@ static	char	*alias_special_char (char **buffer, char *ptr, const char *args, cha
 			if (ptr)
 				*ptr++ = 0;
 			else
-				yell("Unmatched { (continuing anyways)");
+				yell("Unmatched { after $ starting at [%-.20s] (continuing anyways)", ph);
 
-			if ((tmp = parse_inline(tmp, args, args_flag)) != NULL)
+			if ((tmp = parse_inline(tmp, args)) != NULL)
 			{
 				TruncateAndQuote(buffer, tmp, length, quote_em);
 				new_free(&tmp);
@@ -1703,8 +1677,9 @@ static	char	*alias_special_char (char **buffer, char *ptr, const char *args, cha
 		 */
 		case '*':
 		{
+			if (args == NULL)
+				args = LOCAL_COPY(empty_string);
 			TruncateAndQuote(buffer, args, length, quote_em);
-			*args_flag = 1;
 			return (ptr + 1);
 		}
 
@@ -1728,26 +1703,22 @@ static	char	*alias_special_char (char **buffer, char *ptr, const char *args, cha
 			if (rest == ptr + 1)
 			{
 			    sub_buffer = malloc_strdup(args);
-			    *args_flag = 1;
 			}
 			else
 			{
 			    c2 = *rest;
 			    *rest = 0;
 			    alias_special_char(&sub_buffer, ptr + 1, 
-						args, quote_em, args_flag);
+						args, quote_em);
 			    *rest = c2;
 			}
 
 			if (c == '#')
-			    val = malloc_strdup(ltoa(count_words(sub_buffer, DWORD_YES, "\"")));
+			    val = malloc_strdup(ltoa(count_words(sub_buffer, DWORD_EXTRACTW, "\"")));
 			else
 			    val = malloc_strdup(ltoa(strlen(sub_buffer)));
 
-#if 0	/* Don't need to do this here */
-			if (!(x_debug & DEBUG_SLASH_HACK))
-#endif
-				TruncateAndQuote(buffer, val, length, quote_em);
+			TruncateAndQuote(buffer, val, length, quote_em);
 			new_free(&val);
 			new_free(&sub_buffer);
 
@@ -1780,8 +1751,6 @@ static	char	*alias_special_char (char **buffer, char *ptr, const char *args, cha
 			{
 			    char *tmp2;
 
-			    *args_flag = 1;
-
 			    /*
 			     * Handle $~.  EOS especially handles this
 			     * condition.
@@ -1809,7 +1778,7 @@ static	char	*alias_special_char (char **buffer, char *ptr, const char *args, cha
 				ptr++;
 				my_upper = parse_number(&ptr);
 				if (my_upper == -1)
-				    return empty_string; /* error */
+				    return endstr(ptr); /* error */
 			    }
 
 			    /*
@@ -1842,11 +1811,13 @@ static	char	*alias_special_char (char **buffer, char *ptr, const char *args, cha
 			    if (!args)
 				tmp2 = malloc_strdup(empty_string);
 			    else
-				tmp2 = extractw2(args, my_lower, my_upper);
+				tmp2 = extractew2(args, my_lower, my_upper);
 
 			    TruncateAndQuote(buffer, tmp2, length, quote_em);
 			    new_free(&tmp2);
-			    return (ptr ? ptr : empty_string);
+			    if (!ptr)
+				panic(1, "ptr is NULL after parsing numeric expando");
+			    return ptr;
 			}
 
 			/*
@@ -1866,9 +1837,9 @@ static	char	*alias_special_char (char **buffer, char *ptr, const char *args, cha
 			    }
 
 			    if (function_call)
-				tmp = call_function(ptr, args, args_flag);
+				tmp = call_function(ptr, args);
 			    else
-				tmp = get_variable_with_args(ptr, args, args_flag);
+				tmp = get_variable_with_args(ptr, args);
 
 			    if (!tmp)
 				tmp = malloc_strdup(empty_string);
@@ -1889,7 +1860,7 @@ static	char	*alias_special_char (char **buffer, char *ptr, const char *args, cha
 			}
 		}
 	}
-	panic("Returning NULL from alias_special_char");
+	panic(1, "Returning NULL from alias_special_char");
 	return NULL;
 }
 
@@ -1903,7 +1874,7 @@ static	void	TruncateAndQuote (char **buff, const char *add, ssize_t length, cons
 	char *	buffer;
 
 	if (!add)
-		panic("add is NULL");
+		panic(1, "add is NULL");
 
 	/* 
 	 * Semantics:
