@@ -1,4 +1,4 @@
-/* $EPIC: expr2.c,v 1.18 2003/12/03 05:21:11 jnelson Exp $ */
+/* $EPIC: expr2.c,v 1.40 2009/06/27 16:35:58 jnelson Exp $ */
 /*
  * Zsh: math.c,v 3.1.2.1 1997/06/01 06:13:15 hzoli Exp 
  * math.c - mathematical expression evaluation
@@ -33,12 +33,12 @@
  *
  */
 /*
- * Substantial modifications by Jeremy Nelson which are
+ * Substantial modifications by EPIC Software Labs which are
  * Coypright 1998, 2003 EPIC Software Labs, All rights reserved.
  *
  * You may distribute this file under the same terms as the above, by 
- * including the parties "Jeremy Nelson" and "EPIC Software Labs" to the 
- * limitations of liability and the express disclaimer of all warranties.
+ * including the parties "EPIC Software Labs" to the limitations of 
+ * liability and the express disclaimer of all warranties.  
  * This software is provided "AS IS".
  */
 #include <math.h>
@@ -136,23 +136,6 @@ typedef 	int		BooL;
 #define USED_FLOAT		1 << 4
 #define USED_BOOLEAN		1 << 5
 
-/*
- * Theoretically, all these macros are independant because the return value of
- * INT*FUNC is typecasted back to INTTYPE.  One caveat is that INT2STR
- * must return a malloc'd string.
- */
-#ifdef HAVE_LONG_LONG
-typedef long long INTTYPE;
-#define FORMAT "%lld"
-#define STR2INT(x) ((INTTYPE)atoll(x))
-#define INT2STR(x) (malloc_sprintf(NULL, FORMAT , (INTTYPE)(x)))
-#else
-typedef long INTTYPE;
-#define FORMAT "%ld"
-#define STR2INT(x) ((INTTYPE)atol(x))
-#define INT2STR(x) (malloc_sprintf(NULL, FORMAT , (INTTYPE)(x)))
-#endif
-
 /*
  * This is a symbol table entry.
  */
@@ -162,7 +145,7 @@ typedef struct TOKEN_type
 	char *	lval;			/* Cached unexpanded variable name */
 	char *	raw_value;		/* Cached unexpected string */
 	char *	expanded_value;		/* Cached full expanded string */
-	INTTYPE	integer_value;		/* Cached integer value */
+	intmax_t integer_value;		/* Cached integer value */
 	double	float_value;		/* Cached floating point value */
 	short	boolean_value;		/* Cached boolean value */
 } SYMBOL;
@@ -174,6 +157,9 @@ typedef struct TOKEN_type
  */
 typedef struct
 {
+	/* This is the original expression, used for debugging output */
+	char 	*orig_expr;
+
 	/* POSITION AND STATE INFORMATION */
 	/*
 	 * This is the current position in the lexing.
@@ -237,7 +223,6 @@ typedef struct
 	TOKEN	last_token;
 
 	const char	*args;
-	int	*args_flag;
 } expr_info;
 
 /* 
@@ -247,11 +232,11 @@ typedef struct
 #define TOK(c, v) 	c->tokens[v]
 
 /* Forward function references */
-__inline static	TOKEN	tokenize_raw (expr_info *c, char *t);
+__inline static	TOKEN	tokenize_raw (expr_info *c, const char *t);
 	static	char *	after_expando_special (expr_info *c);
 	static	char *	alias_special_char (char **buffer, char *ptr, 
-					const char *args, char *quote_em, 
-					int *args_flag);
+					const char *args, char *quote_em);
+__inline static	const char *	get_token_expanded (expr_info *c, TOKEN v);
 
 
 /******************** EXPRESSION CONSTRUCTOR AND DESTRUCTOR ****************/
@@ -261,12 +246,15 @@ __inline static	TOKEN	tokenize_raw (expr_info *c, char *t);
  */
 static void	setup_expr_info (expr_info *c)
 {
+#ifdef NO_CHEATING
 	int	i;
+#endif
 
 	c->ptr = NULL;
 	c->noeval = 0;
 	c->operand = 1;
 	c->token = 0;
+#ifdef NO_CHEATING
 	for (i = 0; i <= TOKENCOUNT; i++)
 	{
 		TOK(c, i).used = USED_NONE;
@@ -279,6 +267,10 @@ static void	setup_expr_info (expr_info *c)
 	}
 	for (i = 0; i <= STACKSZ; i++)
 		c->stack[i] = 0;
+#else
+	memset(c->tokens, 0, sizeof(c->tokens));
+	memset(c->stack, 0, sizeof(c->stack));
+#endif
 	c->sp = -1;
 	c->mtok = 0;
 	c->errflag = 0;
@@ -330,17 +322,17 @@ static void 	destroy_expr_info (expr_info *c)
  */
 
 enum LEX {
-	M_INPAR,
+	M_FUNCTION,	M_INPAR,
 	NOT, 		COMP, 		PREMINUS,	PREPLUS,
 			UPLUS,		UMINUS,		STRLEN,
-			WORDC,		DEREF,
+			WORDC,		DEREF,		REEXP,
 	POWER,
 	MUL,		DIV,		MOD,
 	PLUS,		MINUS,		STRCAT,
 	SHLEFT,		SHRIGHT,
 	LES,		LEQ,		GRE,		GEQ,
 	MATCH,		NOMATCH,
-	DEQ,		NEQ,
+	DEQ,		NEQ,		LITEQ,		NOTLITEQ,
 	AND,
 	XOR,
 	OR,
@@ -379,17 +371,17 @@ enum LEX {
  */
 static	int	prec[TOKCOUNT] = 
 {
-	1,
+	1,		1,
 	2,		2,		2,		2,
 			2,		2,		2,
-			2,		2,
+			2,		2,		2,
 	3,
 	4,		4,		4,
 	5,		5,		5,
 	6,		6,
 	7,		7,		7,		7,
 	8,		8,
-	9,		9,
+	9,		9,		9,		9,
 	10,
 	11,
 	12,
@@ -433,17 +425,17 @@ static	int	prec[TOKCOUNT] =
  */
 static 	int 	assoc[TOKCOUNT] =
 {
-	LR,
+	LR,		LR,
 	RL,		RL,		RL,		RL,
 			RL,		RL,		RL,
-			RL,		RL,
+			RL,		RL,		RL,
 	RL,
 	LR,		LR,		LR,
 	LR,		LR,		LR,
 	LR,		LR,
 	LR,		LR,		LR,		LR,
 	LR,		LR,
-	LR,		LR,
+	LR,		LR,		LR,		LR,
 	LR,
 	LR,
 	LR,
@@ -471,7 +463,7 @@ __inline static	TOKEN		tokenize_lval (expr_info *c, const char *t)
 {
 	if (c->token >= TOKENCOUNT)
 	{
-		error("Too many tokens for this expression");
+		my_error("Too many tokens for this expression");
 		return -1;
 	}
 	TOK(c, c->token).used = USED_LVAL;
@@ -488,11 +480,11 @@ __inline static	TOKEN		tokenize_lval (expr_info *c, const char *t)
  * result of operators) directly create "expanded" tokens.
  */
 /* THIS FUNCTION MAKES A NEW COPY OF 'T'.  YOU MUST DISPOSE OF 'T' YOURSELF */
-__inline static	TOKEN		tokenize_raw (expr_info *c, char *t)
+__inline static	TOKEN		tokenize_raw (expr_info *c, const char *t)
 {
 	if (c->token >= TOKENCOUNT)
 	{
-		error("Too many tokens for this expression");
+		my_error("Too many tokens for this expression");
 		return -1;
 	}
 	TOK(c, c->token).used = USED_RAW;
@@ -510,7 +502,7 @@ __inline static	TOKEN		tokenize_expanded (expr_info *c, char *t)
 {
 	if (c->token >= TOKENCOUNT)
 	{
-		error("Too many tokens for this expression");
+		my_error("Too many tokens for this expression");
 		return -1;
 	}
 	TOK(c, c->token).used = USED_EXPANDED;
@@ -524,11 +516,11 @@ __inline static	TOKEN		tokenize_expanded (expr_info *c, char *t)
  * is both expensive, and unnecesary.  This type of token is created by
  * integer-only operations like &, |, ^, <<, >>.
  */
-__inline static	TOKEN		tokenize_integer (expr_info *c, INTTYPE t)
+__inline static	TOKEN		tokenize_integer (expr_info *c, intmax_t t)
 {
 	if (c->token >= TOKENCOUNT)
 	{
-		error("Too many tokens for this expression");
+		my_error("Too many tokens for this expression");
 		return -1;
 	}
 	TOK(c, c->token).used = USED_INTEGER;
@@ -545,7 +537,7 @@ __inline static	TOKEN		tokenize_float (expr_info *c, double t)
 {
 	if (c->token >= TOKENCOUNT)
 	{
-		error("Too many tokens for this expression");
+		my_error("Too many tokens for this expression");
 		return -1;
 	}
 	TOK(c, c->token).used = USED_FLOAT;
@@ -562,7 +554,7 @@ __inline static	TOKEN		tokenize_bool (expr_info *c, BooL t)
 {
 	if (c->token >= TOKENCOUNT)
 	{
-		error("Too many tokens for this expression");
+		my_error("Too many tokens for this expression");
 		return -1;
 	}
 	TOK(c, c->token).used = USED_BOOLEAN;
@@ -605,7 +597,7 @@ __inline static	const char *	get_token_raw (expr_info *c, TOKEN v)
 
 	if (v < 0 || v >= c->token)
 	{
-		error("Token index [%d] is out of range", v);
+		my_error("Token index [%d] is out of range", v);
 		return get_token_raw(c, 0);	/* The empty token */
 	}
 
@@ -614,17 +606,13 @@ __inline static	const char *	get_token_raw (expr_info *c, TOKEN v)
 
 	if (c->noeval)
 		return get_token_raw(c, 0);
-#if 0
-		panic("c->noeval is not valid here. [1]");
-#endif
-
 
 	if ((TOK(c, v).used & USED_RAW) == 0)
 	{
 		TOK(c, v).used |= USED_RAW;
 
 		if (TOK(c, v).used & USED_EXPANDED)
-			panic("Cannot convert EXPANDED token to RAW");
+			panic(1, "Cannot convert EXPANDED token to RAW");
 		else if (TOK(c, v).used & USED_FLOAT)
 		{
 			TOK(c, v).raw_value = 
@@ -633,10 +621,12 @@ __inline static	const char *	get_token_raw (expr_info *c, TOKEN v)
 		}
 		else if (TOK(c, v).used & USED_INTEGER)
 			TOK(c, v).raw_value = 
-				INT2STR(TOK(c, v).integer_value);
+				malloc_sprintf(NULL, INTMAX_FORMAT, 
+						TOK(c, v).integer_value);
 		else if (TOK(c, v).used & USED_BOOLEAN)
 			TOK(c, v).raw_value = 
-				malloc_sprintf(NULL, "%d", TOK(c, v).boolean_value);
+				malloc_sprintf(NULL, "%d",
+						TOK(c, v).boolean_value);
 		else if (TOK(c, v).used & USED_LVAL)
 		{
 			if (x_debug & DEBUG_NEW_MATH_DEBUG)
@@ -644,14 +634,14 @@ __inline static	const char *	get_token_raw (expr_info *c, TOKEN v)
 					v, TOK(c, v).lval);
 
 			TOK(c, v).raw_value = expand_alias(TOK(c, v).lval, 
-					c->args, c->args_flag, NULL);
+								c->args);
 
 			if (x_debug & DEBUG_NEW_MATH_DEBUG)
 				yell(">>> Expanded var name [%d]: [%s] to [%s]",
 					v, TOK(c, v).lval, TOK(c, v).raw_value);
 		}
 		else
-			panic("Can't convert this token to raw format");
+			panic(1, "Can't convert this token to raw format");
 	}
 
 	return TOK(c, v).raw_value;
@@ -671,30 +661,24 @@ __inline static const char *	get_token_lval (expr_info *c, TOKEN v)
 
 	if (v < 0 || v >= c->token)
 	{
-		error("Token index [%d] is out of range", v);
+		my_error("Token index [%d] is out of range", v);
 		return NULL;			/* No lvalue here! */
 	}
 
 	if (v == 0)
 		return NULL;		/* Suppress the operation entirely */
 	else if (c->noeval)
-	{
 		return NULL;		/* Suppress the operation entirely */
-#if 0
-		return get_token_raw(c, 0);
-		panic("c->noeval is not valid here. [0]");
-#endif
-	}
 	else if (((TOK(c, v).used & USED_LVAL) == 0))
 	{
-		error("Token [%d] is not an lvalue", v);
+		my_error("Token [%d] (\"%s\") is not an lvalue", 
+				v, get_token_expanded(c, v));
 		return NULL;			/* No lvalue here! */
 	}
 	else
 		return get_token_raw(c, v);
 }
 
-
 /*
  * Get the "expanded" representation of the token.  The first time you
  * call this function, it gets the "raw" value, if there is one, and then
@@ -708,7 +692,7 @@ __inline static	const char *	get_token_expanded (expr_info *c, TOKEN v)
 
 	if (v < 0 || v >= c->token)
 	{
-		error("Token index [%d] is out of range", v);
+		my_error("Token index [%d] is out of range", v);
 		return get_token_expanded(c, 0);	/* The empty token */
 	}
 
@@ -717,9 +701,6 @@ __inline static	const char *	get_token_expanded (expr_info *c, TOKEN v)
 
 	if (c->noeval)
 		return get_token_raw(c, 0);
-#if 0
-		panic("c->noeval is not valid here. [2]");
-#endif
 
 	if (x_debug & DEBUG_NEW_MATH_DEBUG)
 		yell(">>> Getting token [%d] now.", v);
@@ -745,7 +726,7 @@ __inline static	const char *	get_token_expanded (expr_info *c, TOKEN v)
 					v, myval);
 
 			alias_special_char(&buffer, myval, c->args, 
-					NULL, c->args_flag);
+					NULL);
 			if (!buffer)
 				buffer = malloc_strdup(empty_string);
 			TOK(c, v).expanded_value = buffer;
@@ -767,16 +748,14 @@ __inline static	const char *	get_token_expanded (expr_info *c, TOKEN v)
 					v, myval);
 
 			TOK(c, v).used |= USED_EXPANDED;
-			TOK(c, v).expanded_value = 
-				expand_alias(myval, c->args, 
-						c->args_flag, NULL);
+			TOK(c, v).expanded_value = expand_alias(myval, c->args);
 
 			if (x_debug & DEBUG_NEW_MATH_DEBUG)
 				yell("<<< Expanded token [%d]: [%s] to: [%s]", 
 					v, myval, TOK(c, v).expanded_value);
 		}
 		else
-			panic("Cannot convert from this token to EXPANDED");
+			panic(1, "Cannot convert from this token to EXPANDED");
 	}
 
 	if (x_debug & DEBUG_NEW_MATH_DEBUG)
@@ -784,19 +763,45 @@ __inline static	const char *	get_token_expanded (expr_info *c, TOKEN v)
 	return TOK(c, v).expanded_value;
 }
 
+/*
+ * This is seriously funky.  In the old math parser, function calls could
+ * be made upon any rvalue (ick), and CrazyEddy asked me nicely to implement
+ * this backwards compatability, and how could I ever say no to him? :-D
+ */
+__inline static const char *	get_token_fname (expr_info *c, TOKEN v)
+{
+	if (v == MAGIC_TOKEN)	/* Magic token */
+		return c->args;			/* XXX Probably wrong */
+
+	if (v < 0 || v >= c->token)
+	{
+		my_error("Token index [%d] is out of range", v);
+		return NULL;			/* No lvalue here! */
+	}
+
+	if (v == 0)
+		return NULL;		/* Suppress the operation entirely */
+	else if (c->noeval)
+		return NULL;		/* Suppress the operation entirely */
+	else if (TOK(c, v).used & USED_LVAL)
+		return get_token_raw(c, v);
+	else
+		return get_token_expanded(c, v);
+}
+
 /*
  * Get the integer representation of the token.  The first time you call
  * this function, it calls atof() on the "raw" value, if there is one, 
  * to get the result.
  */
-__inline static	INTTYPE	get_token_integer (expr_info *c, TOKEN v)
+__inline static	intmax_t	get_token_integer (expr_info *c, TOKEN v)
 {
 	if (v == MAGIC_TOKEN)	/* Magic token */
 		return STR2INT(c->args);		/* XXX Probably wrong */
 
 	if (v < 0 || v >= c->token)
 	{
-		error("Token index [%d] is out of range", v);
+		my_error("Token index [%d] is out of range", v);
 		return 0;		/* The empty token */
 	}
 	
@@ -820,7 +825,7 @@ __inline static	double	get_token_float (expr_info *c, TOKEN v)
 
 	if (v < 0 || v >= c->token)
 	{
-		error("Token index [%d] is out of range", v);
+		my_error("Token index [%d] is out of range", v);
 		return 0.0;		/* The empty token */
 	}
 	
@@ -844,7 +849,7 @@ __inline static	BooL	get_token_boolean (expr_info *c, TOKEN v)
 
 	if (v < 0 || v >= c->token)
 	{
-		error("Token index [%d] is out of range", v);
+		my_error("Token index [%d] is out of range", v);
 		return 0;		/* The empty token */
 	}
 	
@@ -870,7 +875,7 @@ __inline static	TOKEN	push_token (expr_info *c, TOKEN t)
 {
 	if (c->sp == STACKSZ - 1)
 	{
-		error("Expressions may not have more than %d operands",
+		my_error("Expressions may not have more than %d operands",
 			STACKSZ);
 		return -1;
 	}
@@ -893,7 +898,7 @@ __inline static	TOKEN	push_float (expr_info *c, double val)
 	return push_token(c, tokenize_float(c, val));
 }
 
-__inline static	TOKEN	push_integer (expr_info *c, INTTYPE val)
+__inline static	TOKEN	push_integer (expr_info *c, intmax_t val)
 { 
 	return push_token(c, tokenize_integer(c, val));
 }
@@ -914,7 +919,7 @@ __inline static TOKEN	top (expr_info *c)
 {
 	if (c->sp < 0)
 	{
-		error("No operands.");
+		my_error("No operands.");
 		return -1;
 	}
 	else
@@ -930,7 +935,7 @@ __inline static	TOKEN	pop_token (expr_info *c)
 		 * Yeilds empty values.  Thats probably the most reasonable
 		 * course of action.
 		 */
-		error("Cannot pop operand: no more operands");
+		my_error("Cannot pop operand: no more operands");
 		return 0;
 	}
 	else
@@ -969,7 +974,7 @@ __inline static	void	pop_2_floats (expr_info *c, double *a, double *b)
 	*a = pop_float(c);
 }
 
-__inline static	void	pop_2_integers (expr_info *c, INTTYPE *a, INTTYPE *b)
+__inline static	void	pop_2_integers (expr_info *c, intmax_t *a, intmax_t *b)
 {
 	*b = pop_integer(c);
 	*a = pop_integer(c);
@@ -1009,7 +1014,7 @@ static void	reduce (expr_info *cx, int what)
 {
 	double	a, b;
 	BooL	c, d;
-	INTTYPE	i, j;
+	intmax_t i, j;
 	const char *s, *t;
 	TOKEN	v, w;
 
@@ -1018,7 +1023,7 @@ static void	reduce (expr_info *cx, int what)
 
 	if (cx->sp < 0) 
 	{
-		error("An operator is missing a required operand");
+		my_error("An operator is missing a required operand");
 		return;
 	}
 
@@ -1054,7 +1059,8 @@ static void	reduce (expr_info *cx, int what)
 		CHECK_NOEVAL						\
 		push_integer(cx, (intop)); 				\
 		if (x_debug & DEBUG_NEW_MATH_DEBUG) 			\
-			yell("O: %s (" FORMAT " " FORMAT ") -> " FORMAT, \
+			yell("O: %s (" INTMAX_FORMAT " " INTMAX_FORMAT	\
+					 ") -> " INTMAX_FORMAT, 	\
 				#intop, i, j, intop); 			\
 		break; 							\
 	}
@@ -1081,7 +1087,7 @@ static void	reduce (expr_info *cx, int what)
 			if (x_debug & DEBUG_NEW_MATH_DEBUG) 		\
 				yell("O: %s (%f %f) -> []", 		\
 					#floatop, a, b); 		\
-			error("Division by zero"); 			\
+			my_error("Division by zero"); 			\
 			push_token(cx, 0);				\
 		} 							\
 		else 							\
@@ -1102,15 +1108,17 @@ static void	reduce (expr_info *cx, int what)
 		if (j == 0) 						\
 		{ 							\
 			if (x_debug & DEBUG_NEW_MATH_DEBUG) 		\
-				yell("O: %s (" FORMAT " " FORMAT ") -> []", \
+				yell("O: %s (" INTMAX_FORMAT " " INTMAX_FORMAT \
+						") -> []", 		\
 					#intop, i, j); 			\
-			error("Division by zero"); 			\
+			my_error("Division by zero"); 			\
 			push_token(cx, 0);				\
 		} 							\
 		else 							\
 		{ 							\
 			if (x_debug & DEBUG_NEW_MATH_DEBUG) 		\
-			    yell("O: %s (" FORMAT " " FORMAT ") -> " FORMAT, \
+			    yell("O: %s (" INTMAX_FORMAT " " INTMAX_FORMAT \
+						") -> " INTMAX_FORMAT,  \
 					#intop, i, j, intop); 		\
 			push_integer(cx, (intop)); 			\
 		} 							\
@@ -1136,7 +1144,8 @@ static void	reduce (expr_info *cx, int what)
 		j = get_token_integer(cx, w);				\
 									\
 		if (x_debug & DEBUG_NEW_MATH_DEBUG) 			\
-			yell("O: %s = %s (" FORMAT " " FORMAT ") -> " FORMAT, \
+			yell("O: %s = %s (" INTMAX_FORMAT " " INTMAX_FORMAT \
+					") -> " INTMAX_FORMAT, 		\
 				s, #intop, i, j, intop); 		\
 									\
 		w = tokenize_integer(cx, (intop));			\
@@ -1201,7 +1210,7 @@ static void	reduce (expr_info *cx, int what)
 			if (x_debug & DEBUG_NEW_MATH_DEBUG) 		\
 				yell("O: %s = %s (%f %f) -> 0",  	\
 					s, #floatop, a, b); 		\
-			error("Division by zero"); 			\
+			my_error("Division by zero"); 			\
 			add_var_alias(s, empty_string, 0);		\
 			push_token(cx, 0);				\
 			break;						\
@@ -1232,16 +1241,19 @@ static void	reduce (expr_info *cx, int what)
 		if (j == 0) 						\
 		{ 							\
 			if (x_debug & DEBUG_NEW_MATH_DEBUG) 		\
-				yell("O: %s = %s (" FORMAT " " FORMAT ") -> 0",\
+				yell("O: %s = %s (" INTMAX_FORMAT " " 	\
+						INTMAX_FORMAT ") -> 0",	\
 					s, #intop, i, j); 		\
-			error("Division by zero"); 			\
+			my_error("Division by zero"); 			\
 			add_var_alias(s, empty_string, 0);		\
 			push_token(cx, 0);				\
 			break;						\
 		} 							\
 									\
 		if (x_debug & DEBUG_NEW_MATH_DEBUG) 			\
-			yell("O: %s =  %s (" FORMAT " "  FORMAT ") -> " FORMAT,\
+			yell("O: %s =  %s (" INTMAX_FORMAT " "  	\
+						INTMAX_FORMAT ") -> " 	\
+						INTMAX_FORMAT,		\
 				s, #intop, i, j, intop); 		\
 									\
 		w = tokenize_float(cx, (intop));			\
@@ -1268,7 +1280,8 @@ static void	reduce (expr_info *cx, int what)
 									\
 		j = get_token_integer(cx, v);				\
 		if (x_debug & DEBUG_NEW_MATH_DEBUG) 			\
-			yell("O: %s (%s " FORMAT ") -> " FORMAT, 	\
+			yell("O: %s (%s " INTMAX_FORMAT ") -> " 	\
+					INTMAX_FORMAT, 			\
 				#intop_result, s, j, (intop_result));	\
 									\
 		w = tokenize_integer(cx, (intop_assign));		\
@@ -1327,7 +1340,8 @@ static void	reduce (expr_info *cx, int what)
 			i = pop_integer(cx);
 			CHECK_NOEVAL
 			if (x_debug & DEBUG_NEW_MATH_DEBUG)
-				yell(": ~" FORMAT " -> " FORMAT, i, ~i);
+				yell(": ~" INTMAX_FORMAT " -> "
+					   INTMAX_FORMAT, i, ~i);
 			push_integer(cx, ~i);
 			break;
 		case UPLUS:
@@ -1349,15 +1363,15 @@ static void	reduce (expr_info *cx, int what)
 			CHECK_NOEVAL
 			i = strlen(s);
 			if (x_debug & DEBUG_NEW_MATH_DEBUG)
-				yell("O: @(%s) -> " FORMAT, s, i);
+				yell("O: @(%s) -> " INTMAX_FORMAT, s, i);
 			push_integer(cx, i);
 			break;
 		case WORDC:
 			s = pop_expanded(cx);
 			CHECK_NOEVAL
-			i = count_words(s, DWORD_YES, "\"");
+			i = count_words(s, DWORD_EXTRACTW, "\"");
 			if (x_debug & DEBUG_NEW_MATH_DEBUG)
-				yell("O: #(%s) -> " FORMAT, s, i);
+				yell("O: #(%s) -> " INTMAX_FORMAT, s, i);
 			push_integer(cx, i);
 			break;
 		case DEREF:
@@ -1371,12 +1385,25 @@ static void	reduce (expr_info *cx, int what)
 			 * to ensure this defeats auto-append.  Ick.
 			 */
 			s = pop_expanded(cx);
-			*cx->args_flag = 1;
 
 			CHECK_NOEVAL
 			push_lval(cx, s);
 			break;
 		}
+		case REEXP:
+		{
+			const char *val;
+
+			s = pop_expanded(cx);
+			if (cx->noeval)
+				cx->last_token = 0;
+			else
+				cx->last_token = tokenize_raw(cx, s);
+
+			push_token(cx, cx->last_token);
+			break;
+		}
+
 
 		/* (pre|post)(in|de)crement operators. */
 		case PREPLUS:   AUTO_UNARY(j + 1, j + 1)
@@ -1508,7 +1535,8 @@ static void	reduce (expr_info *cx, int what)
 		}
 
 		/* Comparison operators */
-		case DEQ:
+		case DEQ:	COMPARE(a == b,  my_stricmp(s, t) == 0)
+#if 0
 		{
 			pop_2_strings(cx, &s, &t);
 			CHECK_NOEVAL
@@ -1520,7 +1548,9 @@ static void	reduce (expr_info *cx, int what)
 			push_boolean(cx, c);
 			break;
 		}
-		case NEQ:
+#endif
+		case NEQ:	COMPARE(a != b,  my_stricmp(s, t) != 0)
+#if 0
 		{
 			pop_2_strings(cx, &s, &t);
 			CHECK_NOEVAL
@@ -1532,6 +1562,32 @@ static void	reduce (expr_info *cx, int what)
 			push_boolean(cx, c);
 			break;
 		}
+#endif
+		case LITEQ:
+		{
+			pop_2_strings(cx, &s, &t);
+			CHECK_NOEVAL
+			c = strcmp(s, t) ? 0 : 1;
+
+			if (x_debug & DEBUG_NEW_MATH_DEBUG)
+				yell("O: %s === %s -> %d", s, t, c);
+
+			push_boolean(cx, c);
+			break;
+		}
+		case NOTLITEQ:
+		{
+			pop_2_strings(cx, &s, &t);
+			CHECK_NOEVAL
+			c = strcmp(s, t) ? 1 : 0;
+
+			if (x_debug & DEBUG_NEW_MATH_DEBUG)
+				yell("O: %s !== %s -> %d", s, t, c);
+
+			push_boolean(cx, c);
+			break;
+		}
+
 		case MATCH:
 		{
 			pop_2_strings(cx, &s, &t);
@@ -1586,17 +1642,29 @@ static void	reduce (expr_info *cx, int what)
 			pop_2_tokens(cx, &v, &w);
 			CHECK_NOEVAL
 
+			/* 
+			 * XXX We need to EXPAND the left value, even
+			 * if we don't need it, because it might be a 
+			 * function call!
+			 */
+			get_token_expanded(cx, v);
+
 			if (x_debug & DEBUG_NEW_MATH_DEBUG)
 				yell("O: %s , %s -> %s", 
 					get_token_expanded(cx, v),
 					get_token_expanded(cx, w),
 					get_token_expanded(cx, w));
+
+			/*
+			 * But we DONT want to expand the right value, 
+			 * until we use it LATER.
+			 */
 			push_token(cx, w);
 			break;
 		}
 
 		default:
-			error("Unknown operator or out of operators");
+			my_error("Unknown operator or out of operators");
 			return;
 	}
 }
@@ -1614,7 +1682,7 @@ static int	lexerr (expr_info *c, const char *format, ...)
 	vsnprintf(buffer, BIG_BUFFER_SIZE, format, a);
 	va_end(a);
 
-	error("%s", buffer);
+	my_error("%s", buffer);
 	c->errflag = 1;
 	return EOI;
 }
@@ -1632,7 +1700,6 @@ static __inline int	check_implied_arg (expr_info *c)
 	{
 		push_token(c, MAGIC_TOKEN);	/* XXXX Bleh */
 		c->operand = 0;
-		*c->args_flag = 1;
 		return 0;
 	}
 
@@ -1680,8 +1747,43 @@ static int	zzlex (expr_info *c)
 	    switch (*(c->ptr++)) 
 	    {
 		case '(':
-			c->operand = 1;
-			return M_INPAR;
+		{
+			char *p = c->ptr;
+			char oc = 0;
+			ssize_t	span;
+
+			/*
+			 * If we see an open paren, and we're expecting an
+			 * operator, then this is a function call.  If we 
+			 * are expecting an operand, then it is a paren-set.
+			 */
+			if (c->operand == 0)
+			{
+			    if ((span = MatchingBracket(p, '(', ')')) >= 0)
+			    {
+				c->ptr = p + span;
+				oc = *c->ptr;
+				*c->ptr = 0;
+			    }
+			    else
+				c->ptr = endstr(c->ptr);
+
+			    if (c->noeval)
+				c->last_token = 0;
+			    else
+				c->last_token = tokenize_raw(c, p);
+
+			    if (oc)
+				*c->ptr++ = oc;
+			    c->operand = 0;
+			    return M_FUNCTION;
+			}
+			else
+			{
+				c->operand = 1;
+				return M_INPAR;
+			}
+		}
 		case ')':
 			/*
 			 * If we get a close paren and the lexer is expecting
@@ -1740,6 +1842,11 @@ static int	zzlex (expr_info *c)
 				c->ptr++;
 				if (*c->ptr == '=') 
 					OPERATOR("**=", 1, POWEREQ)
+				else if (c->operand)
+				{
+					dummy = 2;
+					UNARY("**", 0, REEXP)
+				}
 				else
 					OPERATOR("**", 0, POWER)
 			}
@@ -1770,8 +1877,14 @@ static int	zzlex (expr_info *c)
 
 		case '!':
 		{
-			if (*c->ptr == '=')
-				OPERATOR("!=", 1, NEQ)
+			if (*c->ptr == '=') 
+			{
+				c->ptr++;
+				if (*c->ptr == '=')
+					OPERATOR("!==", 1, NOTLITEQ)
+				else
+					OPERATOR("!=", 0, NEQ)
+			}
 			else if (*c->ptr == '~')
 				OPERATOR("!~", 1, NOMATCH)
 			else
@@ -1892,7 +2005,13 @@ static int	zzlex (expr_info *c)
 
 		case '=':
 			if (*c->ptr == '=') 
-				OPERATOR("==", 1, DEQ)
+			{
+				c->ptr++;
+				if (*c->ptr == '=')
+					OPERATOR("===", 1, LITEQ)
+				else
+					OPERATOR("==", 0, DEQ)
+			}
 			else if (*c->ptr == '~')
 				OPERATOR("=~", 1, MATCH)
 			else
@@ -1952,7 +2071,7 @@ static int	zzlex (expr_info *c)
 				*c->ptr = 0;
 			}
 			else
-				c->ptr = empty_string;
+				c->ptr = endstr(c->ptr);
 
 			c->last_token = 0;
 			if (!c->noeval)
@@ -1996,7 +2115,7 @@ static int	zzlex (expr_info *c)
 				*c->ptr = 0;
 			}
 			else
-				c->ptr = empty_string;
+				c->ptr = endstr(c->ptr);
 
 			if (c->noeval)
 				c->last_token = 0;
@@ -2008,9 +2127,82 @@ static int	zzlex (expr_info *c)
 			c->operand = 0;
 			return ID;
 		}
-		case ' ':
-		case '\t':
-		case '\n':
+
+		/*
+		 * This is a DOUBLE QUOTED (unexpanded-string) operand type.
+		 * Extract the inside of "..." and tokenize that as a "raw"
+		 * value.  It will be expanded later as needed.
+		 */
+		case '"':
+		{
+			char *p = c->ptr;
+			char oc = 0;
+			ssize_t	span;
+
+			if (!c->operand)
+				return lexerr(c, "Misplaced \" token");
+
+			if ((span = MatchingBracket(p, '\0', '"')) >= 0)
+			{
+				c->ptr = p + span;
+				oc = *c->ptr;
+				*c->ptr = 0;
+			}
+			else
+				c->ptr = endstr(c->ptr);
+
+			if (c->noeval)
+				c->last_token = 0;
+			else
+				c->last_token = tokenize_raw(c, p);
+
+			if (oc)
+				*c->ptr++ = oc;
+			c->operand = 0;
+			return ID;
+		}
+
+		/*
+		 * This is a SINGLE QUOTED (expanded-string) operand type.
+		 * Extract the inside of '...' and tokenize that as an
+		 * "expanded" value.  It will be expanded later as needed.
+		 */
+		case '\'':
+		{
+			char *p = c->ptr;
+			char oc = 0;
+			ssize_t	span;
+
+			if (!c->operand)
+				return lexerr(c, "Misplaced ' token");
+
+			if ((span = MatchingBracket(p, '\0', '\'')) >= 0)
+			{
+				c->ptr = p + span;
+				oc = *c->ptr;
+				*c->ptr = 0;
+			}
+			else
+				c->ptr = endstr(c->ptr);
+
+			if (c->noeval)
+				c->last_token = 0;
+			else
+			{
+			    char *ick = NULL;
+			    malloc_strcat_ues(&ick, p, "'");
+			    c->last_token = tokenize_expanded(c, ick);
+			    new_free(&ick);
+			}
+
+			if (oc)
+				*c->ptr++ = oc;
+			c->operand = 0;
+			return ID;
+		}
+
+		/* The "Space" characters. */
+		case 9: case 10: case 11: case 12: case 13: case ' ':
 			start++;
 			break;
 
@@ -2105,7 +2297,7 @@ handle_expando:
 			else
 			{
 				c->last_token = 0; /* Empty token */
-				c->ptr = empty_string;
+				c->ptr = endstr(c->ptr);
 			}
 
 			if (x_debug & DEBUG_NEW_MATH_DEBUG)
@@ -2166,6 +2358,7 @@ static void	mathparse (expr_info *c, int pc)
 		 * Getting the value is done on an as-needed basis.
 		 */
 		case ID:
+		{
 			if (x_debug & DEBUG_NEW_MATH_DEBUG)
 				yell("Parsed identifier token [%s]", 
 					get_token_expanded(c, c->last_token));
@@ -2178,6 +2371,46 @@ static void	mathparse (expr_info *c, int pc)
 			 */
 			push_token(c, c->last_token);
 			break;
+		}
+
+		/*
+		 * For a function call, the argument list is in c->last_token
+		 * and c->mtok is FUNCTION.  The function's name is on the top
+		 * of the queue.  XXX This is a hack because this is treated 
+		 * as a special case, and not as an operator.  But I don't
+		 * really care.  Maybe I'll "fix" this later. XXX
+		 */
+		case M_FUNCTION:
+		{
+			const char *funcname, *args;
+			char *hack = NULL;
+			size_t clue = 0;
+			char *retval;
+			int	func_token;
+
+			func_token = pop_token(c);
+			funcname = get_token_fname(c, func_token);
+			args = get_token_raw(c, c->last_token);
+
+			if (c->noeval)
+				push_token(c, 0);
+			else
+			{
+			    malloc_strcpy_c(&hack, funcname, &clue);
+			    malloc_strcat_c(&hack, "(", &clue);
+			    malloc_strcat_c(&hack, args, &clue);
+			    malloc_strcat_c(&hack, ")", &clue);
+
+			    if (x_debug & DEBUG_NEW_MATH_DEBUG)
+				yell("Parsed function call %s", hack);
+
+			    retval = call_function(hack, c->args);
+			    c->last_token = tokenize_expanded(c, retval);
+
+			    push_token(c, c->last_token);
+			}
+			break;
+		}
 
 		/*
 		 * An open-parenthesis indicates that we should
@@ -2196,7 +2429,7 @@ static void	mathparse (expr_info *c, int pc)
 			if (c->mtok != M_OUTPAR) 
 			{
 				if (!c->errflag)
-				    error("')' expected");
+				    my_error("')' expected");
 				return;
 			}
 			break;
@@ -2303,7 +2536,7 @@ static void	mathparse (expr_info *c, int pc)
  * of state information stored in global variables.  Therefore, this math
  * parser is re-entrant safe.
  */
-static char *	matheval (char *s, const char *args, int *args_flag)
+static char *	matheval (char *s, const char *args)
 {
 	expr_info	context;
 	char *		ret = NULL;
@@ -2316,7 +2549,7 @@ static char *	matheval (char *s, const char *args, int *args_flag)
 	setup_expr_info(&context);
 	context.ptr = s;
 	context.args = args;
-	context.args_flag = args_flag;
+	context.orig_expr = LOCAL_COPY(s);
 
 	/* Actually do the parsing */
 	mathparse(&context, TOPPREC);
@@ -2330,14 +2563,16 @@ static char *	matheval (char *s, const char *args, int *args_flag)
 
 	/* Check for leftover operands */
 	if (context.sp)
-		error("The expression has too many operands");
+	{
+		my_error("This expression failed because it is missing operators: %s", context.orig_expr);
+	}
 
 	if (x_debug & DEBUG_NEW_MATH_DEBUG)
 	{
 		int i;
-		yell("Terms left: %d", context.sp);
+		yell("Operands left: %d", context.sp + 1);
 		for (i = 0; i <= context.sp; i++)
-			yell("Term [%d]: [%s]", i, 
+			yell("Operand [%d]: [%s]", i, 
 				get_token_expanded(&context, context.stack[i]));
 	}
 
