@@ -1,5 +1,6 @@
+/* $EPIC: glob.c,v 1.16 2012/11/24 01:42:51 jnelson Exp $ */
 #include "config.h"
-#if defined(NEED_GLOB)
+#ifdef NEED_GLOB
 
 /*
  * Copyright (c) 1989, 1993
@@ -34,7 +35,6 @@
  */
 
 /* from: static char sccsid[] = "@(#)glob.c	8.3 (Berkeley) 10/13/93"; */
-/* $Id: glob.c,v 1.1.1.1 2000/12/05 00:11:57 jnelson Exp $ */
 
 /*
  * glob(3) -- a superset of the one defined in POSIX 1003.2.
@@ -63,7 +63,9 @@
  *	Number of matches in the current invocation of glob.
  */
 
+#ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
+#endif
 #include <sys/stat.h>
 #include <ctype.h>
 #include <dirent.h>
@@ -75,8 +77,17 @@
 #include <string.h>
 #include <unistd.h>
 #include "irc.h"
-#ifndef MAXPATHLEN
-#define MAXPATHLEN PATHSIZE
+#include "compat.h"
+#ifndef PATH_MAX
+# ifndef MAXPATHLEN
+#  ifndef PATHSIZE
+#   define PATH_MAX 1024
+#  else
+#   define PATH_MAX PATHSIZE
+#  endif
+# else
+#  define PATH_MAX MAXPATHLEN
+# endif
 #endif
 
 #undef EOS
@@ -104,7 +115,10 @@
 #define	M_MASK		0xffff
 #define	M_ASCII		0x00ff
 
-typedef u_short Char;
+#ifdef Char
+#undef Char
+#endif
+typedef unsigned short Char;
 
 #define	CHAR(c)		((Char)((c)&M_ASCII))
 #define	META(c)		((Char)((c)|M_QUOTE))
@@ -119,10 +133,10 @@ typedef u_short Char;
 
 static int	 	compare (const void *, const void *);
 static void	 	g_Ctoc (const Char *, char *);
-static int	 	g_lstat (Char *, struct stat *, glob_t *);
+static int	 	g_lstat (Char *, Stat *, glob_t *);
 static DIR	*	g_opendir (Char *, glob_t *);
-static Char	*	g_strchr (Char *, int);
-static int	 	g_stat (Char *, struct stat *, glob_t *);
+static ssize_t		g_strchr (const Char *, int);
+static int	 	g_stat (Char *, Stat *, glob_t *);
 static int	 	glob0 (const Char *, glob_t *);
 static int	 	glob1 (Char *, glob_t *);
 static int	 	glob2 (Char *, Char *, Char *, glob_t *);
@@ -138,11 +152,11 @@ int bsd_glob		(	const char *pattern,
 				int (*errfunc) (const char *, int),
 				glob_t *pglob				)
 {
-	const u_char *patnext;
+	const unsigned char *patnext;
 	int c;
-	Char *bufnext, *bufend, patbuf[MAXPATHLEN+1];
+	Char *bufnext, *bufend, patbuf[PATH_MAX+1];
 
-	patnext = (u_char *) pattern;
+	patnext = (const unsigned char *) pattern;
 	if (!(flags & GLOB_APPEND)) 
 	{
 		pglob->gl_pathc = 0;
@@ -155,7 +169,7 @@ int bsd_glob		(	const char *pattern,
 	pglob->gl_matchc = 0;
 
 	bufnext = patbuf;
-	bufend = bufnext + MAXPATHLEN;
+	bufend = bufnext + PATH_MAX;
 	if (flags & GLOB_QUOTE) 
 	{
 		/* Protect the quoted characters. */
@@ -196,14 +210,18 @@ static int globexp1	(	const Char *pattern,
 {
 	const Char* ptr = pattern;
 	int rv;
+	ssize_t	span;
 
 	/* Protect a single {}, for find(1), like csh */
 	if (pattern[0] == LBRACE && pattern[1] == RBRACE && pattern[2] == EOS)
 		return glob0(pattern, pglob);
 
-	while ((ptr = (const Char *) g_strchr((Char *) ptr, LBRACE)) != NULL)
+	while ((span = g_strchr(ptr, LBRACE)) >= 0)
+	{
+		ptr += span;
 		if (!globexp2(ptr, pattern, pglob, &rv))
 			return rv;
+	}
 
 	return glob0(pattern, pglob);
 }
@@ -222,7 +240,11 @@ static int globexp2	(	const Char *ptr,
 	int     i;
 	Char   *lm, *ls;
 	const Char *pe, *pm, *pl;
-	Char    patbuf[MAXPATHLEN + 1];
+	Char    patbuf[PATH_MAX + 1];
+
+	/* There are bugs in here that I can't get to the bottom of */
+	/* This just papers over a string overrun */
+	memset(patbuf, 0, PATH_MAX + 1);
 
 	/* copy part up to the brace */
 	for (lm = patbuf, pm = pattern; pm != ptr; *lm++ = *pm++)
@@ -397,7 +419,7 @@ static int glob0		(	const Char *pattern,
 	int		err;
 	int		oldpathc;
 	Char *		bufnext;
-	Char		patbuf[MAXPATHLEN+1];
+	Char		patbuf[PATH_MAX+1];
 
 	qpatnext = globtilde(pattern, patbuf, pglob);
 	oldpathc = pglob->gl_pathc;
@@ -413,7 +435,7 @@ static int glob0		(	const Char *pattern,
 			if (c == NOT)
 				++qpatnext;
 			if (*qpatnext == EOS || 
-			    g_strchr((Char *) qpatnext+1, RBRACKET) == NULL) 
+			    g_strchr(qpatnext+1, RBRACKET) < 0)
 			{
 				*bufnext++ = LBRACKET;
 				if (c == NOT)
@@ -435,7 +457,7 @@ static int glob0		(	const Char *pattern,
 					qpatnext += 2;
 				}
 			} 
-			while ((c = *qpatnext++) != RBRACKET);
+			while ((c = *qpatnext++), (c && c != RBRACKET));
 
 			pglob->gl_flags |= GLOB_MAGCHAR;
 			*bufnext++ = M_END;
@@ -486,13 +508,13 @@ static int glob0		(	const Char *pattern,
 static int compare		(	const void *p,
 					const void *q		)
 {
-	return (strcmp(*(char **)p, *(char **)q));
+	return (strcmp(*(const char * const *)p, *(const char * const *)q));
 }
 
 static int glob1		(	Char *pattern,
 					glob_t *pglob		)
 {
-	Char pathbuf[MAXPATHLEN+1];
+	Char pathbuf[PATH_MAX+1];
 
 	/* A null pathname is invalid -- POSIX 1003.1 sect. 2.4. */
 	if (*pattern == EOS)
@@ -510,7 +532,7 @@ static int glob2		(	Char *pathbuf,
 					Char *pattern,
 					glob_t *pglob		)
 {
-	struct stat sb;
+	Stat sb;
 	Char *p, *q;
 	int anymeta;
 
@@ -573,7 +595,7 @@ static int glob3		(	Char *pathbuf,
 	register struct dirent *dp;
 	DIR *dirp;
 	int err;
-	char buf[MAXPATHLEN];
+	char buf[PATH_MAX];
 
 	/*
 	 * The readdirfunc declaration can't be prototyped, because it is
@@ -581,12 +603,12 @@ static int glob3		(	Char *pathbuf,
 	 * and dirent.h as taking pointers to differently typed opaque
 	 * structures.
 	 */
-	struct dirent *(*readdirfunc)();
+	struct dirent *(*readdirfunc)(DIR *);
 
 	*pathend = EOS;
 	errno = 0;
-	    
-	if ((dirp = g_opendir(pathbuf, pglob)) == NULL) 
+
+	if ((dirp = g_opendir(pathbuf, pglob)) == NULL)
 	{
 		/* TODO: don't call for ENOENT or ENOTDIR? */
 		if (pglob->gl_errfunc) 
@@ -603,20 +625,20 @@ static int glob3		(	Char *pathbuf,
 
 	/* Search directory for matching names. */
 	if (pglob->gl_flags & GLOB_ALTDIRFUNC)
-		readdirfunc = pglob->gl_readdir;
+		readdirfunc = (struct dirent *(*)(DIR *))pglob->gl_readdir;
 	else
 		readdirfunc = readdir;
 
 	while ((dp = (*readdirfunc)(dirp))) 
 	{
-		register u_char *sc;
+		register unsigned char *sc;
 		register Char *dc;
 		int	nocase = 0;
 
 		/* Initial DOT must be matched literally. */
 		if (dp->d_name[0] == DOT && *pattern != DOT)
 			continue;
-		for (sc = (u_char *) dp->d_name, dc = pathend; 
+		for (sc = (unsigned char *) dp->d_name, dc = pathend; 
 		     (*dc++ = *sc++) != EOS;)
 			continue;
 
@@ -784,10 +806,10 @@ void bsd_globfree 		(	glob_t *pglob			)
 static DIR *g_opendir		(	register Char *str,
 					glob_t *pglob			)
 {
-	char buf[MAXPATHLEN];
+	char buf[PATH_MAX];
 
 	if (!*str)
-		strcpy(buf, ".");
+		strlcpy(buf, ".", sizeof buf);
 	else
 		g_Ctoc(str, buf);
 
@@ -798,10 +820,10 @@ static DIR *g_opendir		(	register Char *str,
 }
 
 static int g_lstat		(	register Char *fn,
-					struct stat *sb,
+					Stat *sb,
 					glob_t *pglob			)
 {
-	char buf[MAXPATHLEN];
+	char buf[PATH_MAX];
 
 	g_Ctoc(fn, buf);
 	if (pglob->gl_flags & GLOB_ALTDIRFUNC)
@@ -810,10 +832,10 @@ static int g_lstat		(	register Char *fn,
 }
 
 static int g_stat		(	register Char *fn,
-					struct stat *sb,
+					Stat *sb,
 					glob_t *pglob			)
 {
-	char buf[MAXPATHLEN];
+	char buf[PATH_MAX];
 
 	g_Ctoc(fn, buf);
 	if (pglob->gl_flags & GLOB_ALTDIRFUNC)
@@ -821,14 +843,15 @@ static int g_stat		(	register Char *fn,
 	return(stat(buf, sb));
 }
 
-static Char *g_strchr		(	Char *str, 
+static ssize_t	g_strchr	(	const Char *str, 
 					int ch				)
 {
+	const Char *start = str;
 	do {
 		if (*str == ch)
-			return (str);
+			return str - start;
 	} while (*str++);
-	return (NULL);
+	return -1;
 }
 
 static void g_Ctoc		(	register const Char *str,
