@@ -1,4 +1,4 @@
-/* $EPIC: keys.c,v 1.59 2010/01/08 05:15:03 jnelson Exp $ */
+/* $EPIC: keys.c,v 1.61 2012/11/26 05:09:35 jnelson Exp $ */
 /*
  * keys.c:  Keeps track of what happens whe you press a key.
  *
@@ -342,7 +342,7 @@ void key_exec_bt (struct Key *key) {
 	 * have any luck, simply discard the key and continue plugging
 	 * forward. */
 	if (nstr == (kstr + kslen)) {
-	    if (kp->bound != NULL || kslen == 1) {
+	    if (kp && (kp->bound != NULL || kslen == 1)) {
 		if (kp->bound != NULL)
 		    key_exec(kp);
 		len -= (nstr - kstr);
@@ -955,7 +955,7 @@ struct BindStack {
 static struct BindStack *bind_stack = NULL;
 void do_stack_bind (int type, char *arg) {
     struct Key *key = NULL;
-    struct Key *map = head_keymap;
+    struct Key *map;
     struct BindStack *bsp = NULL;
     struct BindStack *bsptmp = NULL;
     unsigned char *cs, *s;
