@@ -1,4 +1,4 @@
-/* $EPIC: log.c,v 1.14 2004/02/20 23:40:23 jnelson Exp $ */
+/* $EPIC: log.c,v 1.26 2009/12/28 20:05:54 jnelson Exp $ */
 /*
  * log.c: handles the irc session logging functions 
  *
@@ -35,61 +35,88 @@
  */
 
 #include "irc.h"
+#include "levels.h"
 #include "log.h"
 #include "vars.h"
 #include "output.h"
 #include "ircaux.h"
 #include "alias.h"
+#include "screen.h"
 
 	FILE	*irclog_fp;
 	int	logfile_line_mangler;
+	int	current_log_refnum = -1;
 
-
-static FILE *open_log (const char *logfile, FILE **fp)
+static FILE *	open_log (const char *logfile, FILE **fp)
 {
-	time_t	t;
-	char	my_buffer[256];
-	struct	tm	*ugh;
-	Filename fullname;
+	char *		tempname;
+	size_t		templen;
+	Filename 	fullname;
 
-	time(&t);
-	ugh = localtime(&t);		/* Not gmtime, m'kay? */
-	/* Ugh.  Solaris. */
-	strftime(my_buffer, 255, "%a %b %d %H:%M:%S %Y", ugh);
+	/* If the user hasn't specified a logfile, do nothing */
+	if (!logfile)
+		return NULL;
 
+	/* If the user is already logging here, tell them. */
 	if (*fp) 
 	{
 		say("Logging is already on");
 		return *fp;
 	}
 
-	if (!logfile)
-		return NULL;
-			
-	if (normalize_filename(logfile, fullname))
+	/* If necessary, remove "s on the outside of the filename */
+	/* I do this way too much on accident; it's annoying not to have it */
+	tempname = LOCAL_COPY(logfile);
+	templen = strlen(tempname);
+	if (templen > 2 && tempname[0] == '"' 
+			&& tempname[templen - 1] == '"')
 	{
-		say("SET LOGFILE: %s contains an invalid directory", logfile);
+		tempname[templen - 1] = 0;
+		tempname++;
+	}
+
+	/* If the result is an empty filename then just punt */
+	if (!tempname || !*tempname)
+	{
+		yell("Cannot log to the filename [%s] because the result "
+			"was an empty filename", logfile);
 		return NULL;
 	}
 
+	if (normalize_filename(tempname, fullname))
+	{
+		yell("Warning: I could not normalize the filename [%s] "
+			"(the result was [%s] -- watch out", 
+			logfile, fullname);
+		(void)0;		/* Do nothing... */
+	}
+
 	if ((*fp = fopen(fullname, "a")) != NULL)
 	{
+		time_t		t;
+		struct	tm *	ltime;
+		char		timestr[256];
+
+		/* Convert the time to a string to insert in the file */
+		time(&t);
+		ltime = localtime(&t);		/* Not gmtime, m'kay? */
+		strftime(timestr, 255, "%a %b %d %H:%M:%S %Y", ltime);
+
 		chmod(fullname, S_IREAD | S_IWRITE);
 		say("Starting logfile %s", fullname);
-
-		fprintf(*fp, "IRC log started %s\n", my_buffer);
+		fprintf(*fp, "IRC log started %s\n", timestr);
 		fflush(*fp);
 	}
 	else
 	{
-		say("Couldn't open logfile %s: %s", fullname, strerror(errno));
+		yell("Couldn't open logfile %s: %s", fullname, strerror(errno));
 		*fp = NULL;
 	}
 
 	return (*fp);
 }
 
-static FILE *close_log (FILE **fp)
+static FILE *	close_log (FILE **fp)
 {
 	time_t	t;
 	char	my_buffer[256];
@@ -112,7 +139,16 @@ struct	tm	*ugh;
 	return (*fp);
 }
 
-FILE *do_log (int flag, const char *logfile, FILE **fp)
+/*
+ * do_log: open or close a logfile
+ *	flag 	- 1 if the logfile should be opened
+ *		  0 if the logfile should be closed
+ *	logfile - The name of the logfile (only needed if flag == 1)
+ *	fp	- A pointer to a (FILE *) which gets the result of fopen().
+ *		  Will be set to NULL if the file is not open (due to 
+ *		  error or flag == 0)
+ */
+FILE *	do_log (int flag, const char *logfile, FILE **fp)
 {
 	if (flag)
 		return open_log(logfile, fp);
@@ -121,91 +157,52 @@ FILE *do_log (int flag, const char *logfile, FILE **fp)
 }
 
 /* logger: if flag is 0, logging is turned off, else it's turned on */
-void	logger (const void *stuff)
+void	logger (void *stuff)
 {
-	int	flag = *(const int *)stuff;
-	char	*logfile;
+	VARIABLE *v;
+	int	flag;
+	char *	logfile;
+
+	v = (VARIABLE *)stuff;
+	flag = v->integer;
 
 	if ((logfile = get_string_var(LOGFILE_VAR)) == (char *) 0)
 	{
 		say("You must set the LOGFILE variable first!");
-		set_int_var(LOG_VAR, 0);
+		v->integer = 0;
 		return;
 	}
 	do_log(flag, logfile, &irclog_fp);
 	if (!irclog_fp && flag)
-		set_int_var(LOG_VAR, 0);
-}
-
-/*
- * set_log_file: sets the log file name.  If logging is on already, this
- * closes the last log file and reopens it with the new name.  This is called
- * automatically when you SET LOGFILE. 
- */
-void	set_log_file (const void *stuff)
-{
-	const char *filename = (const char *)stuff;
-	Filename expand;
-
-	if (!filename)
-		return;
-
-	if (normalize_filename(filename, expand))
-	{
-		say("SET LOGFILE: %s contains an invalid directory", filename);
-		return;
-	}
-
-	set_string_var(LOGFILE_VAR, expand);
-	if (irclog_fp)
-	{
-		int	value;
-
-		value = 0;
-		logger(&value);
-		value = 1;
-		logger(&value);
-	}
+		v->integer = 0;
 }
 
 /*
  * add_to_log: add the given line to the log file.  If no log file is open
  * this function does nothing. 
+ * (Note:  "logref" should be -1 unless we are logging from a /log log, ie,
+ * any place that is not inside logfiles.c)
  */
-void 	add_to_log (FILE *fp, unsigned winref, const unsigned char *line, int mangler, const char *rewriter)
+void 	add_to_log (int logref, FILE *fp, long winref, const unsigned char *line, int mangler, const char *rewriter)
 {
-	char	*local_line;
-	size_t	size;
-	int	must_free = 0;
+	char	*local_line = NULL;
+	int	old_logref;
 
 	if (!fp || inhibit_logging)
 		return;
 
-	/*
-	 * We need to make a local copy because 'mangle_line' 
-	 * diddles around with the source, and so we can't subject
-	 * line to that, it is 'const'.
-	 *
-	 * 'mangle_line' can expand the input string, so it is 
-	 * neccesary to allocate more than we need.
-	 */
-	size = (strlen(line) + 1) * 11;
-	local_line = alloca(size + 1);
-	strlcpy(local_line, line, size + 1);
+	old_logref = current_log_refnum;
+	current_log_refnum = logref;
 
 	/* Do this first */
 	if (mangler == 0)
 		mangler = logfile_line_mangler;
-	if (mangler)
-	   if (mangle_line(local_line, mangler, size) > size)
-		(void)0; /* Whimper -- what to do, what to do? */
-
 	if (get_int_var(NO_CONTROL_LOG_VAR))
-	{
-		char *tmp = alloca(strlen(local_line) + 1);
-		strip_control(local_line, tmp);
-		strlcpy(local_line, tmp, size);
-	}
+		mangler |= STRIP_UNPRINTABLE;
+	if (mangler)
+		local_line = new_normalize_string(line, 1, mangler);
+	else
+		local_line = malloc_strdup(line);
 
 	if (rewriter == NULL)
 		rewriter = get_string_var(LOG_REWRITE_VAR);
@@ -213,22 +210,20 @@ void 	add_to_log (FILE *fp, unsigned winref, const unsigned char *line, int mang
 	{
 		char    *prepend_exp;
 		char    argstuff[10240];
-		int     args_flag;
 
 		/* First, create the $* list for the expando */
-		snprintf(argstuff, 10240, "%u %s", winref, local_line);
+		snprintf(argstuff, 10240, "%ld %s", winref, local_line);
+		new_free(&local_line);
 
 		/* Now expand the expando with the above $* */
-		prepend_exp = expand_alias(rewriter, argstuff,
-					   &args_flag, NULL);
-
+		prepend_exp = expand_alias(rewriter, argstuff);
 		local_line = prepend_exp;
-		must_free = 1;
 	}
 
 	fprintf(fp, "%s\n", local_line);
 	fflush(fp);
 
-	if (must_free)
-		new_free(&local_line);
+	new_free(&local_line);
+	current_log_refnum = old_logref;
 }
+
