@@ -1,8 +1,8 @@
-/* $EPIC: network.c,v 1.51 2003/12/13 17:25:58 jnelson Exp $ */
+/* $EPIC: network.c,v 1.86 2012/06/11 02:39:30 jnelson Exp $ */
 /*
  * network.c -- handles stuff dealing with connecting and name resolving
  *
- * Copyright  1995, 2003 Jeremy Nelson
+ * Copyright  1995, 2005 Jeremy Nelson
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -38,20 +38,20 @@
 #include "output.h"
 #include <sys/ioctl.h>
 
+/* This will eventually become a configurable */
+#ifndef NO_JOB_CONTROL
+#define ASYNC_DNS
+#endif
+
 #ifdef HAVE_SYS_UN_H
 #include <sys/un.h>
 typedef struct sockaddr_un USA;
 #endif
 
-static int	set_non_blocking (int fd);
-static int	set_blocking (int fd);
-static int	inet_remotesockaddr (int family, const char *host, const char *port, SS *storage, socklen_t *len);
-int	inet_vhostsockaddr (int family, int port, SS *storage, socklen_t *len);
-static int	Connect (int fd, SA *addr);
-static int	Getaddrinfo (const char *nodename, const char *servname, const AI *hints, AI **res);
-static void	Freeaddrinfo (AI *ai);
-static socklen_t	socklen (SA *sockaddr);
-static int	Getnameinfo(const SA *sa, socklen_t salen, char *host, size_t hostlen, char *serv, size_t servlen, int flags);
+static int	Connect 	 (int, SA *);
+static socklen_t socklen  	 (SA *);
+static int	Getnameinfo 	 (const SA *, socklen_t, char *, size_t, char *, size_t, int);
+static int	Socket 		(int, int, int);
 
 /*
    Retval    Meaning
@@ -73,63 +73,6 @@ static int	Getnameinfo(const SA *sa, socklen_t salen, char *host, size_t hostlen
 */
 
 /*****************************************************************************/
-/*
- * NAME: connectory
- * USAGE: Connect to a given "host" and "port" with the given "family"
- * ARGS: family - AF_INET is the only supported argument.
- *       host - A hostname or "dotted quad" (or equivalent).
- *       port - The remote port to connect to in *HOST ORDER*.
- *
- * XXX - This so violates everything I really wanted this function to be,
- * but I changed it to call getaddrinfo() directly instead of calling
- * inet_vhostsockaddr() because I wanted it to be able to take advantage
- * of connecting to multiple protocols and multiple ip addresses (for things
- * like ``us.undernet.org'') without having to do multiple calls to
- * getaddrinfo() which could be quite costly.
- */
-int	connectory (int family, const char *host, const char *port)
-{
-	AI	hints, *results, *ai;
-	int	err;
-	int	fd;
-	SS	  localaddr;
-	socklen_t locallen;
-
-	memset(&hints, 0, sizeof(hints));
-	hints.ai_family = family;
-	hints.ai_socktype = SOCK_STREAM;
-	if ((err = Getaddrinfo(host, port, &hints, &results)))
-	{
-		yell("Hostname lookup for [%s:%s] failed: %s (%d)", host, port, gai_strerror(err), err);
-		return -5;
-	}
-
-	fd = -1;
-	for (ai = results; ai; ai = ai->ai_next) 
-	{
-	    /* First, look up the virtual host we use for this protocol. */
-	    err = inet_vhostsockaddr(ai->ai_family, -1, &localaddr, &locallen);
-	    if (err < 0)
-		continue;
-
-	    /* Now try to do the connection. */
-	    fd = client_connect((SA *)&localaddr, locallen, ai->ai_addr, ai->ai_addrlen);
-	    if (fd < 0)
-	    {
-		err = fd;
-		fd = -1;
-		continue;
-	    }
-	    else
-		break;
-	}
-
-	Freeaddrinfo(results);
-	if (fd < 0)
-		return err;
-	return fd;
-}
-
 /*
  * NAME: client_connect
  * USAGE: Create a new socket and establish both endpoints with the 
@@ -147,9 +90,6 @@ int	client_connect (SA *l, socklen_t ll, SA *r, socklen_t rl)
 {
 	int	fd = -1;
 	int	family = AF_UNSPEC;
-	int	err;
-	fd_set	set;
-	Timeval	to;
 
 	if (ll == 0)
 		l = NULL;
@@ -157,90 +97,61 @@ int	client_connect (SA *l, socklen_t ll, SA *r, socklen_t rl)
 		r = NULL;
 
 	if (!r)
-		return -7;
+	{
+		syserr(-1, "client_connect: remote addr missing (connect to who?)");
+		return -1;
+	}
 
 	if (l && r && l->sa_family != r->sa_family)
-		return -8;
+	{
+		syserr(-1, "client_connect: local addr protocol (%d) is different "
+			"from remote addr protocol (%d)", 
+			l->sa_family, r->sa_family);
+		return -1;
+	}
 
 	if (l)
 		family = l->sa_family;
 	if (r)
 		family = r->sa_family;
 
-	if ((fd = socket(family, SOCK_STREAM, 0)) < 0)
-		return -1;
-
-	set_socket_options(fd);
-
-	/* Unix domain server */
-	if (family == AF_UNIX)
+	if ((fd = Socket(family, SOCK_STREAM, 0)) < 0)
 	{
-		alarm(get_int_var(CONNECT_TIMEOUT_VAR));
-		if (Connect(fd, r) < 0)
-		{
-			alarm(0);
-			return close(fd), -1;
-		}
-		alarm(0);
+		syserr(-1, "client_connect: socket(%d) failed: %s", 
+						family, strerror(errno));
+		return -1;
 	}
-	else if (family == AF_INET 
+
+	if (family == AF_UNIX || family == AF_INET 
 #ifdef INET6
-				   || family == AF_INET6
+			      || family == AF_INET6
 #endif
 							 )
 	{
 		if (l && bind(fd, l, ll))
 		{
-		    if (errno == EADDRINUSE || errno == EADDRNOTAVAIL)
-			return close(fd), -10;
-		    else
-			return close(fd), -1;
+		    syserr(-1, "client_connect: bind(%d) failed: %s", 
+						fd, strerror(errno));
+		    close(fd);
+		    return -1;
 		}
 
-		/*
-		 * Simulate a blocking connect, using a nonblocking connect.
-		 * Due to logic flow issues, we cannot just start a non-
-		 * blocking connect and return.  Bad things happen.  But 
-		 * what we can do is do a nonblocking connect and then wait
-		 * here until it is finished.  This lets us graciously 
-		 * trap ^C's from the user without resorting to alarm()s.
-		 */
-		set_non_blocking(fd);
-
-		/* Star the connect.  If any bad error occurs, punt. */
-		errno = 0;
-		err = Connect(fd, r);
-		if (err < 0 && errno != EAGAIN && errno != EINPROGRESS)
-			return close(fd), -11;
-
-		/* Now stall for a while until it succeeds or times out. */
-		errno = 0;
-		FD_ZERO(&set);
-		FD_SET(fd, &set);
-		to.tv_sec = get_int_var(CONNECT_TIMEOUT_VAR);
-		to.tv_usec = 0;
-		switch (select(fd + 1, NULL, &set, NULL, &to))
+		if (Connect(fd, r))
 		{
-			case 0:
-				errno = ECONNABORTED;
-				/* FALLTHROUGH */
-			case -1:
-				return close(fd), -9;
-			default:
-			{
-				SS	peer;
-				int	peerlen;
-
-				peerlen = sizeof(peer);
-				if (getpeername(fd, (SA *)&peer, &peerlen))
-					return close(fd), -12;
-				set_blocking(fd);
-			}
+		    syserr(-1, "client_connect: connect(%d) failed: %s", 
+						fd, strerror(errno));
+		    close(fd);
+		    return -1;
 		}
 	}
 	else
-		return -2;
+	{
+	    syserr(-1, "client_connect: protocol (%d) not supported", family);
+	    return -1;
+	}
 
+	if (x_debug & DEBUG_SERVER_CONNECT)
+		yell("Connected on des [%d]", fd);
 	return fd;
 }
 
@@ -265,16 +176,30 @@ int	client_connect (SA *l, socklen_t ll, SA *r, socklen_t rl)
  */
 int	ip_bindery (int family, unsigned short port, SS *storage)
 {
-	int	err;
 	socklen_t len;
+	int	fd;
 
-	if ((err = inet_vhostsockaddr(family, port, storage, &len)))
-		return err;
+	if (inet_vhostsockaddr(family, port, NULL, storage, &len))
+	{
+		syserr(-1, "ip_bindery: inet_vhostsockaddr(%d,%d) failed.", 
+							family, port);
+		return -1;
+	}
 
 	if (!len)
-		return -6;
+	{
+		syserr(-1, "ip_bindery: inet_vhostsockaddr(%d,%d) didn't "
+			"return an address I could bind", family, port);
+		return -1;
+	}
 
-	return client_bind((SA *)storage, len);
+	if ((fd = client_bind((SA *)storage, len)) < 0)
+	{
+		syserr(-1, "ip_bindery: client_bind(%d,%d) failed.", family, port);
+		return -1;
+	}
+
+	return fd;
 }
 
 /*
@@ -294,14 +219,20 @@ int	client_bind (SA *local, socklen_t local_len)
 	if (local_len == 0)
 		local = NULL;
 	if (!local)
-		return -13;
+	{
+		syserr(-1, "client_bind: address to bind to not provided");
+		return -1;
+	}
 
 	family = local->sa_family;
 
-	if ((fd = socket(family, SOCK_STREAM, 0)) < 0)
+	if ((fd = Socket(family, SOCK_STREAM, 0)) < 0)
+	{
+		syserr(-1, "client_bind: socket(%d) failed: %s", 
+					family, strerror(errno));
 		return -1;
+	}
 
-	set_socket_options (fd);
 #ifdef IP_PORTRANGE
 	/*
 	 * On 4.4BSD systems, this socket option asks the
@@ -319,26 +250,32 @@ int	client_bind (SA *local, socklen_t local_len)
 
 	if (bind(fd, local, local_len))
 	{
-	    if (errno == EADDRINUSE || errno == EADDRNOTAVAIL)
-		return close(fd), -10;
-	    else
-		return close(fd), -1;
+		syserr(-1, "client_bind: bind(%d) failed: %s", 
+					fd, strerror(errno));
+		close(fd);
+		return -1;
 	}
 
-	if (family != AF_UNIX)
+	/*
+	 * Get the local sockaddr of the passive socket,
+	 * specifically the port number, and stash it in
+	 * 'port' which is the return value to the caller.
+	 */	
+	if (getsockname(fd, (SA *)local, &local_len))
 	{
-		/*
-		 * Get the local sockaddr of the passive socket,
-		 * specifically the port number, and stash it in
-		 * 'port' which is the return value to the caller.
-		 */	
-		if (getsockname(fd, (SA *)local, &local_len))
-			return close(fd), -1;
+		syserr(-1, "client_bind: getsockname(%d) failed: %s", 
+					fd, strerror(errno));
+		close(fd);
+		return -1;
 	}
 
-
 	if (listen(fd, 4) < 0)
-		return close(fd), -1;
+	{
+		syserr(-1, "client_bind: listen(%d,4) failed: %s",
+					fd, strerror(errno));
+		close(fd);
+		return -1;
+	}
 
 	return fd;
 }
@@ -354,12 +291,13 @@ int	client_bind (SA *local, socklen_t local_len)
  *             the given family.
  * NOTES: If "len" is set to 0, do not attempt to bind() 'storage'!
  */
-int	inet_vhostsockaddr (int family, int port, SS *storage, socklen_t *len)
+int	inet_vhostsockaddr (int family, int port, const char *wanthost, SS *storage, socklen_t *len)
 {
 	char	p_port[12];
 	char	*p = NULL;
 	AI	hints, *res;
 	int	err;
+	const char *lhn;
 
 	/*
 	 * If port == -1, then this is a client connection, so we punt
@@ -368,12 +306,28 @@ int	inet_vhostsockaddr (int family, int port, SS *storage, socklen_t *len)
 	 * so we need to use LocalHostName, even if it's NULL.  If you 
 	 * return *len == 0 for port != -1, then /dcc breaks.
 	 */
-	if (family == AF_UNIX || (port == -1 && !LocalHostName))
+	if ((family == AF_UNIX) || 
+            (family == AF_INET && port == -1 && LocalIPv4HostName == NULL) 
+#ifdef INET6
+	 || (family == AF_INET6 && port == -1 && LocalIPv6HostName == NULL)
+#endif
+									   )
 	{
 		*len = 0;
 		return 0;		/* No vhost needed */
 	}
 
+	if (wanthost && *wanthost)
+		lhn = wanthost;
+	else if (family == AF_INET)
+		lhn = LocalIPv4HostName;
+#ifdef INET6
+	else if (family == AF_INET6)
+		lhn = LocalIPv6HostName;
+#endif
+	else
+		lhn = NULL;
+
 	/*
 	 * Can it really be this simple?
 	 */
@@ -387,37 +341,19 @@ int	inet_vhostsockaddr (int family, int port, SS *storage, socklen_t *len)
 		p = p_port;
 	}
 
-	if ((err = Getaddrinfo(LocalHostName, p, &hints, &res)))
-		return -10;
+	if ((err = my_getaddrinfo(lhn, p, &hints, &res)))
+	{
+		syserr(-1, "inet_vhostsockaddr: my_getaddrinfo(%s,%s) failed: %s", 
+				lhn, p, gai_strerror(err));
+		return -1;
+	}
+
 	memcpy(storage, res->ai_addr, res->ai_addrlen);
+	my_freeaddrinfo(res);
 	*len = socklen((SA *)storage);
 	return 0;
 }
 
-/*
- * NAME: inet_remotesockaddr
- * USAGE: Get a sockaddr of the specified host and port.
- * ARGS: family - The family whose sockaddr info is to be retrieved
- *       host - The host whose address shall be put in the sockaddr
- *       port - The port to put into the sockaddr -- MUST BE IN HOST ORDER!
- *       storage - Pointer to a sockaddr structure appropriate for family.
- */
-static int	inet_remotesockaddr (int family, const char *host, const char *port, SS *storage, socklen_t *len)
-{
-	int	err;
-
-	((SA *)storage)->sa_family = family;
-
-	if ((err = inet_strton(host, port, (SA *)storage, 0)))
-		return err;
-
-	if ((*len = socklen((SA *)storage)) == 0)
-		return -2;
-
-	return 0;
-}
-
-
 /************************************************************************/
 /*
  * NAME: inet_strton
@@ -461,19 +397,19 @@ int	inet_strton (const char *host, const char *port, SA *storage, int flags)
 		hints.ai_socktype = SOCK_STREAM;
 		hints.ai_protocol = 0;
 
-		if ((retval = Getaddrinfo(host, port, &hints, &results))) {
-		    yell("getaddrinfo(%s): %s", host, gai_strerror(retval));
-		    return -5;
+		if ((retval = my_getaddrinfo(host, port, &hints, &results))) 
+		{
+			syserr(-1, "inet_strton: my_getaddrinfo(%s,%s) failed: %s", 
+					host, port, gai_strerror(retval));
+			return -1;
 		}
 
 		/* memcpy can bite me. */
 		memcpy(storage, results->ai_addr, results->ai_addrlen);
 
-		Freeaddrinfo(results);
+		my_freeaddrinfo(results);
 		return 0;
 	}
-
-	return -2;
 }
 
 /*
@@ -495,15 +431,19 @@ int	inet_ntostr (SA *name, char *host, int hsize, char *port, int psize, int fla
 	socklen_t len;
 
 	len = socklen(name);
-	if ((retval = Getnameinfo(name, len, host, hsize, port, psize, flags | NI_NUMERICSERV))) {
-		yell("Getnameinfo (sockaddr->p_addr): %s", gai_strerror(retval));
-		return retval;
+	if ((retval = Getnameinfo(name, len, host, hsize, port, psize, 
+					flags | NI_NUMERICSERV))) 
+	{
+		syserr(-1, "inet_ntostr: Getnameinfo(sockaddr->p_addr) failed: %s", 
+					gai_strerror(retval));
+		return -1;
 	}
 
 	return 0;
 }
 
 /* * * * * * * * * */
+/* Only used by $convert() and $nametoip() */
 /*
  * NAME: inet_hntop
  * USAGE: Convert a Hostname into a "presentation address" (p-addr)
@@ -515,21 +455,27 @@ int	inet_ntostr (SA *name, char *host, int hsize, char *port, int psize, int fla
  * RETURN VALUE: "retval" is returned upon success
  *		 "empty_string" is returned for any error.
  */
-char *	inet_hntop (int family, const char *host, char *retval, int size)
+int	inet_hntop (int family, const char *host, char *retval, int size)
 {
-	int	err;
 	SS	buffer;
 
 	((SA *)&buffer)->sa_family = family;
-	if ((err = inet_strton(host, NULL, (SA *)&buffer, 0)))
-		return empty_string;
+	if (inet_strton(host, NULL, (SA *)&buffer, AI_ADDRCONFIG))
+	{
+		syserr(-1, "inet_hntop: inet_strton(%d,%s) failed", family, host);
+		return -1;
+	}
 
 	if (inet_ntostr((SA *)&buffer, retval, size, NULL, 0, NI_NUMERICHOST))
-		return empty_string;
+	{
+		syserr(-1, "inet_hntop: inet_ntostr(%d,%s) failed", family, host);
+		return -1;
+	}
 
-	return retval;
+	return 0;
 }
 
+/* Only used by $convert() and $iptoname() */
 /*
  * NAME: inet_ptohn
  * USAGE: Convert a "presentation address" (p-addr) into a Hostname
@@ -541,21 +487,27 @@ char *	inet_hntop (int family, const char *host, char *retval, int size)
  * RETURN VALUE: "retval" is returned upon success
  *		 "empty_string" is returned for any error.
  */
-char *	inet_ptohn (int family, const char *ip, char *retval, int size)
+int	inet_ptohn (int family, const char *ip, char *retval, int size)
 {
-	int	err;
 	SS	buffer;
 
 	((SA *)&buffer)->sa_family = family;
-	if ((err = inet_strton(ip, NULL, (SA *)&buffer, AI_NUMERICHOST)))
-		return empty_string;
+	if (inet_strton(ip, NULL, (SA *)&buffer, AI_NUMERICHOST))
+	{
+		syserr(-1, "inet_ptohn: inet_strton(%d,%s) failed", family, ip);
+		return -1;
+	}
 
 	if (inet_ntostr((SA *)&buffer, retval, size, NULL, 0, NI_NAMEREQD))
-		return empty_string;
+	{
+		syserr(-1, "inet_ptohn: inet_ntostr(%d,%s) failed", family, ip);
+		return -1;
+	}
 
-	return retval;
+	return 0;
 }
 
+/* Only used by $convert() */
 /*
  * NAME: one_to_another
  * USAGE: Convert a p-addr to a Hostname, or a Hostname to a p-addr.
@@ -572,15 +524,30 @@ char *	inet_ptohn (int family, const char *ip, char *retval, int size)
  * NOTES: If "what" is a p-addr and there is no hostname associated with that 
  *        address, that is considered an error and empty_string is returned.
  */
-char *	one_to_another (int family, const char *what, char *retval, int size)
+int	one_to_another (int family, const char *what, char *retval, int size)
 {
-	if (inet_ptohn(family, what, retval, size) == empty_string)
-		inet_hntop(family, what, retval, size);
-	return retval;
+	/* XXX I wish this wasn't necessary */
+	int	old_window_display = window_display;
+	window_display = 0;
+
+	if (inet_ptohn(family, what, retval, size))
+	{
+		if (inet_hntop(family, what, retval, size))
+		{
+			window_display = old_window_display;
+			syserr(-1, "one_to_another: both inet_ptohn and "
+					"inet_hntop failed (%d,%s)", 
+					family, what);
+			return -1;
+		}
+	}
+
+	window_display = old_window_display;
+	return 0;
 }
 
 /****************************************************************************/
-static int	set_non_blocking (int fd)
+int	set_non_blocking (int fd)
 {
 	int	flag, rval;
 
@@ -596,17 +563,29 @@ static int	set_non_blocking (int fd)
 
 #if defined(O_NONBLOCK) || defined(O_NDELAY)
 	if ((rval = fcntl(fd, F_GETFL, 0)) == -1)
+	{
+		syserr(-1, "set_non_blocking: fcntl(%d, F_GETFL) failed: %s",
+				fd, strerror(errno));
 		return -1;
+	}
 	if (fcntl(fd, F_SETFL, rval | flag) == -1)
+	{
+		syserr(-1, "set_non_blocking: fcntl(%d, F_SETFL) failed: %s",
+				fd, strerror(errno));
 		return -1;
+	}
 #else
 	if (ioctl(fd, FIONBIO, &flag) < 0)
+	{
+		syserr(-1, "set_non_blocking: ioctl(%d, FIONBIO) failed: %s",
+				fd, strerror(errno));
 		return -1;
+	}
 #endif
 	return 0;
 }
 
-static int	set_blocking (int fd)
+int	set_blocking (int fd)
 {
 	int	flag, rval;
 
@@ -622,12 +601,24 @@ static int	set_blocking (int fd)
 
 #if defined(O_NONBLOCK) || defined(O_NDELAY)
 	if ((rval = fcntl(fd, F_GETFL, 0)) == -1)
+	{
+		syserr(-1, "set_blocking: fcntl(%d, F_GETFL) failed: %s",
+				fd, strerror(errno));
 		return -1;
+	}
 	if (fcntl(fd, F_SETFL, rval & ~flag) == -1)
+	{
+		syserr(-1, "set_blocking: fcntl(%d, F_SETFL) failed: %s",
+				fd, strerror(errno));
 		return -1;
+	}
 #else
 	if (ioctl(fd, FIONBIO, &flag) < 0)
+	{
+		syserr(-1, "set_blocking: ioctl(%d, FIONBIO) failed: %s",
+				fd, strerror(errno));
 		return -1;
+	}
 #endif
 	return 0;
 }
@@ -640,26 +631,40 @@ static int	set_blocking (int fd)
  * has been closed in the interim.  This wrapper for accept() attempts to
  * defeat this by making the accept() call nonblocking.
  */
-int	Accept (int s, SA *addr, int *addrlen)
+int	my_accept (int s, SA *addr, socklen_t *addrlen)
 {
 	int	retval;
 
 	set_non_blocking(s);
-	retval = accept(s, addr, addrlen);
+	if ((retval = accept(s, addr, addrlen)) < 0)
+		syserr(-1, "my_accept: accept(%d) failed: %s", 
+				s, strerror(errno));
 	set_blocking(s);
+	set_blocking(retval);		/* Just in case */
 	return retval;
 }
 
 static int Connect (int fd, SA *addr)
 {
-	return connect(fd, addr, socklen(addr));
+	int	retval;
+
+	set_non_blocking(fd);
+	if ((retval = connect(fd, addr, socklen(addr))))
+	{
+	    if (errno != EINPROGRESS)
+		syserr(-1, "Connect: connect(%d) failed: %s", fd, strerror(errno));
+	    else
+		retval = 0;
+	}
+	set_blocking(fd);
+	return retval;
 }
 
 /*
  * XXX - Ugh!  Some getaddrinfo()s take AF_UNIX paths as the 'servname'
  * instead of as the 'nodename'.  How heinous!
  */
-static int	Getaddrinfo (const char *nodename, const char *servname, const AI *hints, AI **res)
+int	my_getaddrinfo (const char *nodename, const char *servname, const AI *hints, AI **res)
 {
 #ifdef GETADDRINFO_DOES_NOT_DO_AF_UNIX
 	int	do_af_unix = 0;
@@ -712,7 +717,7 @@ static int	Getaddrinfo (const char *nodename, const char *servname, const AI *hi
 		return getaddrinfo(nodename, servname, hints, res);
 }
 
-static void	Freeaddrinfo (AI *ai)
+void	my_freeaddrinfo (AI *ai)
 {
 #ifdef GETADDRINFO_DOES_NOT_DO_AF_UNIX
 	if (ai->ai_family == AF_UNIX)
@@ -729,12 +734,51 @@ static void	Freeaddrinfo (AI *ai)
 
 static int	Getnameinfo(const SA *sa, socklen_t salen, char *host, size_t hostlen, char *serv, size_t servlen, int flags)
 {
-	if ((flags & GNI_INTEGER) && sa->sa_family == AF_INET) {
+#ifdef GETADDRINFO_DOES_NOT_DO_AF_UNIX
+	if (sa->sa_family == AF_UNIX) 
+	{
+		socklen_t	len;
+		const USA *	usa = (const USA *)sa;
+
+#ifdef HAVE_SA_LEN
+                len = usa->sun_len;
+#else
+# ifdef SUN_LEN
+		len = SUN_LEN(usa);
+# else
+		len = strlen(usa->sun_path);
+# endif
+#endif
+
+		if (host && hostlen)
+		{
+		    if (hostlen <= len)
+		    {
+			memcpy(host, usa->sun_path, hostlen);
+			host[hostlen - 1] = 0;
+		    }
+		    else
+		    {
+			memcpy(host, &usa->sun_path, len);
+			host[len] = 0;
+		    }
+		}
+
+		if (serv && servlen)
+			*serv = 0;
+
+                return 0;
+	}
+#endif
+
+	if ((flags & GNI_INTEGER) && sa->sa_family == AF_INET) 
+	{
 		snprintf(host, hostlen, "%lu", 
 			(unsigned long)ntohl(((const ISA *)sa)->sin_addr.s_addr));
 		host = NULL;
 		hostlen = 0;
 	}
+
 	flags = flags & ~(GNI_INTEGER);
 	return getnameinfo(sa, salen, host, hostlen, serv, servlen, flags);
 }
@@ -753,3 +797,175 @@ static socklen_t	socklen (SA *sockaddr)
 		return 0;
 }
 
+/* set's socket options */
+int	Socket (int domain, int type, int protocol)
+{
+        int     opt = 1;
+        int     optlen = sizeof(opt);
+	int	s;
+#ifndef NO_STRUCT_LINGER
+        struct linger   lin;
+#endif
+
+	if ((s = socket(domain, type, protocol)) < 0)
+	{
+		syserr(-1, "Socket: socket(%d,%d,%d) failed: %s",
+				domain, type, protocol, strerror(errno));
+		return -1;
+	}
+
+#ifndef NO_STRUCT_LINGER
+        lin.l_onoff = lin.l_linger = 0;
+        setsockopt(s, SOL_SOCKET, SO_LINGER, (char *)&lin, optlen);
+#endif
+
+        setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char *)&opt, optlen);
+        opt = 1;
+        setsockopt(s, SOL_SOCKET, SO_KEEPALIVE, (char *)&opt, optlen);
+	return s;
+}
+
+pid_t	async_getaddrinfo (const char *nodename, const char *servname, const AI *hints, int fd)
+{
+	AI *results = NULL;
+	ssize_t	err;
+
+#ifdef ASYNC_DNS
+	{
+	/* XXX Letting /exec clean up after us is a hack. */
+	pid_t	helper;
+	if ((helper = fork()))
+		return helper;
+	}
+#endif
+
+        if ((err = my_getaddrinfo(nodename, servname, hints, &results)))
+        {
+		err = -abs(err);		/* Always a negative number */
+		write(fd, &err, sizeof(err));
+		close(fd);
+#ifdef ASYNC_DNS
+		exit(0);
+#else
+		return 0;
+#endif
+        }
+
+	if (!results)
+	{
+		err = 0;
+		write(fd, &err, sizeof(err));
+		close(fd);
+#ifdef ASYNC_DNS
+		exit(0);
+#else
+		return 0;
+#endif
+        }
+
+        marshall_getaddrinfo(fd, results);
+        my_freeaddrinfo(results);
+        close(fd);
+#ifdef ASYNC_DNS
+	exit(0);
+#endif
+	return 0;	/* XXX This function should be void */
+}
+
+void	marshall_getaddrinfo (int fd, AI *results)
+{
+	ssize_t	len, gah;
+	ssize_t	alignment = sizeof(void *);
+	AI 	*result, *copy;
+	char 	*retval, *ptr;
+
+	for (len = 0, result = results; result; result = result->ai_next)
+	{
+		len += sizeof(AI);
+		len += result->ai_addrlen;
+		if (result->ai_canonname)
+			len += strlen(result->ai_canonname) + 1;
+		while (len % alignment != 0)
+			len++;
+	}
+
+	/* Why do I know I'm gonna regret this? */
+	ptr = retval = new_malloc(len + 1);
+	memset(retval, 0, len + 1);
+	for (result = results; result; result = result->ai_next)
+	{
+		copy = (AI *)ptr;
+
+		/* Copy over the AI */
+		memcpy(copy, result, sizeof(AI));
+
+		/* Copy over the ai_addr */
+		ptr += sizeof(AI);
+		memcpy(ptr, result->ai_addr, result->ai_addrlen);
+
+		/* Point the AI at the ai_addr */
+		copy->ai_addr = (SA *)ptr;
+
+		/* Copy over the canonname */
+		/* Point the AI at the canonname */
+		ptr += result->ai_addrlen;
+		if (result->ai_canonname)
+		{
+		    gah = strlen(result->ai_canonname) + 1;
+		    memcpy(ptr, result->ai_canonname, gah);
+		    copy->ai_canonname = ptr;
+		    ptr += gah;
+		}
+		else
+		    copy->ai_canonname = NULL;
+
+		 /* Calculate the start of the next entry */
+		 while ((intptr_t)ptr % alignment != 0)
+			ptr++;
+
+		/* Point the AI at the next entry */
+		if (result->ai_next)
+			copy->ai_next = (AI *)ptr;
+		else
+			copy->ai_next = NULL;
+	}
+
+	write(fd, (void *)&len, sizeof(len));
+	write(fd, (void *)retval, len);
+	new_free(&retval);
+}
+
+void	unmarshall_getaddrinfo (AI *results)
+{
+	ssize_t	len;
+	AI 	*result;
+	char 	*ptr;
+	ssize_t	alignment = sizeof(void *);
+
+	/* Why do I know I'm gonna regret this? */
+	for (result = results; result; result = result->ai_next)
+	{
+		ptr = (char *)result;
+
+		/* Copy over the ai_addr */
+		ptr += sizeof(AI);
+		result->ai_addr = (SA *)ptr;
+		ptr += result->ai_addrlen;
+
+		if (result->ai_canonname)
+		{
+		    result->ai_canonname = ptr;
+		    len = strlen(result->ai_canonname) + 1;
+		    ptr += len;
+		}
+
+		 /* Calculate the start of the next entry */
+		 while ((intptr_t)ptr % alignment != 0)
+			ptr++;
+
+		/* Point the AI at the next entry */
+		if (result->ai_next)
+			result->ai_next = (AI *)ptr;
+	}
+}
+
