@@ -1,4 +1,4 @@
-/* $EPIC: numbers.c,v 1.70 2006/12/08 15:47:50 jnelson Exp $ */
+/* $EPIC: numbers.c,v 1.101 2013/02/17 13:02:46 jnelson Exp $ */
 /*
  * numbers.c: handles all those strange numeric response dished out by that
  * wacky, nutty program we call ircd 
@@ -49,11 +49,11 @@
 #include "names.h"
 #include "parse.h"
 #include "commands.h"
-#include "notice.h"
 #include "notify.h"
 #include "vars.h"
 #include "who.h"
 #include "alias.h"
+#include "reg.h"
 
 static void	add_user_who (int refnum, const char *from, const char *comm, const char **ArgList);
 static void	add_user_end (int refnum, const char *from, const char *comm, const char **ArgList);
@@ -68,14 +68,13 @@ const char *	banner (void)
 	static	char	thing[80];
 	char *str;
 
-	if (current_numeric < 0 && get_int_var(SHOW_NUMERICS_VAR))
-		snprintf(thing, sizeof thing, "%3.3u", -current_numeric);
+	if (current_numeric > 0 && get_int_var(SHOW_NUMERICS_VAR))
+		snprintf(thing, sizeof thing, "%3.3u", current_numeric);
 	else if ((str = get_string_var(BANNER_VAR)))
 	{
 		if (get_int_var(BANNER_EXPAND_VAR))
 		{
-			int af;
-			char *foo = expand_alias(str, empty_string, &af, NULL);
+			char *foo = expand_alias(str, empty_string);
 			strlcpy(thing, foo, sizeof thing);
 			new_free(&foo);
 		}
@@ -93,7 +92,7 @@ const char *	banner (void)
  * display_msg: handles the displaying of messages from the variety of
  * possible formats that the irc server spits out.
  *
- * Simplified by Jeremy Nelson (esl) some time in 1996.
+ * Simplified some time in 1996.
  * -- called by more than one place.
  */
 static void 	display_msg (const char *from, const char *comm, const char **ArgList)
@@ -102,6 +101,11 @@ static void 	display_msg (const char *from, const char *comm, const char **ArgLi
 	const char	*rest;
 	int	drem;
 
+	/* XXX - 
+	 * ArgList[0] was passed to who_from() which means we need to 
+	 * either reset who_from() with our own copy, or not detokenize
+	 * ArgList[0], because this breaks channel targeting!
+	 */
 	if (!(rest = PasteArgs(ArgList, 0)))
 		{ rfc1459_odd(from, comm, ArgList); return; }
 
@@ -110,13 +114,12 @@ static void 	display_msg (const char *from, const char *comm, const char **ArgLi
 		from = NULL;
 
 	/* This fix by SrfRog, again by |Rain| */
-	ptr = strchr(rest, ':');
 	if (ptr && ptr > rest && ptr[-1] == ' ')	/* per RFC 1459 */
 		*ptr++ = 0;
 	else
 		ptr = NULL;
 
-        drem = (from) && (!get_int_var(SUPPRESS_FROM_REMOTE_SERVER));
+        drem = (from) && (!get_int_var(SUPPRESS_FROM_REMOTE_SERVER_VAR));
 
         /*
          * This handles all the different cases of server messages.
@@ -173,31 +176,46 @@ static void 	display_msg (const char *from, const char *comm, const char **ArgLi
  */
 void 	numbered_command (const char *from, const char *comm, char const **ArgList)
 {
-	const char	*target;
+	const char	*recipient;
+	char *	target = NULL;
 	char	*copy;
 	int	i;
-	int	lastlog_level;
 	int	old_current_numeric = current_numeric;
 	int	numeric;
+	int	l;
 
-	/* All numerics must have a target (our nickname) */
+	/* All numerics must be in the range (000, 999) */
 	if (!comm || !*comm)
 		{ rfc1459_odd(from, comm, ArgList); return; }
 	numeric = atol(comm);
-	if (numeric < 0 || numeric > 999)
+	if (numeric < 0 || numeric >= FIRST_NAMED_HOOK - 1)
 		{ rfc1459_odd(from, comm, ArgList); return; }
 
-	if (!(target = ArgList[0]))
+	/* All numerics must have a recipient (our nickname) */
+	if (!ArgList[0])
 		{ rfc1459_odd(from, comm, ArgList); return; }
+	recipient = LOCAL_COPY(ArgList[0]);
 	ArgList++;
 
-	lastlog_level = set_lastlog_msg_level(LOG_CRAP);
-	if (ArgList[0] && is_channel(ArgList[0]))
-		message_from(ArgList[0], LOG_CRAP);
+	/* 
+	 * Numerics may have a channel target as 1st argument
+	 *
+	 * We must make a copy of ArgList[0] to pass to message_from
+	 * because display_message (above) will call PasteArgs which
+	 * will destroy ArgList[0].
+	 *
+	 * Please note that we don't consume the ArgList[0] argument,
+	 * we only peek at it to see if we should target a channel
+	 * with message_from().
+	 */
+	if (ArgList[0])
+		target = LOCAL_COPY(ArgList[0]);
+	if (target && is_channel(target))
+		l = message_from(target, LEVEL_OTHER);
 	else
-		message_from(NULL, LOG_CRAP);
+		l = message_from(NULL, LEVEL_OTHER);
 
-	current_numeric = -numeric;	/* must be negative of numeric! */
+	current_numeric = numeric;	/* must be negative of numeric! */
 
 	/*
 	 * This first switch statement is only used for those numerics
@@ -225,15 +243,7 @@ void 	numbered_command (const char *from, const char *comm, char const **ArgList
 	 */
 	case 001:	/* #define RPL_WELCOME          001 */
 	{
-		Timeval i;
-
-                i.tv_sec = 0;
-                i.tv_usec = 50000;
-                select(0, NULL, NULL, NULL, &i);
-
-		accept_server_nickname(from_server, target);
-		server_is_registered(from_server, 1);
-		userhostbase(from_server, NULL, got_my_userhost, 1);
+		server_is_registered(from_server, from, recipient);
 		break;
 	}
 
@@ -249,27 +259,14 @@ void 	numbered_command (const char *from, const char *comm, char const **ArgList
 	case 004:	/* #define RPL_MYINFO           004 */
 	{
 		const char 	*server = NULL, 
-				*version = NULL, 
-				*umodes = NULL;
+				*version = NULL;
 
 		if (!(server = ArgList[0]))
 			{ rfc1459_odd(from, comm, ArgList); goto END; }
 		else if (!(version = ArgList[1]))
 			{ rfc1459_odd(from, comm, ArgList); goto END; }
-		else if (!(umodes = ArgList[2]))
-			{ rfc1459_odd(from, comm, ArgList); goto END; }
 
-		/* Work around ratbox-1.2-3. */
-		if (!my_stricmp(umodes, "(brown"))
-		 if (ArgList[3] && !my_stricmp(ArgList[3], "paper"))
-		  if (ArgList[4] && !my_stricmp(ArgList[4], "bag"))
-		   if (ArgList[5] && !my_stricmp(ArgList[5], "release)"))
-		   {
-			if (!(umodes = ArgList[6]))
-				{ rfc1459_odd(from, comm, ArgList); goto END; }
-		   }
-
-		got_initial_version_28(server, version, umodes);
+		set_server_version_string(from_server, version);
 		break;
 	}
 
@@ -284,14 +281,14 @@ void 	numbered_command (const char *from, const char *comm, char const **ArgList
 			if (value && *value) 
 				*value++ = 0;
 
-			if (*set == '+') /* parameter append */
+			if (*set == '+')	/* Parameter append */
 			{
 				const char *ov = get_server_005(from_server, ++set);
 				value = malloc_strdup2(ov, value);
 				set_server_005(from_server, set, value);
 				new_free(&value);
 			}
-			else if (*set == '-') /* parameter removal */
+			if (*set == '-')	/* Parameter removal */
 				set_server_005(from_server, ++set, NULL);
 			else if (value && *value)
 				set_server_005(from_server, set, value);
@@ -305,6 +302,8 @@ void 	numbered_command (const char *from, const char *comm, char const **ArgList
 	{
 		const char *new_server, *new_port_s, *message;
 		int	new_port, old_server;
+		char *	str = NULL;
+		int	new_servref;
 
 		PasteArgs(ArgList, 2);
 		if (!(new_server = ArgList[0]))
@@ -313,15 +312,16 @@ void 	numbered_command (const char *from, const char *comm, char const **ArgList
 			{ rfc1459_odd(from, comm, ArgList); goto END; }
 		if (!(message = ArgList[2]))
 			{ rfc1459_odd(from, comm, ArgList); goto END; }
-		new_port = atol(ArgList[1]);
+		new_port = (int)atol(ArgList[1]);
 
 		/* Must do these things before calling "display_msg" */
 		old_server = from_server;
-		add_to_server_list(new_server, new_port, NULL, NULL,
-				get_server_group(from_server), NULL, 0);
-		server_reconnects_to(old_server, from_server);
+		malloc_sprintf(&str, "%s:%d:::%s:", new_server, new_port, 
+					get_server_group(from_server));
+		if ((new_servref = str_to_servref(str)) == NOSERV)
+			new_servref = str_to_newserv(str);
+		change_window_server(old_server, new_servref);
 		from_server = old_server;
-
 		break;
 	}
 
@@ -338,22 +338,18 @@ void 	numbered_command (const char *from, const char *comm, char const **ArgList
 		goto END;
 	}
 
-	case 42:		/* ircnet's "unique id" numeric	042 */
+	case 42:		/* IRCNet's "unique id" numeric 042 */
 	{
-		const char *	unique_id;
-		const char *	message;
+		const char *unique_id, *message;
 
 		PasteArgs(ArgList, 1);
 		if (!(unique_id = ArgList[0]))
 			{ rfc1459_odd(from, comm, ArgList); goto END; }
-		else if (!(message = ArgList[1]))
+		if (!(message = ArgList[1]))
 			{ rfc1459_odd(from, comm, ArgList); goto END; }
 
 		set_server_unique_id(from_server, unique_id);
-		if (do_hook(current_numeric, "%s %s %s", 
-					from, unique_id, message))
-			goto DISPLAY;
-		goto END;
+		break;
 	}
 
         case 301:               /* #define RPL_AWAY             301 */
@@ -368,11 +364,23 @@ void 	numbered_command (const char *from, const char *comm, char const **ArgList
 
 		/* Ach.  /on 301 doesn't offer 'from' as $0.  Bummer. */
                 if (do_hook(current_numeric, "%s %s", nick, message))
+		{
+			/* XXXX Hack -- figure out another way */
+			copy = alloca(IRCD_BUFFER_SIZE + 1);
+			*copy = 0;
+
+			for (i = 0; ArgList[i]; i++)
+			{
+				if (i)
+					strlcat(copy, " ", IRCD_BUFFER_SIZE);
+				strlcat(copy, ArgList[i], IRCD_BUFFER_SIZE);
+			}
 			goto DISPLAY;
+		}
 		goto END;
         }
 
-	case 307:		/* #define RPL_USERIP           307 */
+	case 340:		/* #define RPL_USERIP?          340 */
 		if (!get_server_005(from_server, "USERIP"))
 			break;
 		/* FALLTHROUGH */
@@ -427,6 +435,10 @@ void 	numbered_command (const char *from, const char *comm, char const **ArgList
 		funny_max = get_server_funny_max(from_server);
 		funny_match = get_server_funny_match(from_server);
 
+		/* List messages NEVER go to a chanwin */
+		pop_message_from(l);
+		l = message_from(NULL, LEVEL_OTHER);
+
 		/*
 		 * Do not display if the channel has no topic and the user asked
 		 * for only channels with topics.
@@ -489,22 +501,24 @@ void 	numbered_command (const char *from, const char *comm, char const **ArgList
 		    update_channel_mode(channel, mode);
 		    update_all_status();
 
-		    maxnum = get_server_max_cached_chan_size(from_server);
-		    if (maxnum >= 0)
-		    {
-			numonchannel = number_on_channel(copy, from_server);
-			if (numonchannel <= maxnum)
-			    whobase(from_server, copy, add_user_who, 
-						add_user_end);
-			else
-			    channel_not_waiting(copy, from_server);
-		    }
+		    if (is_channel_anonymous(copy, from_server))
+			channel_not_waiting(copy, from_server);
 		    else
-			    whobase(from_server, copy, add_user_who, 
+		    {
+			maxnum = get_server_max_cached_chan_size(from_server);
+			if (maxnum >= 0)
+			{
+			    numonchannel = number_on_channel(copy, from_server);
+			    if (numonchannel <= maxnum)
+				whobase(from_server, copy, add_user_who, 
 						add_user_end);
-#if 0
-		    goto END;
-#endif
+			    else
+				channel_not_waiting(copy, from_server);
+			}
+			else
+			    whobase(from_server, copy, add_user_who, 
+					add_user_end);
+		    }
 		}
 
 		break;
@@ -533,7 +547,7 @@ void 	numbered_command (const char *from, const char *comm, char const **ArgList
 
 		    while ((nick = next_arg(line_copy, &line_copy)) != NULL)
 		    {
-			/* XXX - Hack to work around space at end of 353 */
+			/* XXX Hack to work around space at end of 353 */
 			forcibly_remove_trailing_spaces(nick, NULL);
 
 			/*
@@ -552,7 +566,6 @@ void 	numbered_command (const char *from, const char *comm, char const **ArgList
 								0, 0, 0, 0);
 		    }
 
-		    message_from(channel, LOG_CRAP);
 		    break;
 		}
 		else
@@ -599,21 +612,18 @@ void 	numbered_command (const char *from, const char *comm, char const **ArgList
 		goto END;
 
 	/* XXX Yea yea, these are out of order. so shoot me. */
-	case 346:		/* #define RPL_INVITELIST (+I for erf) */
-	case 348:		/* #define RPL_EXCEPTLIST (+e for erf) */
+	case 346:               /* #define RPL_INVITELIST (+I for erf) */
+	case 348:               /* #define RPL_EXCEPTLIST (+e for erf) */
 	case 367:		/* #define RPL_BANLIST */
 		number_of_bans++;
 		break;
 
-	case 347:		/* #define END_OF_INVITELIST */
-	case 349:		/* #define END_OF_EXCEPTLIST */
+	case 347:               /* #define END_OF_INVITELIST */
+	case 349:               /* #define END_OF_EXCEPTLIST */
 	case 368:		/* #define END_OF_BANLIST */
 	{
 		const char	*channel;
 
-		if (!get_int_var(SHOW_END_OF_MSGS_VAR))
-			goto END;
-
 		if (!(channel = ArgList[0]))
 			{ rfc1459_odd(from, comm, ArgList); goto END; }
 
@@ -627,10 +637,10 @@ void 	numbered_command (const char *from, const char *comm, char const **ArgList
 		{
 			put_it("%s Total number of %s on %s - %d",
 				banner(), 
-				 numeric == 347 ? "invites" :
-				(numeric == 349 ? "exceptions" :
-				(numeric == 368 ? "bans" : "wounds")),
-				channel, number_of_bans);
+                                numeric == 347 ? "invites" :
+                               (numeric == 349 ? "exceptions" :
+                               (numeric == 368 ? "bans" : "wounds")),
+                                channel, number_of_bans);
 		}
 		goto END;
 	}
@@ -650,18 +660,7 @@ void 	numbered_command (const char *from, const char *comm, char const **ArgList
 			{ rfc1459_odd(from, comm, ArgList); goto END; }
 
 		if (!is_channel(nick))
-		{
 		    notify_mark(from_server, nick, 0, 0);
-		    if (get_int_var(AUTO_WHOWAS_VAR))
-		    {
-			int foo = get_int_var(NUM_OF_WHOWAS_VAR);
-
-			if (foo > -1)
-				send_to_server("WHOWAS %s %d", nick, foo);
-			else
-				send_to_server("WHOWAS %s", nick);
-		    }
-		}
 
 		break;
 	}
@@ -749,23 +748,6 @@ void 	numbered_command (const char *from, const char *comm, char const **ArgList
 
 		break;
 	}
-	case 432:		/* #define ERR_ERRONEUSNICKNAME 432 */
-	{
-		const char	*nick;
-
-		if (!(nick = ArgList[0]))
-			{ rfc1459_odd(from, comm, ArgList); goto END; }
-
-		if (!my_stricmp(target, nick))
-			yell("WARNING:  Strange invalid nick message received."
-					"  You are probably lagged.");
-		else if (get_int_var(AUTO_NEW_NICK_VAR))
-			fudge_nickname(from_server);
-		else
-			reset_nickname(from_server);
-
-		break;
-	}
 
 	case 437:		/* av2.9's "Nick collision" numeric 437 */
 				/* Also, undernet/dalnet "You are banned" */
@@ -812,7 +794,7 @@ void 	numbered_command (const char *from, const char *comm, char const **ArgList
 		 */
 		if (ArgList[0] && ArgList[1] == NULL)
 		{
-			accept_server_nickname(from_server, target);
+			accept_server_nickname(from_server, recipient);
 			break;
 		}
 
@@ -831,17 +813,6 @@ void 	numbered_command (const char *from, const char *comm, char const **ArgList
 			break;
 		}
 
-		/*
-		 * Otherwise, a nick command failed.  Oh boy.
-		 * If we are registered, abort the nick change and
-		 * hope for the best.
-		 */
-		if (is_server_registered(from_server))
-		{
-			accept_server_nickname(from_server, target);
-			break;
-		}
-
 		/* 
 		 * Otherwise, it's an ircnet "nick not available" error.
 		 * Let the nickname reset numerics handle this mess.
@@ -849,8 +820,10 @@ void 	numbered_command (const char *from, const char *comm, char const **ArgList
 		/* FALLTHROUGH */
 	}
 
+	case 432:		/* #define ERR_ERRONEUSNICKNAME 432 */
 	case 433:		/* #define ERR_NICKNAMEINUSE    433 */ 
-	case 438:		/* EFnet/TS4 "nick collision" numeric 438 */
+	case 438:		/* Undernet's "Stop changing your nick" */
+	case 439:		/* Comstud's "Can't change nickname" */
 	case 453:		/* EFnet/TS4 "nickname lost" numeric 453 */
 	{
 		const char	*nick;
@@ -858,16 +831,18 @@ void 	numbered_command (const char *from, const char *comm, char const **ArgList
 		if (!(nick = ArgList[0]))
 			{ rfc1459_odd(from, comm, ArgList); goto END; }
 
-		if (!my_stricmp(target, nick))
-			/* 
-			 * This should stop the "rolling nicks" in their tracks.
-			 */
-			yell("WARNING:  Strange invalid nick message received."
-					"  You are probably lagged.");
-		else if (get_int_var(AUTO_NEW_NICK_VAR))
-			fudge_nickname(from_server);
-		else
-			reset_nickname(from_server);
+		/* 
+		 * This is a hack for BBC from inspircd.
+		 * A 439 numeric received before 001 that is sent to the
+		 * nickname we requested is NOT a rejection of that nickname,
+		 * it is an informational message from inspircd.
+		 */
+		if (numeric == 439 && 
+		    !is_server_registered(from_server) &&
+		    !strcmp(recipient, get_pending_nickname(from_server)))
+			break;
+
+		nickname_change_rejected(from_server, recipient);
 
 		if (!from)
 			from = "-1";
@@ -875,12 +850,6 @@ void 	numbered_command (const char *from, const char *comm, char const **ArgList
 		break;
 	}
 
-	case 439:		/* Comstud's "Can't change nickname" */
-	{
-		accept_server_nickname(from_server, target);
-		break;
-	}
-
 	case 442:		/* #define ERR_NOTONCHANNEL	442 */
 	{
 		const char *	s;
@@ -919,17 +888,6 @@ void 	numbered_command (const char *from, const char *comm, char const **ArgList
 		change_server_nickname(from_server, NULL);
 		break;
 
-	case 465:		/* #define ERR_YOUREBANNEDCREEP 465 */
-	{
-		/* 
-		 * There used to be a say() here, but if we arent 
-		 * connected to a server, then doing say() is not
-		 * a good idea.  So now it just doesnt do anything.
-		 */
-		server_reconnects_to(from_server, NOSERV);
-		break;
-	}
-
 	case 477:		/* #define ERR_NEEDREGGEDNICK	477 */
 		/* IRCnet has a different 477 numeric. */
 		if (ArgList[0] && *ArgList[0] == '+')
@@ -973,6 +931,9 @@ void 	numbered_command (const char *from, const char *comm, char const **ArgList
 
 DISPLAY:
 /* DEFAULT DISPLAY */
+	if (!do_hook(NUMERIC_LIST, "%d %s %s", current_numeric, from, copy))
+		goto END;
+
 	/*
 	 * This is the "default display" case, where if the user does not 
 	 * hook the numeric, we output the message in some special way.
@@ -1130,8 +1091,7 @@ DISPLAY:
 	case 318:		/* #define RPL_ENDOFWHOIS       318 */
 	{
 		PasteArgs(ArgList, 0);
-		if (get_int_var(SHOW_END_OF_MSGS_VAR))
-			display_msg(from, comm, ArgList);
+		display_msg(from, comm, ArgList);
 		break;
 	}
 
@@ -1287,6 +1247,19 @@ DISPLAY:
 		break;
 	}
 
+	case 338:		/* #define RPL_WHOISACTUALLY	338 */
+	{
+		const char *who, *host;
+
+		if (!(who = ArgList[0]))
+			{ rfc1459_odd(from, comm, ArgList); goto END; }
+		if (!(host = ArgList[1]))
+			{ rfc1459_odd(from, comm, ArgList); goto END; }
+
+		put_it("%s %s is actually using %s", banner(), who, host);
+		break;
+	}
+
 	case 341:		/* #define RPL_INVITING         341 */
 	{
 		const char *nick, *channel;
@@ -1296,7 +1269,8 @@ DISPLAY:
 		if (!(channel = ArgList[1]))
 			{ rfc1459_odd(from, comm, ArgList); goto END; }
 
-		message_from(channel, LOG_CRAP);
+		pop_message_from(l);
+		l = message_from(channel, LEVEL_OTHER);
 		put_it("%s Inviting %s to channel %s", banner(), nick, channel);
 		break;
 	}
@@ -1308,8 +1282,11 @@ DISPLAY:
 		PasteArgs(ArgList, 2);
 		if (!(version = ArgList[0]))
 			{ rfc1459_odd(from, comm, ArgList); goto END; }
-		if (!(itsname = ArgList[1]))
-			{ rfc1459_odd(from, comm, ArgList); goto END; }
+		if (!(itsname = ArgList[1])) {
+			/* As a favor to larne, for inspired */
+			put_it("%s Server %s", banner(), version);
+			break;
+		}
 		if (!(stuff = ArgList[2]))
 			{ rfc1459_odd(from, comm, ArgList); goto END; }
 
@@ -1335,7 +1312,8 @@ DISPLAY:
 		if (channel_is_syncing(channel, from_server))
 		{
 			/* If the user bites on /ON NAMES, then skip the rest */
-			message_from(channel, LOG_CRAP);
+			pop_message_from(l);
+			l = message_from(channel, LEVEL_OTHER);
 			if (do_hook(NAMES_LIST, "%s %s", channel, line))
 			    if (get_int_var(SHOW_CHANNEL_NAMES_VAR))
 				say("Users on %s: %s",
@@ -1360,7 +1338,8 @@ DISPLAY:
 		else
 			strlcpy(format, "%s: %s\t%s", sizeof format);
 
-		message_from(channel, LOG_CRAP);
+		pop_message_from(l);
+		l = message_from(channel, LEVEL_OTHER);
 		if (*type == '=') 
 		{
 		    if (last_width && ((int)strlen(channel) > last_width))
@@ -1404,9 +1383,6 @@ DISPLAY:
 	{
 		const char	*channel;
 
-		if (!get_int_var(SHOW_END_OF_MSGS_VAR))
-			break;
-
 		if (!(channel = ArgList[0]))
 			{ rfc1459_odd(from, comm, ArgList); goto END; }
 
@@ -1416,9 +1392,9 @@ DISPLAY:
 		break;
 	}
 
-	case 346:		/* +I on erf */
-	case 348:		/* +e on erf */
-	case 367:		/* +b */
+	case 346:               /* #define RPL_INVITELIST (+I for erf) */
+	case 348:               /* #define RPL_EXCEPTLIST (+e for erf) */
+	case 367:
 	{
 		const char	*channel, *ban, *perp, *when_str;
 		time_t	howlong;
@@ -1464,8 +1440,7 @@ DISPLAY:
 	case 394:		/* #define RPL_ENDOFUSERS       394 */
 	{
 		PasteArgs(ArgList, 0);
-		if (get_int_var(SHOW_END_OF_MSGS_VAR))
-			display_msg(from, comm, ArgList);
+		display_msg(from, comm, ArgList);
 		break;
 	}
 
@@ -1558,36 +1533,31 @@ END:
 	 */
 	switch (numeric)
 	{
-	case 347:		/* #define END_OF_INVITELIST */
-	case 349:		/* #define END_OF_EXCEPTLIST */
+	case 347:               /* #define END_OF_INVITELIST */
+	case 349:               /* #define END_OF_EXCEPTLIST */
 	case 368:
 		number_of_bans = 0;
 		break;
 	case 464:		/* #define ERR_PASSWDMISMATCH   464 */
 	{
-		char	server_num[8];
-
 		if (oper_command)
 			oper_command = 0;
 		else if (!is_server_registered(from_server))
 		{
-			server_reconnects_to(from_server, NOSERV);
 			say("Password required for connection to server %s",
 				get_server_name(from_server));
 			if (!dumb_mode)
 			{
-				strlcpy(server_num, ltoa(from_server), sizeof server_num);
 				add_wait_prompt("Server Password:", 
-					password_sendline,
-				       server_num, WAIT_PROMPT_LINE, 0);
+					password_sendline, NUMSTR(from_server),
+					WAIT_PROMPT_LINE, 0);
 			}
 		}
 	}
 	}
 
 	current_numeric = old_current_numeric;
-	set_lastlog_msg_level(lastlog_level);
-	message_from(NULL, LOG_CRAP);
+	pop_message_from(l);
 }
 
 
