@@ -1,4 +1,4 @@
-/* $EPIC: term.c,v 1.12 2003/12/03 22:17:40 jnelson Exp $ */
+/* $EPIC: term.c,v 1.27 2013/07/28 23:16:14 jnelson Exp $ */
 /*
  * term.c -- termios and (termcap || terminfo) handlers
  *
@@ -43,10 +43,11 @@
 #include "irc.h"
 #include "ircaux.h"
 #include "vars.h"
-#include "term.h"
+#include "termx.h"
 #include "window.h"
 #include "screen.h"
 #include "output.h"
+#include "newio.h"
 
 /*
  * If "HAVE_TERMINFO" is #define'd then we will use terminfo type function
@@ -57,45 +58,95 @@
  */
 
 
-#ifdef _ALL_SOURCE
-#include <termios.h>
+#ifdef HAVE_TERMIOS_H
+# include <termios.h>
 #else
-#include <sys/termios.h>
+# ifdef HAVE_SYS_TERMIOS_H
+#  include <sys/termios.h>
+# endif
 #endif
 #include <sys/ioctl.h>
 
 volatile	int		need_redraw;
 static	int		tty_des;		/* descriptor for the tty */
 static	struct	termios	oldb, newb;
-	char		my_PC, *BC, *UP;
-	int		BClen, UPlen;
+	char		my_PC;
 
 /*
- * Systems can't seem to agree where to put these.
- * System's can't seem to agree if (char *) is spelled (unsigned char *)
- * or (const char *) or (const unsigned char *), so don't prototype these
- * either.  You have been warned!
+ * XXX -- Supporting this correctly is a hack, but it's not my fault.
+ * 
+ * Configure has already determined if <term.h> can be used to get
+ * prototypes for these functions.  It has also determined whether it is 
+ * X/OPEN curses, and therefore requires us to include <termio.h> and 
+ * <curses.h> before including <term.h>.
+ *
+ * For those systems that do not have a usable <term.h>, we will set forth
+ * some empty decl prototypes for the functions, just to establish the 
+ * return value.  All systems have either the terminfo functions or the 
+ * termcap functions, but not all systems have tparm(), so to avoid clashing
+ * with the my_tparm() decl in "compat.h", we only decl tparm() if we are
+ * using the system's (and not our) tparm().
  */
-#ifdef HAVE_TERMINFO
-extern	int		setupterm();
-extern	char		*tigetstr();
-extern	int		tigetnum();
-extern	int		tigetflag();
-#define Tgetstr(x, y) 	tigetstr(x.iname)
-#define Tgetnum(x) 	tigetnum(x.iname);
-#define Tgetflag(x) 	tigetflag(x.iname);
+#if defined(HAVE_TERM_H) && !defined(DONT_USE_TERM_H)
+/*
+# if defined(TERM_H_REQUIRES_CURSES_H)
+#  include <termio.h>
+#  include <curses.h>
+# endif
+# include <term.h>
+*/
 #else
-extern	int		tgetent();
-extern	char		*tgetstr();
-extern	int		tgetnum();
-extern	int		tgetflag();
-#define Tgetstr(x, y) 	tgetstr(x.tname, &y)
-#define Tgetnum(x) 	tgetnum(x.tname)
-#define Tgetflag(x) 	tgetflag(x.tname)
+# ifdef HAVE_TERMINFO
+	extern  int     setupterm();
+	extern  char *  tigetstr();
+	extern  int     tigetnum();
+	extern  int     tigetflag();
+# else
+	extern  int     tgetent();
+	extern  char *  tgetstr();
+	extern  int     tgetnum();
+	extern  int     tgetflag();
+# endif
+# ifdef HAVE_TPARM
+	extern  char *  tparm();
+# endif
 #endif
 
-extern	char	*getenv();
-extern	char	*tparm();
+/*
+ * We use macro wrappers to hide the difference between terminfo and termcap.
+ */
+#ifdef HAVE_TERMINFO
+# define Tgetstr(x, y)	tigetstr(x.iname)
+# define Tgetnum(x)	tigetnum(x.iname);
+# define Tgetflag(x)	tigetflag(x.iname);
+#else
+# define Tgetstr(x, y)	tgetstr(x.tname, &y)
+# define Tgetnum(x)	tgetnum(x.tname)
+# define Tgetflag(x)	tgetflag(x.tname)
+#endif
+
+/*
+ * And for the final inanity, X/OPEN curses uses
+ *     char *tparm (char *, int, int, int, int, int, int, int, int, int);
+ * but everyone else (including ncurses and solaris) uses
+ *     char *tparm (const char *, ...);
+ * and there's no way to cover those two arglists with one prototype, so
+ * we're left with these idiot wrapper functions.
+ */
+static char *tparm1 (const char *str, int l1)
+{
+	return tparm(str, l1, 0, 0, 0, 0, 0, 0, 0, 0);
+}
+
+static char *tparm2 (const char *str, int l1, int l2)
+{
+	return tparm(str, l1, l2, 0, 0, 0, 0, 0, 0, 0);
+}
+
+static char *tparm4 (const char *str, int l1, int l2, int l3, int l4)
+{
+	return tparm(str, l1, l2, l3, l4, 0, 0, 0, 0, 0);
+}
 
 /*
  * The old code assumed termcap. termcap is almost always present, but on
@@ -117,7 +168,7 @@ typedef struct cap2info
 	void *		ptr;
 } cap2info;
 
-struct	term	TI;
+struct	my_term	TI;
 
 cap2info tcaps[] =
 {
@@ -621,7 +672,7 @@ cap2info tcaps[] =
 };
 
 
-struct term *	current_term = &TI;
+struct my_term *current_term = &TI;
 static	int	numcaps = sizeof(tcaps) / sizeof(cap2info);
 static	int	term_echo_flag = 1;
 static	int	li;
@@ -645,13 +696,14 @@ int	term_echo (int flag)
 }
 
 /*
- * term_putchar: This is what handles the outputting of the input buffer.
+ * term_inputline_putchar: This is what handles the outputting of the input 
+ * buffer.
  * It used to handle more, but slowly the stuff in screen.c prepared away
  * all of the nasties that this was intended to handle.  Well anyhow, all
  * we need to worry about here is making sure nothing suspcious, like an
  * escape, makes its way to the output stream.
  */
-void	term_putchar (unsigned char c)
+void	term_inputline_putchar (unsigned char c)
 {
 	if (!term_echo_flag)
 	{
@@ -659,19 +711,10 @@ void	term_putchar (unsigned char c)
 		return;
 	}
 
-	/*
-	 * If the user has /set eight_bit_characters off, then we lop off
-	 * any 8 bit characters we might stumble across.
-	 */
-	if (!(newb.c_cflag & CS8) && (c & 0x80))
-		c &= ~0x80;
-
 	/*
 	 * Any nonprintable characters we lop off.  In addition to this,
 	 * we catch the very nasty 0x9b which is the escape character with
-	 * the high bit set.  It is possible (indeed, likely) that a terminal
-	 * with only 7 bit capabilities wont be reflected in /set eight_bit.
-	 * It is therefore our job to head that possibility off at the pass.
+	 * the high bit set.  
 	 */
 	if (c < 0x20 || c == 0x9b)
 	{
@@ -707,7 +750,7 @@ void	term_reset (void)
 	tcsetattr(tty_des, TCSADRAIN, &oldb);
 
 	if (current_term->TI_csr)
-		tputs_x(tparm(current_term->TI_csr, 0, main_screen->li - 1));
+		tputs_x(tparm2(current_term->TI_csr, 0, main_screen->li - 1));
 	term_gotoxy(0, main_screen->li - 1);
 #if use_alt_screen
 	if (current_term->TI_rmcup)
@@ -745,7 +788,7 @@ SIGNAL_HANDLER(term_cont)
 /*
  * term_pause: sets terminal back to pre-program days, then SIGSTOPs itself. 
  */
-void 	term_pause (char unused, char *not_used)
+BUILT_IN_KEYBINDING(term_pause)
 {
 	term_reset();
 	kill(getpid(), SIGSTOP);
@@ -769,7 +812,7 @@ int 	term_init (void)
 	char	*term;
 
 	if (dumb_mode)
-		panic("term_init called in dumb_mode");
+		panic(1, "term_init called in dumb_mode");
 
 	if ((term = getenv("TERM")) == (char *) 0)
 	{
@@ -824,22 +867,6 @@ int 	term_init (void)
 		}
 	}
 
-	BC = malloc_strdup(current_term->TI_cub1);
-	UP = malloc_strdup(current_term->TI_cuu1);
-	if (current_term->TI_pad)
-		my_PC = current_term->TI_pad[0];
-	else
-		my_PC = 0;
-
-	if (BC)
-		BClen = strlen(BC);
-	else
-		BClen = 0;
-	if (UP)
-		UPlen = strlen(UP);
-	else
-		UPlen = 0;
-
 	li = current_term->TI_lines;
 	co = current_term->TI_cols;
 	if (!co)
@@ -854,24 +881,28 @@ int 	term_init (void)
 
 	current_term->TI_normal[0] = 0;
 	if (current_term->TI_sgr0)
-		strlcat(current_term->TI_normal, current_term->TI_sgr0, sizeof current_term->TI_normal);
+		strlcat(current_term->TI_normal, current_term->TI_sgr0, 
+				sizeof current_term->TI_normal);
 	if (current_term->TI_rmso)
 	{
 		if (current_term->TI_sgr0 && 
 		    strcmp(current_term->TI_rmso, current_term->TI_sgr0))
-			strlcat(current_term->TI_normal, current_term->TI_rmso, sizeof current_term->TI_sgr0);
+			strlcat(current_term->TI_normal, current_term->TI_rmso,
+					 sizeof current_term->TI_normal);
 	}
 	if (current_term->TI_rmul)
 	{
 		if (current_term->TI_sgr0 && 
 		    strcmp(current_term->TI_rmul, current_term->TI_sgr0))
-			strlcat(current_term->TI_normal, current_term->TI_rmul, sizeof current_term->TI_normal);
+			strlcat(current_term->TI_normal, current_term->TI_rmul, 
+					sizeof current_term->TI_normal);
 	}
 	/*
 	 * On some systems, alternate char set mode isn't exited with sgr0
 	 */
 	if (current_term->TI_rmacs)
-		strlcat(current_term->TI_normal, current_term->TI_rmacs, sizeof current_term->TI_normal);
+		strlcat(current_term->TI_normal, current_term->TI_rmacs, 
+				sizeof current_term->TI_normal);
 
 
 	/*
@@ -972,7 +1003,7 @@ int 	term_init (void)
 
 	if ((termfeatures & desired) != desired)
 	{
-		fprintf(stderr, "\nYour terminal (%s) cannot run IRC II in full screen mode.\n", term);
+		fprintf(stderr, "\nYour terminal (%s) cannot run EPIC in full screen mode.\n", term);
 		fprintf(stderr, "The following features are missing from your TERM setting.\n");
 		if (!(termfeatures & TERM_CAN_CUP))
 			fprintf (stderr, "\tCursor movement\n");
@@ -1011,9 +1042,9 @@ int 	term_init (void)
 
 		*cbuf = 0;
 		if (current_term->TI_setaf) 
-		    strlcat(cbuf, tparm(current_term->TI_setaf, i & 0x07, 0), sizeof cbuf);
+		    strlcat(cbuf, tparm2(current_term->TI_setaf, i & 0x07, 0), sizeof cbuf);
 		else if (current_term->TI_setf)
-		    strlcat(cbuf, tparm(current_term->TI_setf, i & 0x07, 0), sizeof cbuf);
+		    strlcat(cbuf, tparm2(current_term->TI_setf, i & 0x07, 0), sizeof cbuf);
 		else
 		    snprintf(cbuf, sizeof cbuf, "\033[%dm", (i & 0x07) + 30);
 
@@ -1021,9 +1052,9 @@ int 	term_init (void)
 
 		*cbuf = 0;
 		if (current_term->TI_setab)
-		    strlcat(cbuf, tparm(current_term->TI_setab, i & 0x07, 0), sizeof cbuf);
+		    strlcat(cbuf, tparm2(current_term->TI_setab, i & 0x07, 0), sizeof cbuf);
 		else if (current_term->TI_setb)
-		    strlcat(cbuf, tparm(current_term->TI_setb, i & 0x07, 0), sizeof cbuf);
+		    strlcat(cbuf, tparm2(current_term->TI_setb, i & 0x07, 0), sizeof cbuf);
 		else
 		    snprintf(cbuf, sizeof cbuf, "\033[%dm", (i & 0x07) + 40);
 
@@ -1034,18 +1065,56 @@ int 	term_init (void)
 	tty_des = 0;			/* Has to be. */
 	tcgetattr(tty_des, &oldb);
 
+	/*
+	 * So by default, the kernel intercepts a great deal of keypresses
+	 * and does in-kernel input editing.  These keypresses are handled
+	 * by the kernel, and the result is a line of input which is then 
+	 * passed to the application.  Those special keypresses are never
+	 * sent to the application.
+	 *
+	 * So we need to turn all that off, since we need to be able to 
+	 * bind anything.
+	 */
+
+	/*
+	 * Turning off ICANON tells the kernel not to buffer input from
+	 * the user, but to pass it on to us directly.  Further, it tells
+	 * the kernel not to process the EOF (^D), EOL (^J), ERASE (^?)
+	 * and KILL (^U) keys, but to pass them on.
+	 *
+	 * Turning off ECHO tells the kernel not to output characters as
+	 * they are typed; we take responsibility for that.
+	 */
 	newb = oldb;
 	newb.c_lflag &= ~(ICANON | ECHO); /* set equ. of CBREAK, no ECHO */
-#ifdef IEXTEN
-	if (use_iexten == 1)
-		newb.c_lflag |= IEXTEN;	    /* Turn off DISCARD/LNEXT */
-	else if (use_iexten == 0)
-		newb.c_lflag &= ~(IEXTEN);  /* Turn off DISCARD/LNEXT */
-#endif
 
+	/*
+	 * Turning off IEXTEN tells the kernel not to honor any posix 
+	 * extenstions it might know about.  On 4.4BSD, this tells the 
+	 * kernel not to withhold the EOL2 (undefined), WERASE (^W), 
+	 * REPRINT (^R), LNEXT (^V), DISCARD (^O), and STATUS (^T) 
+	 * characters from us.  You wouldn't be able to /bind those keys
+	 * if this was left on.
+	 */
+	newb.c_lflag &= ~(IEXTEN);  /* Turn off DISCARD/LNEXT */
+
+	/*
+	 * Since we turned off ICANON, we have to tell the kernel how
+	 * we want the read() call to be satisifed.  We want read() to
+	 * return 0 seconds after 1 byte has been received (ie, immediately)
+	 */
 	newb.c_cc[VMIN] = 1;	          /* read() satified after 1 char */
 	newb.c_cc[VTIME] = 0;	          /* No timer */
 
+	/*
+	 * Next, we have to reclaim all of the signal-generating keys.
+	 * Normally you would do that by turning off ISIG, but we can't do
+	 * that, because we want ^C to generate SIGINT; that's how the user
+	 * can bail out of infinite recursion.
+	 *
+	 * So we have to actually turn off all of the signal-generating
+	 * keys *except* ^C by hand:  QUIT (^\), DSUSP (^Y), and SUSP (^Z).
+	 */
 #       if !defined(_POSIX_VDISABLE)
 #               if defined(HAVE_FPATHCONF)
 #                       define _POSIX_VDISABLE fpathconf(tty_des, _PC_VDISABLE)
@@ -1054,10 +1123,6 @@ int 	term_init (void)
 #               endif
 #       endif
 
-	/* 
-	 * We can't just turn off ISIG since we want ^C to send SIGINT,
-	 * so we turn off all of the other signal-generating keys
-	 */
 	newb.c_cc[VQUIT] = _POSIX_VDISABLE;
 #	ifdef VDSUSP
 		newb.c_cc[VDSUSP] = _POSIX_VDISABLE;
@@ -1066,19 +1131,55 @@ int 	term_init (void)
 		newb.c_cc[VSUSP] = _POSIX_VDISABLE;
 #	endif
 
-	if (!use_flow_control)
-		newb.c_iflag &= ~IXON;	/* No XON/XOFF */
+	/*
+	 * Now we have to turn off hardware flow control, in order to reclaim
+	 * the ^S (XOFF) and ^Q (XON) keys.  Hardware flow control is evil,
+	 * because while the flow control is off, epic will block when it
+	 * tries to output something, which could lead to you pinging out 
+	 * all your servers.
+	 */
+	newb.c_iflag &= ~(IXON | IXOFF);	/* No XON/XOFF */
 
 #if use_alt_screen
-	/* Ugh. =) This is annoying! */
+	/*
+	 * Some terminal emulators have two screens; the primary screen,
+	 * and the alternate screen.  Applications can optionally switch
+	 * to the alternate screen at startup, and then switch back to the
+	 * primary screen at shutdown.  I find this feature to be intensely
+	 * obnoxious.  It's here, if you want to #define it.
+	 */
 	if (current_term->TI_smcup)
 		tputs_x(current_term->TI_smcup);
 #endif
+
 #if use_automargins
+	/*
+	 * When you write a character to the last column of a line, most
+	 * terminal emulators will advance the cursor to the start of the
+	 * next line ("automargins") but some will leave the cursor at the end
+	 * of the line ("no automargins").  It's not always obvious which
+	 * state you're in, and it's not always obvious whether the emulator
+	 * supports one, or both modes.
+	 *
+	 * The client avoids all this by refusing to use the last column.  
+	 * But if you want to, you can define this and epic will turn off 
+	 * auto-margins and will attempt to use the final column in each line.
+	 * There are no guarantees it will work, since some emulators say they 
+	 * support turning off auto-margins, but they lie.
+	 */
 	if (current_term->TI_rmam)
 		tputs_x(current_term->TI_rmam);
 #endif
 
+	/*
+	 * Next we tell the kernel that it should support 8 bits both
+	 * coming in and going out, and it should not strip the high bit
+	 * from any keypress.
+	 */
+	newb.c_cflag |= CS8;
+	newb.c_iflag &= ~ISTRIP;
+
+	/* Commit our changes and we're done! */
 	tcsetattr(tty_des, TCSADRAIN, &newb);
 	return 0;
 }
@@ -1099,6 +1200,22 @@ int	term_resize (void)
 	{
 		struct winsize window;
 
+		/*
+		 * This hack is required by a race condition within screen;
+		 * if you have a "caption always" bar, when you reattach to
+		 * a session, it will send us a SIGWINCH, before it has 
+		 * accounted for the "caption bar" stealing a line from the
+		 * screen.  So we race screen, and if we ask for the size of
+		 * the screen before it accounts for the caption bar, then
+		 * we lose, because we'll get the wrong number of lines, and
+		 * that will screw up the status bar.  So we do this very
+		 * small sleep to increase the chances of us losing the race,
+		 * which means we win.  Got it?
+		 *
+		 * P.S. -- GNU Screen is all kinds of icky.
+		 */
+		my_sleep(0.05);
+
 		if (ioctl(tty_des, TIOCGWINSZ, &window) < 0)
 		{
 			current_term->TI_lines = li;
@@ -1157,41 +1274,13 @@ void	term_beep (void)
 	}
 }
 
-static int	orig_term_eight_bit (void)
+void	set_meta_8bit (void *stuff)
 {
-	if (dumb_mode)
-		return 1;
-	return (((oldb.c_cflag) & CSIZE) == CS8) ? 1 : 0;
-}
+	VARIABLE *v;
+	int	value;
 
-int 	term_eight_bit (void)
-{
-	if (dumb_mode)
-		return 1;
-	return (((newb.c_cflag) & CSIZE) == CS8) ? 1 : 0;
-}
-
-void	set_term_eight_bit (int value)
-{
-	if (dumb_mode)
-		return;
-
-	if (value == ON)
-	{
-		newb.c_cflag |= CS8;
-		newb.c_iflag &= ~ISTRIP;
-	}
-	else
-	{
-		newb.c_cflag &= ~CS8;
-		newb.c_iflag |= ISTRIP;
-	}
-	tcsetattr(tty_des, TCSADRAIN, &newb);
-}
-
-void	set_meta_8bit (const void *stuff)
-{
-	int	value = *(const int *)stuff;
+	v = (VARIABLE *)stuff;
+	value = v->integer;
 
 	if (dumb_mode)
 		return;
@@ -1208,11 +1297,11 @@ void	set_meta_8bit (const void *stuff)
 void	term_gotoxy (int col, int row)
 {
 	if (current_term->TI_cup)
-		tputs_x(tparm(current_term->TI_cup, row, col));
+		tputs_x(tparm2(current_term->TI_cup, row, col));
 	else
 	{
-		tputs_x(tparm(current_term->TI_hpa, col));
-		tputs_x(tparm(current_term->TI_vpa, row));
+		tputs_x(tparm1(current_term->TI_hpa, col));
+		tputs_x(tparm1(current_term->TI_vpa, row));
 	}
 }
 
@@ -1239,7 +1328,7 @@ void	term_clrscr (void)
 	/* We can also clear by deleteing lines ... */
 	else if (current_term->TI_dl)
 	{
-		tputs_x(tparm(current_term->TI_dl, current_term->TI_lines));
+		tputs_x(tparm1(current_term->TI_dl, current_term->TI_lines));
 		return;
 	}
 	/* ... in this case one line at a time */
@@ -1252,7 +1341,7 @@ void	term_clrscr (void)
 	/* As a last resort we can insert lines ... */
 	else if (current_term->TI_il)
 	{
-		tputs_x (tparm(current_term->TI_il, current_term->TI_lines));
+		tputs_x (tparm1(current_term->TI_il, current_term->TI_lines));
 		term_gotoxy (0, 0);
 		return;
 	}
@@ -1273,9 +1362,9 @@ void	term_left (int num)
 	if (num == 1 && current_term->TI_cub1)
 		tputs_x(current_term->TI_cub1);
 	else if (current_term->TI_cub)
-		tputs_x (tparm(current_term->TI_cub, num));
+		tputs_x (tparm1(current_term->TI_cub, num));
 	else if (current_term->TI_mrcup)
-		tputs_x (tparm(current_term->TI_mrcup, -num, 0));
+		tputs_x (tparm2(current_term->TI_mrcup, -num, 0));
 	else if (current_term->TI_cub1)
 		while (num--)
 			tputs_x(current_term->TI_cub1);
@@ -1292,9 +1381,9 @@ void	term_right (int num)
 	if (num == 1 && current_term->TI_cuf1)
 		tputs_x(current_term->TI_cuf1);
 	else if (current_term->TI_cuf)
-		tputs_x (tparm(current_term->TI_cuf, num));
+		tputs_x (tparm1(current_term->TI_cuf, num));
 	else if (current_term->TI_mrcup)
-		tputs_x (tparm(current_term->TI_mrcup, num, 0));
+		tputs_x (tparm2(current_term->TI_mrcup, num, 0));
 	else if (current_term->TI_cuf1)
 		while (num--)
 			tputs_x(current_term->TI_cuf1);
@@ -1310,7 +1399,7 @@ void	term_delete (int num)
 		tputs_x(current_term->TI_smdc);
 
 	if (current_term->TI_dch)
-		tputs_x (tparm (current_term->TI_dch, num));
+		tputs_x (tparm1(current_term->TI_dch, num));
 	else if (current_term->TI_dch1)
 		while (num--)
 			tputs_x (current_term->TI_dch1);
@@ -1329,9 +1418,9 @@ void	term_insert (unsigned char c)
 	else if (current_term->TI_ich1)
 		tputs_x (current_term->TI_ich1);
 	else if (current_term->TI_ich)
-		tputs_x (tparm(current_term->TI_ich, 1));
+		tputs_x (tparm1(current_term->TI_ich, 1));
 
-	term_putchar (c);
+	term_inputline_putchar (c);
 
 	if (current_term->TI_rmir)
 		tputs_x(current_term->TI_rmir);
@@ -1343,7 +1432,7 @@ void	term_insert (unsigned char c)
 void	term_repeat (unsigned char c, int rep)
 {
 	if (current_term->TI_rep)
-		tputs_x(tparm (current_term->TI_rep, (int)c, rep));
+		tputs_x(tparm2(current_term->TI_rep, (int)c, rep));
 	else
 		while (rep--)
 			putchar_x (c);
@@ -1382,8 +1471,8 @@ void	term_scroll (int top, int bot, int n)
 		 * region was the full screen.  That test *always* fails,
 		 * because we never scroll the bottom line of the screen.
 		 */
-		strlcpy(start, tparm(current_term->TI_csr, top, bot), sizeof start);
-		strlcpy(final, tparm(current_term->TI_csr, 0, current_term->TI_lines-1), sizeof final);
+		strlcpy(start, tparm2(current_term->TI_csr, top, bot), sizeof start);
+		strlcpy(final, tparm2(current_term->TI_csr, 0, current_term->TI_lines-1), sizeof final);
 
 		if (n > 0)
 		{
@@ -1392,7 +1481,7 @@ void	term_scroll (int top, int bot, int n)
 			if (current_term->TI_indn)
 			{
 				oneshot = 1;
-				strlcpy(thing, tparm(current_term->TI_indn, rn, rn), sizeof thing);
+				strlcpy(thing, tparm2(current_term->TI_indn, rn, rn), sizeof thing);
 			}
 			else
 				strlcpy(thing, current_term->TI_ind, sizeof thing);
@@ -1404,7 +1493,7 @@ void	term_scroll (int top, int bot, int n)
 			if (current_term->TI_rin)
 			{
 				oneshot = 1;
-				strlcpy(thing, tparm(current_term->TI_rin, rn, rn), sizeof thing);
+				strlcpy(thing, tparm2(current_term->TI_rin, rn, rn), sizeof thing);
 			}
 			else
 				strlcpy(thing, current_term->TI_ri, sizeof thing);
@@ -1413,8 +1502,8 @@ void	term_scroll (int top, int bot, int n)
 
 	else if (current_term->TI_wind && (current_term->TI_ri || current_term->TI_rin) && (current_term->TI_ind || current_term->TI_indn))
 	{
-		strlcpy(start, tparm(current_term->TI_wind, top, bot, 0, current_term->TI_cols-1), sizeof start);
-		strlcpy(final, tparm(current_term->TI_wind, 0, current_term->TI_lines-1, 0, current_term->TI_cols-1), sizeof final);
+		strlcpy(start, tparm4(current_term->TI_wind, top, bot, 0, current_term->TI_cols-1), sizeof start);
+		strlcpy(final, tparm4(current_term->TI_wind, 0, current_term->TI_lines-1, 0, current_term->TI_cols-1), sizeof final);
 
 		if (n > 0)
 		{
@@ -1423,7 +1512,7 @@ void	term_scroll (int top, int bot, int n)
 			if (current_term->TI_indn)
 			{
 				oneshot = 1;
-				strlcpy(thing, tparm(current_term->TI_indn, rn, rn), sizeof thing);
+				strlcpy(thing, tparm2(current_term->TI_indn, rn, rn), sizeof thing);
 			}
 			else
 				strlcpy(thing, current_term->TI_ind, sizeof thing);
@@ -1435,7 +1524,7 @@ void	term_scroll (int top, int bot, int n)
 			if (current_term->TI_rin)
 			{
 				oneshot = 1;
-				strlcpy(thing, tparm(current_term->TI_rin, rn, rn), sizeof thing);
+				strlcpy(thing, tparm2(current_term->TI_rin, rn, rn), sizeof thing);
 			}
 			else
 				strlcpy(thing, current_term->TI_ri, sizeof thing);
@@ -1452,7 +1541,7 @@ void	term_scroll (int top, int bot, int n)
 			if (current_term->TI_dl)
 			{
 				oneshot = 1;
-				strlcpy(thing, tparm(current_term->TI_dl, rn, rn), sizeof thing);
+				strlcpy(thing, tparm2(current_term->TI_dl, rn, rn), sizeof thing);
 			}
 			else
 				strlcpy(thing, current_term->TI_dl1, sizeof thing);
@@ -1460,7 +1549,7 @@ void	term_scroll (int top, int bot, int n)
 			if (current_term->TI_il)
 			{
 				oneshot = 1;
-				strlcpy(final, tparm(current_term->TI_il, rn, rn), sizeof final);
+				strlcpy(final, tparm2(current_term->TI_il, rn, rn), sizeof final);
 			}
 			else
 				strlcpy(final, current_term->TI_il1, sizeof final);
@@ -1472,7 +1561,7 @@ void	term_scroll (int top, int bot, int n)
 			if (current_term->TI_il)
 			{
 				oneshot = 1;
-				strlcpy(thing, tparm(current_term->TI_il, rn, rn), sizeof thing);
+				strlcpy(thing, tparm2(current_term->TI_il, rn, rn), sizeof thing);
 			}
 			else
 				strlcpy(thing, current_term->TI_il1, sizeof thing);
@@ -1480,7 +1569,7 @@ void	term_scroll (int top, int bot, int n)
 			if (current_term->TI_dl)
 			{
 				oneshot = 1;
-				strlcpy(final, tparm(current_term->TI_dl, rn, rn), sizeof thing);
+				strlcpy(final, tparm2(current_term->TI_dl, rn, rn), sizeof thing);
 			}
 			else
 				strlcpy(final, current_term->TI_dl1, sizeof thing);
@@ -1571,10 +1660,10 @@ const char *	term_getsgr (int opt, int fore, int back)
 			break;
 		case TERM_SGR_GCHAR:
 			if (current_term->TI_dispc)
-				ret = tparm(current_term->TI_dispc, fore);
+				ret = tparm1(current_term->TI_dispc, fore);
 			break;
 		default:
-			panic ("Unknown option '%d' to term_getsgr", opt);
+			panic(1, "Unknown option '%d' to term_getsgr", opt);
 			break;
 	}
 	return (ret);
@@ -1582,7 +1671,7 @@ const char *	term_getsgr (int opt, int fore, int back)
 
 static char *	control_mangle (unsigned char *text)
 {
-static 	u_char	retval[256];
+static 	unsigned char	retval[256];
 	int 	pos = 0;
 	
 	*retval = 0;
@@ -1649,3 +1738,123 @@ static	char		retval[128];
 	return NULL;	
 }
 
+#ifdef WITH_THREADED_STDOUT
+
+#include <pthread.h>
+#include "tio.h"
+
+#define TIOBUFSZ	2048
+
+typedef struct tio_qentry_stru {
+	tio_file	*file;
+	int		 len, flush, close;
+struct	tio_qentry_stru	*next, *tail;
+	char 		 buf[TIOBUFSZ];
+} tio_qentry;
+
+struct tio_file_stru {
+	FILE		*stdio_file;
+	tio_qentry	*buf;
+};
+
+static void		tio_push(tio_file *);
+static tio_qentry	*new_tiobuf(tio_file *);
+
+tio_file	*tio_stdout;
+
+static	tio_qentry	tio_qhead;
+static	pthread_cond_t	tio_cond = PTHREAD_COND_INITIALIZER;
+static	pthread_mutex_t	tio_mtx = PTHREAD_MUTEX_INITIALIZER;
+static	pthread_t	tio_thread;
+
+static tio_qentry	*new_tiobuf(tio_file *f)
+{
+	tio_qentry	*ret;
+	ret = calloc(1, sizeof(*ret));
+	ret->file = f;
+	return ret;
+}
+
+void	tio_fputc(int c, tio_file *f)
+{
+	if (f->buf->len == TIOBUFSZ)
+		tio_push(f);
+	f->buf->buf[f->buf->len++] = c;
+}
+
+void	tio_fputs(const char *str, tio_file *stream)
+{
+	while (*str)
+		tio_fputc(*str++, stream);
+}
+
+tio_file	*tio_open(FILE *f)
+{
+	tio_file	*ret;
+
+	if (f == NULL)
+		return NULL;
+
+	ret = calloc(1, sizeof(*ret));
+	ret->stdio_file = f;
+	ret->buf = new_tiobuf(ret);
+	return ret;
+}
+
+void	tio_flush(tio_file *stream)
+{
+	stream->buf->flush = 1;
+	tio_push(stream);
+}
+
+static void	tio_push(tio_file *stream)
+{
+	pthread_mutex_lock(&tio_mtx);
+	if (!tio_qhead.next) {
+		tio_qhead.next = tio_qhead.tail = stream->buf;
+	} else {
+		tio_qhead.tail->next = stream->buf;
+		tio_qhead.tail = stream->buf;
+	}
+
+	pthread_cond_signal(&tio_cond);
+	pthread_mutex_unlock(&tio_mtx);
+
+	stream->buf = new_tiobuf(stream);
+}
+
+static void	*tio_thread_run(void *unused)
+{
+	for (;;) {
+		tio_qentry	*tq, *next;
+
+		pthread_mutex_lock(&tio_mtx);
+		if (tio_qhead.next == NULL)
+			pthread_cond_wait(&tio_cond, &tio_mtx);
+		tq = tio_qhead.next;
+		tio_qhead.next = tio_qhead.tail = NULL;
+		pthread_mutex_unlock(&tio_mtx);
+
+		for (next = tq ? tq->next : NULL; tq;
+		     tq = next, next = tq ? tq->next : NULL) {
+			if (tq->len)
+				fwrite(tq->buf, 1, tq->len, tq->file->stdio_file);
+			if (tq->flush)
+				fflush(tq->file->stdio_file);
+			free(tq);
+		}
+	}
+}
+
+void	tio_init(void)
+{
+	tio_stdout = tio_open(stdout);
+	pthread_create(&tio_thread, NULL, tio_thread_run, NULL);
+}
+
+void	tio_close (tio_file *stream)
+{
+	yell("Need to implement tio_close!");
+}
+
+#endif
