@@ -1,4 +1,4 @@
-/* $EPIC: vars.c,v 1.110 2012/07/06 04:52:26 jnelson Exp $ */
+/* $EPIC: vars.c,v 1.112 2012/11/26 05:09:35 jnelson Exp $ */
 /*
  * vars.c: All the dealing of the irc variables are handled here. 
  *
@@ -301,7 +301,8 @@ void 	init_variables_stage1 (void)
 #define DEFAULT_DEFAULT_REALNAME NULL
 	VAR(DEFAULT_REALNAME, 		STR,  NULL);
 #define DEFAULT_DEFAULT_USERNAME NULL
-	VAR(DEFAULT_USERNAME, 		STR, NULL);
+	VAR(DEFAULT_SCRIPT_ENCODING,	STR,  NULL);
+	VAR(DEFAULT_USERNAME, 		STR,  NULL);
 	VAR(DISPATCH_UNKNOWN_COMMANDS,	BOOL, NULL);
 	VAR(DISPLAY, 			BOOL, NULL);
 	VAR(DO_NOTIFY_IMMEDIATELY, 	BOOL, NULL);
@@ -544,7 +545,6 @@ void 	set_var_value (int svv_index, const char *value, int noisy)
 int 	set_variable (const char *name, IrcVariable *var, const char *orig_value, int noisy)
 {
 	char	*rest;
-	int	old;
 	int	changed = 0;
 	char	*value;
 	int	retval = 0;
@@ -560,7 +560,6 @@ int 	set_variable (const char *name, IrcVariable *var, const char *orig_value, i
 	    {
 		if (value && *value && (value = next_arg(value, &rest)))
 		{
-			old = var->data->integer;
 			if (do_boolean(value, &(var->data->integer))) {
 			    say("Value must be either ON, OFF, or TOGGLE");
 			    retval = -1;
