@@ -1,4 +1,4 @@
-/* $EPIC: words.c,v 1.19 2003/12/09 04:37:52 jnelson Exp $ */
+/* $EPIC: words.c,v 1.24 2012/11/23 16:04:49 jnelson Exp $ */
 /*
  * words.c -- right now it just holds the stuff i wrote to replace
  * that beastie arg_number().  Eventually, i may move all of the
@@ -42,24 +42,34 @@
 /* 
  * If "extended" is 1, then honor /xdebug extractw.
  * If "extended" is other than 1 but not zero, imply /xdebug extractw.
+ *
+ * This should turn to YES if DWORD_YES and DEBUG_DWORD ...
  */
 #define CHECK_EXTENDED_SUPPORT					\
-	if (extended == DWORD_YES && 				\
-			((x_debug & DEBUG_EXTRACTW) == 0))	\
-		extended = DWORD_NEVER;				\
-        else if (extended == DWORD_ALWAYS)			\
-		extended = DWORD_YES;				\
+	if (extended == DWORD_EXTRACTW)				\
+	{							\
+		if ((x_debug & DEBUG_EXTRACTW) == 0)		\
+			extended = DWORD_NO;			\
+		else						\
+			extended = DWORD_YES;			\
+	}							\
+	else if (extended == DWORD_DWORDS)			\
+	{							\
+		if ((x_debug & DEBUG_DWORD) == 0)		\
+			extended = DWORD_NO;			\
+		else						\
+			extended = DWORD_YES;			\
+	}
 
-
-
-#define risspace(c) (c == ' ')
- 
 /*
  * search() looks for a character forward or backward from mark 
  */
 char *	search_for (char *start, char **mark, char *chars, int how)
 {
-        if (!mark || !*mark)
+	if (!mark)
+		return NULL;		/* Take THAT! */
+
+        if (!*mark)
                 *mark = start;
 
         if (how > 0)   /* forward search */
@@ -115,7 +125,7 @@ static const char *	find_forward_character (const char *input, const char *start
 	 * Make sure that the character before is the start of the
 	 * string or that it is a space.
 	 */
-	if (input > start && !risspace(input[-1]))
+	if (input > start && !isspace(input[-1]))
 		return NULL;
 
 	/*
@@ -134,7 +144,7 @@ static const char *	find_forward_character (const char *input, const char *start
 		else if ( (simple == 1 && input[0] == what) ||
 			  (simple == 0 && strchr(whats, *input)) )
 		{
-			if (input[1] == 0 || risspace(input[1]))
+			if (input[1] == 0 || isspace(input[1]))
 				return input;
 		}
 	}
@@ -186,7 +196,7 @@ static const char *	find_backward_quote (const char *input, const char *start, c
 	 * Make sure that the character after is either the end of the
 	 * string, or that it is a space.
 	 */
-	if (input[1] && !risspace(input[1]))
+	if (input[1] && !isspace(input[1]))
 		return NULL;
 
 	/*
@@ -210,7 +220,7 @@ static const char *	find_backward_quote (const char *input, const char *start, c
 		else if ( (simple == 1 && input[0] == what) ||
 			  (simple == 0 && strchr(whats, *input)) )
 		{
-			if (input == start || risspace(input[-1]))
+			if (input == start || isspace(input[-1]))
 				return input;
 		}
 		else if (input == start)
@@ -224,10 +234,10 @@ static const char *	find_backward_quote (const char *input, const char *start, c
 	 * for a normal word.
 	 */
 	input = saved_input;
-	while (input > start && !risspace(input[0]))
+	while (input > start && !isspace(input[0]))
 		input--;
 
-	if (risspace(input[0]))
+	if (isspace(input[0]))
 		input++;		/* Just in case we've gone too far */
 
 	return input;		/* Wherever we are is fine. */
@@ -335,7 +345,7 @@ static int	move_to_prev_word (const char **str, const char *start, int extended,
 			yell(".... move_to_prev_word: Handling extended word.");
 
 		if (!(before = find_backward_quote(pos, start, delims)))
-			panic("find_backward returned NULL [2]");
+			panic(1, "find_backward returned NULL [2]");
 
 		if (x_debug & DEBUG_EXTRACTW_DEBUG)
 			yell(".... move_to_prev_word: Extended word begins at [%s] (of [%s])", before, start);
@@ -450,9 +460,6 @@ static int	move_to_next_word (const char **str, const char *start, int extended,
 {
 	char	what;
 	int	simple;
-#if 0
-	const char *	mark2;
-#endif
 	const char *	pos;
 
 	/*
@@ -529,7 +536,7 @@ static int	move_to_next_word (const char **str, const char *start, int extended,
 		    yell(".... move_to_next_word: handling extended word");
 
 		if (!(after = find_forward_character(pos, start, delims)))
-			panic("find_forward returned NULL [1]");
+			panic(1, "find_forward returned NULL [1]");
 		if (*after)
 			after++;
 		pos = after;
@@ -552,38 +559,8 @@ static int	move_to_next_word (const char **str, const char *start, int extended,
 		    yell(".... move_to_next_word: at next space: [%s]", pos);
 	}
 
-#if 0
-	/*
-	 * 'pos' now points at the space just after the end of the word;
-	 * This space belongs to us, but any further spaces do not long
-	 * belong to us, unless there are no more words after us!  So we
-	 * need to check this.
-	 */
-	/* -- Is the rest of a string just spaces?  -- */
-	if (x_debug & DEBUG_EXTRACTW_DEBUG)
-	    yell("... move_to_next_word: looking for another word...");
-
-	mark2 = pos;
-	while (*mark2 && my_isspace(*mark2))
-		mark2++;
-
-	if (!*mark2)		/* Only spaces after this.  Ok. */
-	{
-		if (x_debug & DEBUG_EXTRACTW_DEBUG)
-		    yell("... move_to_next_word: didn't find one.");
-		pos = mark2;
-	}
-	/* The start of the next word is after our space. */
-	else
-	{
-		if (x_debug & DEBUG_EXTRACTW_DEBUG)
-		    yell("... move_to_next_word: there's another word.");
-		pos++;
-	}
-#else
 	if (*pos)
 		pos++;
-#endif
 
 	if (x_debug & DEBUG_EXTRACTW_DEBUG)
 	    yell("... move_to_next_word: next word starts with [%s]", pos);
